;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Effect Name : Expander gain with hard knee
;
; Description : Calculates the gain to apply to expand the incoming signal
;
; Nb In       : 1
; Nb Out      : 1
;
; Input       : audio signal in ABSOLUTE VALUE!!!
; Output      : expander gain
;
; Cycles      : 20 + 19 cycles (-1=unknownw)
; Memory      : 7 words (without init)
; Delay Line  : no
;
; Author      : Gregor
; Date        : 27/04/2004
; Version     : 1.0
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

ABS     INIT    0x3FF

        ;; SEC1 Control parameters

ABS	TShold	0x0		; Compression threshold
ABS	Ratio	0x1		; Compression factor (given by (Ratio - 1) / 8)
ABS	Ta	    0x2		; Attack time
ABS	Tr	    0x3		; Release time
ABS Th		0x4		; Hold time

        ;; SEC2 Initial values of control parameters (S3.24 format)

ITShold	EQU -0.75			; {0,1,log}
IRatio	EQU 4.0				; {0,1,log}
ITa		EQU 1.0				; {0,1,log}
ITr		EQU 1.0				; {0,1,log}
ITh		EQU 0x00			;

        ;; SEC3 Constants definitions
; none

        ;; SEC4 Memory variables definition
ABS	Gc	    0x5		; Current gain
ABS Thc		0x6		; Current hold time

        ;; SEC5 Initial values of memory variables
IGc		EQU 1.0				; {0,1,log}
IThc	EQU 0x00			;

        ;; SEC6 Initialization
CM      0x40000 INIT
SKIP    !Z      EndInit
CM      0.0     INIT
1AC     ITShold			    ; 
SCA	    0x0     TShold		; init Tshold
1AC     IRatio			    ; 
SCA	    0x0     Ratio		; init Ratio
1AC     ITa			        ; 
SCA	    0x0     Ta			; init Ta
1AC     ITr			        ; 
SCA	    0x0     Tr			; init Tr
1AC     ITh			        ; 
SCA	    0x0     Th			; init Th
1AC     IGc		            ; 
SCA	    0x0     Gc			; init Gc
1AC     IThc			        ; 
SCA	    0x0     Thc			; init Thc
DAC     0x0     0x40000     ; 1->A
SCA     0x0     INIT        ; do not reinitialize next time
EndInit:
	
        ;; SEC7 Module code

LCM	    0.0     IN1
LOGB
CAM	    -1.0    TShold
SKIP	!N      Low

High:
CM		1.0		Th			;
SCA		0.0		Thc			; reinit Thc with Th, 0 -> A
XCM		1.0		Ta          ; temp -> B, Tattack -> A
SKIP	End
	
Low:
AMC		0.0 	Ratio
CAD		-8.0	0.0

TestHoldTime:
XCM		1.0		Thc			; A -> B, A contains Thc
SKIP	Z		HoldDone	; if hold time != 0, then avoid reset of gain
1AC		0xFFFFFFF			; Thc--
SCA		0.0		Thc			; save new Thc, 0 -> A
HoldDone:

CM		1.0 	Tr          ; temp -> B, Trelease -> A

End:	
SEXPB	Dir0			    ; exp(Temp)->A, t*c->Dir0
	

	;; first order filter
LCMA	-1.0    Gc
AMB	    Dir0			    ; new Gc value in A
SCA	    1.0		Gc		    ; ... and in Gc
SCA     0.0		OUT1	    ; store