;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Effect Name : Mixer 5-to-1
;
; Description : Mixing five mono inputs
;
; Nb In       :	5
; Nb Out      : 1
;
; Input       : 5 times 1 channel signal
; Output      : mixed signal
;
; Cycles      : 11 + 14 cycles (-1=unknownw)
; Memory      : 5 words (without init)
; Delay Line  :	no
;
; Author      : Gregor
; Date        : 02/03/2003
; Version     : 1.0
;
; Control parameters (see SEC1): 
;	Four scalers give control over how the mixing is made.
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

	;; SEC0 General Parameters
ABS	INIT		0x3ff		; init params at first run

	;; SEC1 Control Parameters
ABS	Scale1		0x000		; Mixing level for the 1st channel
ABS	Scale2		0x001		; Mixing level for the 2nd channel
ABS	Scale3		0x002		; Mixing level for the 3rd channel
ABS	Scale4		0x003		; Mixing level for the 4th channel
ABS	Scale5		0x004		; Mixing level for the 5th channel

	;; SEC2 Initial values of control parameters (S3.24 format)
IScale1	EQU		0.2			; Half of the 1st channel will be added to the output
IScale2	EQU		0.2			; Half of the 2nd channel will be added to the output
IScale3	EQU		0.2			; Half of the 3rd channel will be added to the output
IScale4	EQU		0.2			; Half of the 4th channel will be added to the output
IScale5	EQU		0.2			; Half of the 5th channel will be added to the output

	;; SEC3 Constants definitions
;none

	;; SEC4 Memory variables definition
;none

	;; SEC5 Initial values of memory variables
;none

	;; SEC6 Initialization : (should restore register to 0x0)
CM	0x40000 	INIT
SKIP	!Z		EndInit
C		IScale1				; load IScale1
SCA 	0x0  	Scale1		; save to Scale1
C		IScale2				; load IScale2
SCA		0x0		Scale2		; save to Scale2
C		IScale3				; load IScale3
SCA		0x0		Scale3		; save to Scale3
C		IScale4				; load IScale4
SCA		0x0		Scale4		; save to Scale4
C		IScale5				; load IScale5
SCA		0x0		Scale5		; save to Scale5
C       0x1000000     ; 1->A
SCA     0x0     INIT        ; do not reinitialize next time
EndInit:

	;; SEC7 Main
;;; Mixing the inputs
CM		1.0		Scale1	; A = Scale1
AMC		0.0		IN1		; A = Scale1 * input1
XCM		1.0		Scale2	; A = Scale2			B = Scale1 * input1
AMB				IN2		; A = Scale2 * input2 + Scale1 * input1
XCM		1.0		Scale3	; A = Scale3			B = Scale2 * input2 + Scale1 * input1
AMB				IN3		; A = Scale3 * input3 + Scale2 * input2 + Scale1 * input1
XCM		1.0		Scale4	; A = Scale4			B = Scale3 * input3 + Scale2 * input2 + Scale1 * input1
AMB				IN4		; A = Scale4 * input4 + Scale3 * input3 + Scale2 * input2 + Scale1 * input1
XCM		1.0		Scale5	; A = Scale4			B = Scale3 * input3 + Scale2 * input2 + Scale1 * input1
AMB				IN5		; A = Scale5 * input5 + Scale4 * input4 + Scale3 * input3 + Scale2 * input2 + Scale1 * input1
SCA		0.0		OUT1	; output = A

