;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Effect Name : VariableDelay
;
; Description : VariableDelay line
;   We implement a Delay line of maximum MaxDelay samples
;   We return the value at a time t-dt with 0<dt<MaxDelay
;   The dt param is given as an input between 0 and 1
;
; Nb In       : 2
; Nb Out      : 1
;
; Input       : single signal in channel 1, delay fraction in IN2
; Output      : delayed signal
;
; Cycles      : 51 + 8 cycles (-1=unknownw)
; Memory      : 3 words (without init)
; Delay Line  : yes
;
; Author      : Gregor
; Date        : 11/10/2004
; Version     : 1.0
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

ABS     INIT    0x3FF

        ;; SEC1 Control parameters
ABS	MaxDelayLength	0x0	; number of samples in delay line
				;  (48 per ms of delay)

        ;; SEC2 Initial values of control parameters (S3.24 format)
IMaxDelayLength	EQU 0x1d0	; corresponds to 10 ms

        ;; SEC3 Constants definitions
	
        ;; SEC4 Memory variables definition
ABS	ActualValue	0x1
ABS	DelayLine	0x2
	
        ;; SEC5 Initial values of memory variables
IActualValue	EQU	0x0

        ;; SEC6 Initialization
CM      0x40000 INIT
SKIP    !Z      EndInit
C	IMaxDelayLength		; initialize delay time
SCA	0x0 MaxDelayLength
C       IActualValue
SCA     0x0 ActualValue
C       0x1000000               ; 1->A
SCA     0x0 INIT                ; do not reinitialize next time
EndInit:

        ;; SEC7 Module code
	
	;; store the incoming value and update pointer position
LCM	0x0 ActualValue		; get place in the delay line
CM	0x40000 IN1		; Read IN1
SINDF	0x3ff @DelayLine	; stores it in address pointed by B
CAB	0x0			; B -> A
SKIP	!Z EndReinit		; If A!=0 skip next instruction
1MC	0x0 MaxDelayLength	; A=DELAY  (ensures that 0 becomes DELAY-1!)
EndReinit:
1AC	0xfffffff		; A--
SXCA	0x0 ActualValue		; Update delay line pointer
	
	;; retreive out val 1
		                ; NOTE:	in A*B operations below, we expect
				; that internally, A=S3.18 and B=S3.24
CM	0x40000 MaxDelayLength
1AC	0xfffffff
CAD	0x100000 0x00		; << 2
CAD	0x100000 0x00		; << 4
CAD	0x100000 0x00		; << 6
CAD	0x100000 0x00		; << 8
XCM	0x40000 IN2		; load the delay factor 
1AC	0xf000000		; (-1.0)
BAC	0x0			; (IN-1.0)*(Max-1) (SHIFTED)
SCA	0x400 Dir1		; \
SCA	0x40000 Dir0		; | save the mean position
CAD	0x100000 0x00		; | << 2
CAD	0x100000 0x00		; | << 4
CAD	0x100000 0x00		; | << 6
CAD	0x100000 0x00		; | << 8
CMA	0x3c0000 Dir1		; | compute the non-integer
CAD	0x100000 0x00		; | << 10
CAD	0x100000 0x00		; | << 12
CAD	0x100000 0x00		; | << 14
CAD	0x100000 0x00		; | << 16
CAD	0x100000 0x00		; | << 18
CAD	0x100000 0x00		; | << 20
CAD	0x100000 0x00		; | << 22
CAD	0x100000 0x00		; / << 24
SCA	0x0 Dir1		; Save NIP in Dir1
CM	0x40000 Dir0		; retreive (unshifted) position
CMA	0x40000 ActualValue	; return pointer up to a MaxDelay Shift
SKIP	!N EndAddLoop		; test if we need to recenter value
CMA	0x40000 MaxDelayLength
EndAddLoop:
X1AC	0x0			; store that pointer in B
LINDF	0x3ff @DelayLine	; Retreive the first interp value
SCB	0x40000 Dir0		; Writes to Dir0 and retreive pointer
CMA	0x3c0000 MaxDelayLength
1AC	0x1			; A++
SKIP	!N EndGoAtEnd
CMA	0x40000 MaxDelayLength
EndGoAtEnd:	
X1AC	0x0			; store that pointer in B
LINDF	0x3ff @DelayLine	; Retreive the second interp value
SCA	0x40000 Dir2
CAM	0x3c0000 Dir0
AMC	0x0 Dir1
CMA	0x40000 Dir2
SCA	0x0 OUT1
