;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Effect Name : Distorsion
;
; Description : Distorsion with simple damping above threshold
;
; Nb In       :	1
; Nb Out      : 1
;
; Input       : single signal
; Output      : distorded signal
;
; Cycles      : 9 + 8 cycles (-1=unknownw)
; Memory      : 2 words (without init)
; Delay Line  :	no
;
; Author      : Gregor
; Date        : 24/10/2002
; Version     : 1.1
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

ABS	INIT	0x3FF
	
	;; SEC1 Control parameters
ABS	TShold	0x0		; threshold for distorsion
ABS	Coef	0x1		; damping above threshold

	;; SEC2 Initial values of control parameters (S3.24 format)
ITShold	EQU	0x0040000	; intial value of 0.06125
ICoef	EQU	0x0100000	; complete damping

	;; SEC3 Constants definitions
; none

	;; SEC4 Memory variables definition
; none
 
	;; SEC5 Initial values of memory variables
; none

	;; SEC6 Initialization
CM	0x40000	INIT
SKIP	!Z	EndInit
C	ITShold			; load init value for threshold
SCA	0x0 TShold		; ... Stored in DeltaC
C	ICoef			; load init value for damping.
SCA	0x0 Coef		; ... Stored in DeltaS
C	0x1000000		; 1->A
SCA	0x0 INIT		; do not reinitialize next time
EndInit:

	;; SEC7 Module code
CM	0x40000 IN1		; copy IN1 to A
	;; deal with negative values
CMA	0x40000 TShold		; add Threashold to A
SKIP	!N 1			; if A>0 skip next instruction
AMC	0x0 Coef		; Rescale value
	;; deal with positive values
CMA	0x380000 TShold		; remove 2*Threashold from A ...
SKIP	N 1			; if A<0 skip next instruction
AMC	0x0 Coef		; Rescale value
CMA	0x40000 TShold		; re-add TShold
SCA	0x40000 OUT1		; copy A to OUT1
