;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Effect Name : Simple all-pass filter
;
; Description : All-pass filters the signal given a certain coefficient
;
; Nb In       :	2
; Nb Out      : 1
;
; Input       : 1 channel signal and 1 coefficient signal
; Output      : 1 channel signal
;
; Cycles      : 7 + 5 cycles (-1=unknownw)
; Memory      : 1 words (without init)
; Delay Line  :	no
;
; Author      : Lall
; Date        : 02/03/2003
; Version     : 1.0
;
; Control parameters (see SEC1): "none"
;	The filter coefficient is given via the second audio input of the filter
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

	;; SEC0 General Parameters
ABS	INIT		0x3ff		; init params at first run

	;; SEC1 Control Parameters
;none

	;; SEC2 Initial values of control parameters (S3.24 format)
;none

	;; SEC3 Constants definitions
;none

	;; SEC4 Memory variables definition
ABS	zm1			0x000		; State variable of the filter

	;; SEC5 Initial values of memory variables
;none

	;; SEC6 Initialization : (should restore register to 0x0)
CM	0x40000 	INIT
SKIP	!Z		EndInit
SCA 	0x0 		zm1		; initialize state to 0
C       0x1000000           ; 1->A
SCA     0x0     INIT        ; do not reinitialize next time
EndInit:

	;; SEC7 Main
;;; Applying the all-pass
CM	-1.0		IN2		; A = -coeff
AMC	0.0		IN1		; A = InputValue * -coeff
CAM	1.0		zm1		; A = InputValue * -coeff + zm1
SCA	1.0		OUT1		; TempValue = y = A, don't touch A it will be re-used

AMC	0.0		IN2		; A = y * coeff
CAM	1.0		IN1		; A = y * coeff + InputSample
SCA	1.0		zm1		; zm11 = A


