;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Effect Name : iir filter
;
; Description : second order IIR filter
;
; Nb In       : 1
; Nb Out      : 1
;
; Input       : mono signal
; Output      : filtered signal
;
; Cycles      : 15 + 18 cycles (-1=unknownw)
; Memory      : 9 words (without init)
; Delay Line  : no
;
; Author      : 
; Date        : 
; Version     : 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

ABS	INIT	0x3FF

	;; the formula applied is
	;;    y_i = a0 x_i + a1 x_{i-1} + a2 x_{i-2} + b1 y_{i-1} + b2 y_{i-2}
	
	;; SEC1 Control parameters
ABS	CIn0	0x0		; a0
ABS	CIn1	0x1		; a1
ABS	CIn2	0x2		; a2
ABS	COut1	0x3		; b1
ABS	COut2	0x4		; b2

	;; SEC2 Initial values of control parameters (S3.24 format)
ICIn0	EQU	0.001532981
ICIn1	EQU	0.003065962
ICIn2	EQU	0.001532981
ICOut1	EQU	-1.983030
ICOut2	EQU	0.9891624

	;; SEC3 Constants definitions

	;; SEC4 Memory variables definition
ABS	X1	0x5		; x_{i-1}
ABS	X2	0x6		; x_{i-2}
ABS	Y1	0x7		; y_{i-1}
ABS	Y2	0x8		; y_{i-2}
	
	;; SEC5 Initial values of memory variables
	;; all values in sec 4 will be init to 0 !
	
	;; SEC6 Initialization
CM	0x40000	INIT
SKIP	!Z	EndInit
C	ICIn0
SCA	0x0 CIn0
C	ICIn1
SCA	0x0 CIn1
C	ICIn2
SCA	0x0 CIn2
C	ICOut1
SCA	0x0 COut1
C	ICOut2
SCA	0x0 COut2
SCA	0x0 X1
SCA	0x0 X2
SCA	0x0 Y1
SCA	0x0 Y2
C       0x1000000		; 1->A
SCA     0x0     INIT		; do not reinitialize next time
EndInit:

	;; SEC7 Module code
CM	0x40000 X2		; x(i-2) -> A
AMC	0x0 CIn2		; a2 x(i-2) -> A
XCM	0x40000 X1		; A -> B and x(i-1) -> A
SCA	0x40000 X2		; A unchanged, X1 will be X2 at next run
AMB	CIn1			; a1 x(i-1) + a2 x(i-2) -> A
XCM	0x40000 IN1		; A -> B and x(i) -> A
SCA	0x40000 X1		; A unchanged, IN will be X1 at next run
AMB	CIn0			; a0 x(i)+a1 x(i-1)+a2 x(i-2) -> A
XCM	0x40000 Y2		; A -> B and y(i-2) -> A
AMB	COut2			; b2 y(i-2)+a0 x(i)+a1 x(i-1)+a2 x(i-2) -> A
XCM	0x40000 Y1		; A -> B and x(i-1) -> A
SCA	0x40000 Y2		; A unchanged, Y1 will be Y2 at next run
AMB	COut1			; A not contins full output !
SCA	0x40000 Y1		; A unchanged, OUT will be Y1 at next run
SCA	0x0 OUT1		; write output !
