;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Effect Name : saw triangle
;
; Description : Triangle wave generator: done like a saw
;
; Nb In       : 0
; Nb Out      : 1
;
; Input       : nothing
; Output      : 1 channel signal, between -0.5 and 0.5
;
; Cycles      : 8 + 10 cycles (-1=unknownw)
; Memory      : 3 words (without init)
; Delay Line  : no
;
; Author      : Gregor
; Date        : 24/02/2003
; Version     : 1.0
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

ABS	INIT	0x3FF
	
	;; SEC1 Control parameters
ABS	Ampl	0x0		; amplitude
ABS	Delta	0x1		; Delta = 4/(smpfreq * T)
	
	;; SEC2 Initial values of control parameters (S3.24 format)
IAmpl	EQU	0x0c00000
IDelta	EQU	0x00a377f	; ex:	 0x00005f2 for 1 Hz
				;        0x00a377f for 440 Hz
	
	;; SEC3 Constants definitions

	;; SEC4 Memory variables definition
ABS	Value	0x2
	
	;; SEC5 Initial values of memory variables
IValue	EQU	0x0
	
	;; SEC6 Initialization
CM	0x40000	INIT
SKIP	!Z	EndInit
C	IValue			; load value
SCA	0x0 Value
C	IAMpl			; load value
SCA	0x0 Ampl
C	IDelta			; load value
SCA	0x0 Delta
C       0x1000000     ; 1->A
SCA     0x0     INIT        ; do not reinitialize next time
EndInit:

	;; SEC7 Module code
CM	0x40000 Value		; Value -> A
CAM	0x40000 Delta		; Value+Delta -> A
1AC	0xe000000		; 'new'-Value - 1 -> A
SKIP	N continue
1AC	0xc000000		; 'new'-Value - 1 -> A
continue:
1AC	0x2000000		; newValue -> A
SCA	0x10000 Value		; save for next run
SCA	0x0 OUT1		; write output
