;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Effect Name : triangle
;
; Description : Triangle wave generator
;
; Nb In       : 0
; Nb Out      : 1
;
; Input       : nothing
; Output      : 1 channel signal
;
; Cycles      : 12 + 12 cycles (-1=unknownw)
; Memory      : 4 words (without init)
; Delay Line  : no
;
; Author      : Gregor
; Date        : 24/02/2003
; Version     : 1.0
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

ABS	INIT	0x3FF
	
	;; SEC1 Control parameters
ABS	Ampl	0x0		; amplitude
ABS	Delta	0x1		; Delta = 4/(smpfreq * T)
	
	;; SEC2 Initial values of control parameters (S3.24 format)
IAmpl	EQU	0x0c00000
IDelta	EQU	0x00a377f	; ex:	 0x00005f2 for 1 Hz
				;        0x00a377f for 440 Hz
	
	;; SEC3 Constants definitions

	;; SEC4 Memory variables definition
ABS	Up	0x2
ABS	Value	0x3
	
	;; SEC5 Initial values of memory variables
IUp	EQU	0x1000000
IValue	EQU	0x0
	
	;; SEC6 Initialization
CM	0x40000	INIT
SKIP	!Z	EndInit
C	IValue			; load value
SCA	0x0 Value
C	IUp			; load value
SCA	0x0 Up
C	IAMpl			; load value
SCA	0x0 Ampl
C	IDelta			; load value
SCA	0x0 Delta
C       0x1000000     ; 1->A
SCA     0x0     INIT        ; do not reinitialize next time
EndInit:

	;; SEC7 Module code
CM	0x40000 Value		; Value -> A
XCM	0x40000 Delta		; Value -> B, Delta -> A
AMB	Up			; Value +/- Delta -> A
AMC	0x3c0000 Up		; |'new'-Value| - 1 -> A
SKIP	N+Z Output
	;; we have  |overflow| in A
	;; we must 1) invert up
	;;         2) put |overflow|-2 in A
XCM	0x3c0000 Up		; save |overflow| in B, load -up
S1BC	0x380000 Up		; Save new up value, |overflow|-2 -> A 
Output:
DAC	0x100 0x40000		; A+1
AMC	0x0 Up		        ; New A calculated
SCA	0x40000 Value		; update value
AMC	0x0 Ampl		; rescale
SCA	0x0 OUT1		; write output
