////////////////////////////////////////////////////////////////
// Name:        Config.cpp                                    //
// Project:     GUI for Axoris AL3101 application generator   //
// Author:      Gregor(y) Soyez                               //
// Version:     1.0                                           //
// Created:     18/05/02                                      //
// Copyright:   (c) Axoris                                    //
////////////////////////////////////////////////////////////////

#include <stdio.h>
#include "Config.h"

#define CONFIG_OK           0
#define CONFIG_CHECK_ERROR  1
#define CONFIG_LOAD_ERROR   2
#define CONFIG_SAVE_ERROR   3
#define CONFIG_CREATE_ERROR 4

#define CFG_APP_NAME  "AgALag"
#define CFG_VENDOR_NAME "Axoris Software"
#define CONFIG_FILE_VERSION_NUMBER 1

#define DEFAULT_WINDOW_WIDTH   500
#define DEFAULT_WINDOW_HEIGHT  400
#define DEFAULT_STRUCT_WIDTH   400
#define DEFAULT_STRUCT_HEIGHT  300

#define MINIMAL_WINDOW_WIDTH   80
#define MINIMAL_WINDOW_HEIGHT  60
#define MINIMAL_STRUCT_WIDTH   50
#define MINIMAL_STRUCT_HEIGHT  20

CConfig::CConfig(){
#ifdef DEBUG
  fprintf(stderr, "DBG: CConfig: Entering constructor\n");
#endif

  EffectsDir=NULL;
  AssemblerDir=NULL;
  ProjectName=NULL;

  if (Check()==CONFIG_CHECK_ERROR){
    // create the new configuration
    if (Create()==CONFIG_CREATE_ERROR)
      Exists=0;
    else
      Exists=1;
    return;
  }

  Load();
  Exists=1;
}

CConfig::~CConfig(){
#ifdef DEBUG
  fprintf(stderr, "DBG: CConfig: Entering destructor\n");
#endif

  if (Exists==1){
    Save();
    delete config;
    if (EffectsDir!=NULL) delete[] EffectsDir;
    if (AssemblerDir!=NULL) delete[] AssemblerDir;
    if (ProjectDir!=NULL) delete[] ProjectDir;
    if (SkinDir!=NULL) delete[] SkinDir;
    if (SkinName!=NULL) delete[] SkinName;
    if (ProjectName!=NULL) delete[] ProjectName;
  }
}

int CConfig::Check(){
#ifdef DEBUG
  fprintf(stderr, "DBG: CConfig: Entering Check\n");
#endif

  long value;

  // Instanciate the config file and create the config file
  config = new wxConfig(CFG_APP_NAME, CFG_VENDOR_NAME);

  if (config == NULL){
    //fprintf(stderr, "Unable to create the configuration file.\n");
    return CONFIG_CHECK_ERROR;
  }

  // Check if config file is OK for appli and vendor name.
  if (config->GetAppName() != "AgALag") goto Config_Configuration_CheckIntegrity_Error;
  if (config->GetVendorName() != "Axoris Software") goto Config_Configuration_CheckIntegrity_Error;
  
  // Check if the config file version is the good one
  if (!config->Read("ConfigFileVersion", &value)) goto Config_Configuration_CheckIntegrity_Error;
  if (value != 1){
    //fprintf(stderr, "Invalid config file version.\n");
    goto Config_Configuration_CheckIntegrity_Error;
  }
	
  // Check if the other parameters are there
  if (!config->Exists("/Dirs/EffectsDir")) goto Config_Configuration_CheckIntegrity_Error;
  if (!config->Exists("/Dirs/AssemblerDir")) goto Config_Configuration_CheckIntegrity_Error;
  if (!config->Exists("/Dirs/ProjectDir")) goto Config_Configuration_CheckIntegrity_Error;
  if (!config->Exists("/Dirs/SkinDir")) goto Config_Configuration_CheckIntegrity_Error;
  if (!config->Exists("/Project/ProjectName")) goto Config_Configuration_CheckIntegrity_Error;
  if (!config->Exists("/Options/AsmVerbose")) goto Config_Configuration_CheckIntegrity_Error;
  if (!config->Exists("/Options/MkVerbose")) goto Config_Configuration_CheckIntegrity_Error;
  if (!config->Exists("/Options/MkMap")) goto Config_Configuration_CheckIntegrity_Error;
  if (!config->Exists("/Options/SkinName")) goto Config_Configuration_CheckIntegrity_Error;
  if (!config->Exists("/Size/WindowHeight")) goto Config_Configuration_CheckIntegrity_Error;
  if (!config->Exists("/Size/WindowWidth")) goto Config_Configuration_CheckIntegrity_Error;
  if (!config->Exists("/Size/StructHeight")) goto Config_Configuration_CheckIntegrity_Error;
  if (!config->Exists("/Size/StructWidth")) goto Config_Configuration_CheckIntegrity_Error;

  return CONFIG_OK;
    
 Config_Configuration_CheckIntegrity_Error:
  //fprintf(stderr, "Configuration file doesn't exist or is invalid.\n");
  delete config;
  return CONFIG_CHECK_ERROR;
}

int CConfig::Create(){
#ifdef DEBUG
  fprintf(stderr, "DBG: CConfig: Entering Create\n");
#endif


  //fprintf(stdout, "Creating a new configuration file.\n");

  // Instanciate the config file and create the config file
  config = new wxConfig(CFG_APP_NAME, CFG_VENDOR_NAME);
  if (config == NULL) 
    if (!config->Create()) 
      goto Config_Configuration_Create_Error;
	
  // set the default values
  WindowWidth=DEFAULT_WINDOW_WIDTH; 
  WindowHeight=DEFAULT_WINDOW_HEIGHT;
  StructWidth=DEFAULT_STRUCT_WIDTH;
  StructHeight=DEFAULT_STRUCT_HEIGHT;

  ProjectName = new char[12];
  sprintf(ProjectName, "project.axp");
  EffectsDir=new char[11];
  sprintf(EffectsDir, "../effects");
  AssemblerDir=new char[7];
  sprintf(AssemblerDir, "../bin");
  ProjectDir=new char[12];
  sprintf(ProjectDir, "../projects");
  SkinDir=new char[26];
  sprintf(SkinDir, "../resources/AgALag/themes");
  SkinName = new char[12];
  sprintf(SkinName, "default");

  AsmVerbose = MkVerbose = MkMemMap = 0;

  // save them in this configuration file
  //fprintf(stdout, "Saving default values.\n");
  if (Save() != CONFIG_OK) return CONFIG_CREATE_ERROR;
  
  return CONFIG_OK;
  
 Config_Configuration_Create_Error:
  //fprintf(stderr, "Unable to create the configuration file.\n");
  delete config;
  return CONFIG_CREATE_ERROR;
}

int CConfig::Load(){
#ifdef DEBUG
  fprintf(stderr, "DBG: CConfig: Entering Load\n");
#endif

  long value;
  wxString str;

  // Load the "true cycle simulation" parameter
  if (!config->Read("/Dirs/EffectsDir", &str)) goto Config_Configuration_Read_Error;
  EffectsDir=new char[str.Length()+1];
  strcpy(EffectsDir, str.c_str());
  if (!config->Read("/Dirs/AssemblerDir", &str)) goto Config_Configuration_Read_Error;
  AssemblerDir=new char[str.Length()+1];
  strcpy(AssemblerDir, str.c_str());
  if (!config->Read("/Dirs/ProjectDir", &str)) goto Config_Configuration_Read_Error;
  ProjectDir=new char[str.Length()+1];
  strcpy(ProjectDir, str.c_str());
  if (!config->Read("/Dirs/SkinDir", &str)) goto Config_Configuration_Read_Error;
  SkinDir=new char[str.Length()+1];
  strcpy(SkinDir, str.c_str());
  if (!config->Read("/Project/ProjectName", &str)) goto Config_Configuration_Read_Error;
  ProjectName=new char[str.Length()+1];
  strcpy(ProjectName, str.c_str());
  if (!config->Read("/Options/SkinName", &str)) goto Config_Configuration_Read_Error;
  SkinName=new char[str.Length()+1];
  strcpy(SkinName, str.c_str());
  if (!config->Read("/Options/AsmVerbose", &value)) goto Config_Configuration_Read_Error;
  AsmVerbose=value;
  if (!config->Read("/Options/MkVerbose", &value)) goto Config_Configuration_Read_Error;
  MkVerbose=value;
  if (!config->Read("/Options/MkMap", &value)) goto Config_Configuration_Read_Error;
  MkMemMap=value;
  if (!config->Read("/Size/WindowHeight", &value)) goto Config_Configuration_Read_Error;
  WindowHeight=value;
  if (!config->Read("/Size/WindowWidth", &value)) goto Config_Configuration_Read_Error;
  WindowWidth=value;
  if (!config->Read("/Size/StructHeight", &value)) goto Config_Configuration_Read_Error;
  StructHeight=value;
  if (!config->Read("/Size/StructWidth", &value)) goto Config_Configuration_Read_Error;
  StructWidth=value;

  return CONFIG_OK;

 Config_Configuration_Read_Error:
  //fprintf(stderr, "Unable to read from the configuration file. Giving up...\n");
  return CONFIG_LOAD_ERROR;
}

int CConfig::Save(){
#ifdef DEBUG
  fprintf(stderr, "DBG: CConfig: Entering Save\n");
#endif

  long value;
  wxString str;

  //fprintf(stdout, "Saving configuration file.\n");

  // Write the config file version number
  value = CONFIG_FILE_VERSION_NUMBER;
  if (!config->Write("ConfigFileVersion", value)) goto Config_Configuration_Save_Error;
  
  // Write the "true cycle simulation" parameter
  str=EffectsDir;
  if (!config->Write("/Dirs/EffectsDir", str)) goto Config_Configuration_Save_Error;
  str=AssemblerDir;
  if (!config->Write("/Dirs/AssemblerDir", str)) goto Config_Configuration_Save_Error;
  str=ProjectDir;
  if (!config->Write("/Dirs/ProjectDir", str)) goto Config_Configuration_Save_Error;
  str=SkinDir;
  if (!config->Write("/Dirs/SkinDir", str)) goto Config_Configuration_Save_Error;
  value=AsmVerbose;
  if (!config->Write("/Options/AsmVerbose", value)) goto Config_Configuration_Save_Error;
  value=MkVerbose;
  if (!config->Write("/Options/MkVerbose", value)) goto Config_Configuration_Save_Error;
  value=MkMemMap;
  if (!config->Write("/Options/MkMap", value)) goto Config_Configuration_Save_Error;
  str=SkinName;
  if (!config->Write("/Options/SkinName", str)) goto Config_Configuration_Save_Error;
  str=ProjectName;
  if (!config->Write("/Project/ProjectName", str)) goto Config_Configuration_Save_Error;
  value=(WindowHeight > MINIMAL_WINDOW_HEIGHT ? WindowHeight : MINIMAL_WINDOW_HEIGHT);
  if (!config->Write("/Size/WindowHeight", value)) goto Config_Configuration_Save_Error;
  value=(WindowWidth > MINIMAL_WINDOW_WIDTH ? WindowWidth : MINIMAL_WINDOW_WIDTH);
  if (!config->Write("/Size/WindowWidth", value)) goto Config_Configuration_Save_Error;
  value=(StructHeight > MINIMAL_STRUCT_HEIGHT ? StructHeight : MINIMAL_STRUCT_HEIGHT);
  if (!config->Write("/Size/StructHeight", value)) goto Config_Configuration_Save_Error;
  value=(StructWidth > MINIMAL_STRUCT_WIDTH ? StructWidth : MINIMAL_STRUCT_WIDTH);
  if (!config->Write("/Size/StructWidth", value)) goto Config_Configuration_Save_Error;
  
  return CONFIG_OK;

 Config_Configuration_Save_Error:
  //fprintf(stderr, "Unable to write in the configuration file. Giving up...\n");
  return CONFIG_SAVE_ERROR;
}
