/////////////////////////////////////////////////////////////////////////////
// Name:        about.cpp
// Project:     GUI for Axoris AL3101 application generator
// Author:      Gregor(y) Soyez
// Version:     1.1
// Created:     05/01/02
// Copyright:   (c) Axoris
/////////////////////////////////////////////////////////////////////////////

#include "directories.h"

// ID's
#define ID_STATIC  1
#define ID_EFFECTS 2
#define ID_ASM     3
#define ID_PROJECT 4
#define ID_SKIN    5
#define ID_BTN_ASM 6
#define ID_BTN_EFF 7
#define ID_BTN_PRJ 8
#define ID_BTN_SKN 9

#include <wx/stattext.h>
#include <wx/textctrl.h>

BEGIN_EVENT_TABLE(CDirectories, wxDialog)
  EVT_BUTTON(ID_BTN_ASM, CDirectories::OnBrowse)
  EVT_BUTTON(ID_BTN_EFF, CDirectories::OnBrowse)
  EVT_BUTTON(ID_BTN_PRJ, CDirectories::OnBrowse)
  EVT_BUTTON(ID_BTN_SKN, CDirectories::OnBrowse)
END_EVENT_TABLE()

//-----------------------------------------------------
// implementation of CAbout
//-----------------------------------------------------

CDirectories::CDirectories(wxWindow* parent, wxWindowID id, CConfig *cfg)
  : wxDialog(parent, id, "Directories", wxDefaultPosition, wxSize(250,170), wxDEFAULT_DIALOG_STYLE){
#ifdef DEBUG
  fprintf(stderr, "DBG: directories: Entering constructor\n");
#endif

  SetClientSize(330,270);
 
  // creates all widgets inside the dialog box
  TxtIntro = new wxStaticText(this, ID_STATIC, "Configure the directories" , wxPoint( 15, 10), wxSize(210, 20));
 
  EffectsDir << cfg->EffectsDir;
  AssemblerDir << cfg->AssemblerDir;
  ProjectDir << cfg->ProjectDir;
  SkinDir << cfg->SkinDir;
 
  TxtEffects = new wxStaticText(this, ID_STATIC, "Directory for effects:", wxPoint( 15, 40), wxSize(210, 20));
  EdtEffects = new wxTextCtrl(this, ID_EFFECTS, wxString(cfg->EffectsDir), wxPoint(25, 60), wxSize(200, 20),
                              0, wxTextValidator(wxFILTER_ASCII, &EffectsDir));
  BtnEffects = new wxButton(this, ID_BTN_EFF, "Browse", wxPoint(250, 60), wxSize(60, 20));

  TxtAssembler = new wxStaticText(this, ID_STATIC, "Directory for assembler:", wxPoint( 15, 90), wxSize(210, 20));
  EdtAssembler = new wxTextCtrl(this, ID_ASM, wxString(cfg->AssemblerDir), wxPoint(25, 110), wxSize(200, 20),
                                0, wxTextValidator(wxFILTER_ASCII, &AssemblerDir));
  BtnAssembler = new wxButton(this, ID_BTN_ASM, "Browse", wxPoint(250,110), wxSize(60, 20));
 
  TxtProject = new wxStaticText(this, ID_STATIC, "Directory for projects:", wxPoint( 15,140), wxSize(210, 20));
  EdtProject = new wxTextCtrl(this, ID_PROJECT, wxString(cfg->ProjectDir), wxPoint(25,160), wxSize(200, 20),
                              0, wxTextValidator(wxFILTER_ASCII, &ProjectDir));
  BtnEffects = new wxButton(this, ID_BTN_PRJ, "Browse", wxPoint(250,160), wxSize(60, 20));
 
  TxtSkin = new wxStaticText(this, ID_STATIC, "Directory for skins:", wxPoint( 15,190), wxSize(210, 20));
  EdtSkin = new wxTextCtrl(this, ID_SKIN, wxString(cfg->SkinDir), wxPoint(25,210), wxSize(200, 20),
                              0, wxTextValidator(wxFILTER_ASCII, &SkinDir));
  BtnSkin = new wxButton(this, ID_BTN_SKN, "Browse", wxPoint(250,210), wxSize(60, 20));
 
  BtnOk = new wxButton(this, wxID_OK, "Ok", wxPoint(50,240), wxSize(50, 20));
  BtnCancel = new wxButton(this, wxID_CANCEL, "Cancel", wxPoint(150,240), wxSize(50, 20));
}

CDirectories::~CDirectories(){
#ifdef DEBUG
  fprintf(stderr, "DBG: directories: Entering destructor\n");
#endif
}

void CDirectories::OnBrowse(wxCommandEvent &event){
  wxDirDialog *DirChooser;
  wxString DefaultPath;
  wxTextCtrl *edt=NULL;

  switch(event.GetId()){
  case ID_BTN_EFF: DefaultPath=EffectsDir;   edt=EdtEffects;   break;
  case ID_BTN_ASM: DefaultPath=AssemblerDir; edt=EdtAssembler; break;
  case ID_BTN_PRJ: DefaultPath=ProjectDir;   edt=EdtProject;   break;
  case ID_BTN_SKN: DefaultPath=SkinDir;      edt=EdtSkin;      break;
  default: event.Skip();
  }

  // retreive the directory
  DirChooser=new wxDirDialog(this, "Choose a directory", DefaultPath, wxDD_NEW_DIR_BUTTON);
  if (DirChooser->ShowModal()==wxID_OK){
    edt->SetValue(DirChooser->GetPath());
  }

  delete DirChooser;
}
