/////////////////////////////////////////////////////////////////////////////
// Name:        frame.h
// Project:     GUI for Axoris AL3101 application generator
// Author:      Gregor(y) Soyez
// Version:     1.1
// Created:     04/01/02
// Copyright:   (c) Axoris
/////////////////////////////////////////////////////////////////////////////

#ifndef __PROJ_AL_MAINFRAME_H__
#define __PROJ_AL_MAINFRAME_H__

// For compilers that support precompilation, includes "wx/wx.h".
#include "wx/wxprec.h"

#ifdef __BORLANDC__
    #pragma hdrstop
#endif

// for all others, include the necessary headers (this file is usually all you
// need because it includes almost all "standard" wxWindows headers)
#ifndef WX_PRECOMP
    #include "wx/wx.h"
#endif

#include <wx/splitter.h>
#include "glstruct.h"
#include "listmodules.h"
#include "Config.h"
#include "fileio.h"
#include "projopt.h"
#include "theme.h"

// Define a new frame type: this is going to be our main frame

class MainFrame : public wxFrame
{
public:
    // ctor(s)
    MainFrame();
    ~MainFrame();

    int IsInitialised;

    int Resize();

    // event handlers (these functions should _not_ be virtual)
    // File Menu
    void OnNew(wxCommandEvent& event);
    void OnOpen(wxCommandEvent& event);
    void OnSave(wxCommandEvent& event);
    void OnSaveAs(wxCommandEvent& event);
    void OnPrint(wxCommandEvent& event);
    void OnQuit(wxCommandEvent& event);
    // Edit Menu
    void OnClearConsole(wxCommandEvent& event);
    // Project Menu
    void OnMake(wxCommandEvent& WXUNUSED(event));
    void OnAssemble(wxCommandEvent& WXUNUSED(event));
    void OnMAsm(wxCommandEvent& event);
    void OnSimulate(wxCommandEvent& event);
    void OnMissParker(wxCommandEvent& event);
    void OnProjectInfo(wxCommandEvent& event);
    void OnProjectOptions(wxCommandEvent& event);
 //LOAD   void OnLoad(wxCommandEvent& event);
    // Options Menu
    void OnCfgDirs(wxCommandEvent& event);
    void OnCfgColors(wxCommandEvent& event);
    // Help Menu
    void OnHelp(wxCommandEvent& event);
    void OnBugReport(wxCommandEvent& event);
    void OnAbout(wxCommandEvent& event);

    // dragging from module tree
    void OnTreeDrag(wxTreeEvent& event);

    wxGauge *MemStatus, *DSPStatus;

private:
    // initialization functions
    int CreateMenu();
    int CreateStatus();
    int Split();

    // menu
    wxMenuBar *MenuBar;
    wxMenu *MenuFile, *MenuEdit, *MenuProject, *MenuOptions, *MenuHelp;

    // widgets and splitters of the frame
    CModuleList *ModuleList;
    wxTextCtrl *Console;
    CglStruct *Struct;
    wxSplitterWindow *Total, *Upper;
    wxLogTextCtrl *AppConsole;

    // project and options
    CFile *Project;
    TProjectOptions ProjectOptions;

    // theme
    TTheme theme;

    // any class wishing to process wxWindows events must use this macro
    DECLARE_EVENT_TABLE()
};

// ----------------------------------------------------------------------------
// constants
// ----------------------------------------------------------------------------

// IDs for the controls and the menu commands
enum
{
  // menu items
  Menu_New=1,
  Menu_Open,
  Menu_Save,
  Menu_SaveAs,
  Menu_Print,
  Menu_Quit,
  Menu_ClearC,
  Menu_Make,
  Menu_Assemble,
  Menu_MAsm,
  Menu_Info,
  Menu_Options,
  Menu_Simulate,
  Menu_MissParker,
  Menu_Load,
  Menu_CfgDirs,
  Menu_CfgColors,
  Menu_Help,
  Menu_BugReport,
  Menu_About
};

// windows ID
#define IDWinTotal      101
#define IDWinUpper      102
#define IDWinConsole    103
#define IDWinModList    104
#define IDWinGLStruct   105

#endif
