/////////////////////////////////////////////////////////////////////////////
// Name:        glmodule.h
// Project:     GUI for Axoris AL3101 application generator
// Author:      Gregor(y) Soyez
// Version:     1.1
// Created:     06/01/02
// Copyright:   (c) Axoris
/////////////////////////////////////////////////////////////////////////////

#ifndef __PROJ_AL_GLMODULE_H__
#define __PROJ_AL_GLMODULE_H__

// For compilers that support precompilation, includes "wx/wx.h".
#include "wx/wxprec.h"

#ifdef __BORLANDC__
    #pragma hdrstop
#endif

// for all others, include the necessary headers (this file is usually all you
// need because it includes almost all "standard" wxWindows headers)
#ifndef WX_PRECOMP
    #include "wx/wx.h"
#endif

#include "../AL3101ModuleHeader.h"
#include "AL3101DealProperties.h"
#include <wx/image.h>
#include "theme.h"
#include "mpprojectfile.h"

#define MODULE_OK       0
#define MODULE_RENDER   1
#define MODULE_ERROR    2

#define MODULE_OUTSIDE  0
#define MODULE_INSIDE   1
#define MODULE_OUTPUT   2
#define MODULE_INPUT    3

#define MOUSE_BUTTON_LEFT    0
#define MOUSE_BUTTON_RIGHT   1

#define MODULE_STATE_STATIC  0
#define MODULE_STATE_MOVING  1

#define FONT_SIZE 16

class CglModule
{
 public:
  CglModule(wxWindow *i_parent, wxString i_Name, 
	    wxString i_Path, wxString i_Label, 
	    TTheme *i_theme,
	    int x_pos, int y_pos,
	    TModuleInfo *i_info);
  virtual ~CglModule();

  // header informations
  TModuleInfo info;

  // update the module size
  void UpdateSize();

  // update the textures
  void UpdateTextures();

  // when moving the module, tells the mouse moved
  int MousePositionChanged(int new_x, int new_y);

  // Renderize the module
  virtual void Render(wxDC *dc, int UseTextures);

  // check if the given position is inside th module
  // we return MODULE_INSIDE  if inside the module core
  //                  OUTSIDE if not in the module
  //                  OUTPUT  if it correspond to an output
  //                  INOUT   if it correspond to an input 
  // for the last two cases, the index of the IO is returned through io
  int IsInside(int px, int py, int *io);

  // start moving the module
  int StartMove(int mx, int my);

  // finish module move
  int FinishMove();

  // set index or the module and retreive it.
  void SetIndex(int);
  int GetIndex();

  // Get Module Name
  wxString GetName();
  wxString GetPath();

  // Retreive, Update, Set and Save module properties
  int RetreivePropertyList(char *dir);
  int UpdateProperties();
  int SetProperty(int i, char *val);
  int SaveProperties(FILE *flux);
  int BuildParamList();

  int x, y;                    // position is accessible from link !!
  int nx, ny;                  // size in x and y

  int NbProperties;            // Number of Editable Properties
  TModuleProperty *Properties; // Editable Properties (cfr SEC 2 of code)
  wxString Name;               // Name of the module
  wxString Label;              // Label of the module
  TParameterItem *mpparam;     // list of parameters for MissParker project
  int MemoryOffset;            // after building: memory offset

protected:
  wxWindow *Parent;            // parent window
  wxString Path;               // Path to the module (from eeffect dir)
  int Index;                   // index for the module
  int x_offset, y_offset;      // info for moving module
  int State;                   // moving or static
  wxImage TexBk, TexIO;        // textures for background and IO's
  TTheme *theme;               // glStruct theme

  int BuildDefaultParamList(bool Enable);
};

#endif
