/////////////////////////////////////////////////////////////////////////////
// Name:        main.cpp
// Project:     GUI for Axoris AL3101 application generator
// Author:      Gregor(y) Soyez
// Version:     1.1
// Created:     04/01/02
// Copyright:   (c) Axoris
/////////////////////////////////////////////////////////////////////////////

// ----------------------------------------------------------------------------
// headers
// ----------------------------------------------------------------------------
#ifdef __GNUG__
#pragma implementation "minimal.cpp"
#pragma interface "minimal.cpp"
#endif

// For compilers that support precompilation, includes "wx/wx.h".
#include "wx/wxprec.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif

// for all others, include the necessary headers (this file is usually all you
// need because it includes almost all "standard" wxWindows headers)
#ifndef WX_PRECOMP
#include <wx/wx.h>
#endif

#include "frame.h"

#include <wx/textctrl.h>
#include <wx/splash.h>

#ifndef WIN32
#include <config.h>
#endif

//wxTextCtrl *AppConsole;

// ----------------------------------------------------------------------------
// private classes
// ----------------------------------------------------------------------------

// Define a new application type, each program should derive a class from wxApp
class MyApp : public wxApp{
public:
  // override base class virtuals
  // ----------------------------
  // this one is called on application startup and is a good place for the app
  // initialization (doing it here and not in the ctor allows to have an error
  // return: if OnInit() returns false, the application terminates)
  virtual bool OnInit();
  
  virtual int OnExit();
  
private:
  CConfig *Cfg;
};

// Create a new application object: this macro will allow wxWindows to create
// the application object during program execution (it's better than using a
// static object for many reasons) and also declares the accessor function
// wxGetApp() which will return the reference of the right type (i.e. MyApp and
// not wxApp)
IMPLEMENT_APP(MyApp)

// ============================================================================
// implementation
// ============================================================================

// ----------------------------------------------------------------------------
// the application class
// ----------------------------------------------------------------------------

// 'Main program' equivalent: the program execution "starts" here
bool MyApp::OnInit(){
#ifdef DEBUG
  fprintf(stderr, "DBG: main: Initializing application\n");
#endif
  wxSplashScreen* splash=NULL;

  // under linux platform set working dir to installation dir
#ifndef WIN32
  wxSetWorkingDirectory(BIN_INSTALL_DIR);
#endif


  // create the main application window
  MainFrame *frame = new MainFrame();
  // and show it (the frames, unlike simple controls, are not shown when
  // created initially)
  if (!(frame->IsInitialised))
    return FALSE;

  wxBitmap bitmap;
  if (bitmap.LoadFile("../resources/AgALag/intro.xpm", wxBITMAP_TYPE_XPM))
  {
      splash = new wxSplashScreen(bitmap,
				  wxSPLASH_CENTRE_ON_SCREEN|wxSPLASH_TIMEOUT,
				  5000, NULL, -1, 
				  wxDefaultPosition, wxDefaultSize,
				  wxSIMPLE_BORDER|wxSTAY_ON_TOP);
  }
  //wxYield();

  frame->Show(TRUE);
  
  SetTopWindow(frame);
  if (splash!=NULL) delete splash;

  // success: wxApp::OnRun() will be called which will enter the main message
  // loop and the application will run. If we returned FALSE here, the
  // application would exit immediately.
  return TRUE;
}


int MyApp::OnExit(){
#ifdef DEBUG
  fprintf(stderr, "DBG: main: Exiting Application\n");
#endif

  return 0;
}
