/////////////////////////////////////////////////////////////////////////////
// Name:        mpproject.cpp
// Project:     GUI for Axoris AL3101 application generator
// Author:      Gregor(y) Soyez
// Version:     1.4
// Created:     05/01/02
// Copyright:   (c) Axoris
/////////////////////////////////////////////////////////////////////////////

//#define MP_DEBUG

// ID's
#define ID_STATIC  1
#define ID_EFFECTS 2
#define ID_ASM     3
#define ID_PROJECT 4
#define ID_SKIN    5
#define ID_BTN_ASM 6
#define ID_BTN_EFF 7
#define ID_BTN_PRJ 8
#define ID_BTN_SKN 9

#include "mpproject.h"
#include <wx/filename.h>
#include <wx/utils.h>
#include <wx/colour.h>
#include <wx/notebook.h>

#define ID_PARAMLIST  1001
#define ID_STORE      1002
#define ID_DESCR      1003
#define ID_COMP       1004
#define ID_TYPE       1005
#define ID_PROG       1006
#define ID_MINIMUM    1008
#define ID_MAXIMUM    1009
#define ID_DVALUE     1010
#define ID_DMINIMUM   1011
#define ID_DMAXIMUM   1012
#define ID_STEPS      1013
#define ID_DISP       1014
#define ID_TABNAME    1015
#define ID_CBOLAB     1016
#define ID_CBOVAL     1017
#define ID_CBOTABLE   1018
#define ID_ADDTOCBO   1019
#define ID_REMOVECBO  1020
#define ID_DEFAULTCBO 1021
#define ID_ENABLE     1022
#define ID_PAGER      1023
#define ID_PARAMPAGE  1024
#define ID_GROUPPAGE  1025
#define ID_GROUPLIST  1026
#define ID_GROUPNAME  1027
#define ID_GROUPADD   1028
#define ID_GROUPREM   1029
#define ID_GROUPTXT   1030
#define ID_GROUPASSIG 1031

//-----------------------------------------------------
// implementation of CParameterTree
//-----------------------------------------------------

CParameterTree::CParameterTree(wxWindow *i_parent, wxWindowID id, const wxPoint& pos, 
			       const wxSize& size)
  :wxTreeCtrl(i_parent, id, pos, size, wxTR_HAS_BUTTONS | wxTR_SINGLE){
#ifdef MP_DEBUG
  fprintf(stderr, "MP_DBG: CParameterTree: Entering constructor\n");
#endif

  // Create image list
  wxBitmap Icon[3];
  Icon[0].LoadFile("../resources/AgALag/module.xpm",wxBITMAP_TYPE_XPM);
  Icon[1].LoadFile("../resources/AgALag/treefolderclosed.xpm",wxBITMAP_TYPE_XPM);
  Icon[2].LoadFile("../resources/AgALag/treefolderopen.xpm",wxBITMAP_TYPE_XPM);
  image = new wxImageList(16, 16, TRUE);
  image->Add(Icon[0]);
  image->Add(Icon[1]);
  image->Add(Icon[2]);
  SetImageList(image);

#ifdef MP_DEBUG
  fprintf(stderr, "MP_DBG: CParameterTree: Leaving constructor\n");
#endif
}

CParameterTree::~CParameterTree(){
#ifdef MP_DEBUG
  fprintf(stderr, "MP_DBG: CParameterTree: Entering destructor\n");
#endif

  //delete[] image;

#ifdef MP_DEBUG
  fprintf(stderr, "MP_DBG: CParameterTree: Leaving destructor\n");
#endif
}

int CParameterTree::GetParamList(TglModuleList *mods){
#ifdef MP_DEBUG
  fprintf(stderr, "MP_DBG: listmodules: Entering GetParamList\n");
#endif
  wxTreeItemId mod_id;

  // create root
  wxTreeItemId RootId=AddRoot("Parameters", 1, 1);

  TglModuleList *mlist;
  TParameterItem *item;

  mlist = mods;

  while (mlist!=NULL){
    if (mlist->Mod->mpparam!=NULL)
      mod_id = AppendItem(GetRootItem(), mlist->Mod->Label,1,1);
    item = mlist->Mod->mpparam;
    while (item!=NULL){
      AddParam(item, mod_id);
      item = item->next;
    };
    mlist = (TglModuleList*) mlist->Next;
  };

  SetItemImage(RootId, 2, wxTreeItemIcon_Expanded);
  Expand(RootId);

#ifdef MP_DEBUG
  fprintf(stderr, "MP_DBG: CParameterTree: Leaving GetParamList\n");
#endif
  return 0;
}

int CParameterTree::AddParam(TParameterItem *prop, wxTreeItemId mod_id){
#ifdef MP_DEBUG
  fprintf(stderr, "MP_DBG: CParameterTree: Entering AddParam\n");
#endif

  prop->id = AppendItem(mod_id, prop->ParameterName,0,0);

#ifdef MP_DEBUG
  fprintf(stderr, "MP_DBG: CParameterTree: Leaving AddParam\n");
#endif
  return 0;
}

//-----------------------------------------------------
// implementation of parameter page
//-----------------------------------------------------

CParameterManager::CParameterManager(wxWindow *i_parent, wxWindowID id, const wxPoint& pos=wxDefaultPosition, const wxSize& size=wxDefaultSize):
  wxWindow(i_parent, id, pos, size){
#ifdef MP_DEBUG
  fprintf(stderr, "MP_DBG: CParameterManager: Entering constructor\n");
#endif
  
  param_tree=new CParameterTree(this, ID_PARAMLIST, wxPoint(4,4), wxSize(200,370));

  TxtAssign = new wxCheckBox(this, ID_GROUPTXT, "Assign to group", wxPoint(4,380),wxSize(200,24));
  GroupAssign = new wxComboBox(this, ID_GROUPASSIG, "",   wxPoint(14,410), wxSize(180, 24),
			       0, NULL, wxCB_DROPDOWN);
  enable=new wxCheckBox(this, ID_ENABLE, "Enable parameter", wxPoint(4,440), wxSize(200, 30));

  parent = (CMPPreProjectDialog*) i_parent;

#ifdef MP_DEBUG
  fprintf(stderr, "MP_DBG: CParameterManager: Leaving constructor\n");
#endif
}

CParameterManager::~CParameterManager(){
#ifdef MP_DEBUG
  fprintf(stderr, "MP_DBG: CParameterManager: Entering Destructor\n");
#endif

#ifdef MP_DEBUG
  fprintf(stderr, "MP_DBG: CParameterManager: Leaving Destructor\n");
#endif
}

int CParameterManager::AppendGroup(wxString group){
#ifdef MP_DEBUG
  fprintf(stderr, "MP_DBG: CParameterManager: Entering AppendGroup\n");
#endif

  GroupAssign->Append(group);

#ifdef MP_DEBUG
  fprintf(stderr, "MP_DBG: CParameterManager: Leaving AppendGroup\n");
#endif
  return 0;
}

//-----------------------------------------------------
// implementation of group page
//-----------------------------------------------------

BEGIN_EVENT_TABLE(CGroupManager, wxWindow)
  EVT_BUTTON(ID_GROUPADD, CGroupManager::OnGroupAdd)
  EVT_BUTTON(ID_GROUPREM, CGroupManager::OnGroupRemove)
END_EVENT_TABLE()

CGroupManager::CGroupManager(wxWindow *i_parent, wxWindowID id, const wxPoint& pos=wxDefaultPosition, const wxSize& size=wxDefaultSize):
  wxWindow(i_parent, id, pos, size){
#ifdef MP_DEBUG
  fprintf(stderr, "MP_DBG: CGroupManager: Entering constructor\n");
#endif
  
  group_tree=new wxTreeCtrl(this, ID_GROUPLIST, wxPoint(4,4), wxSize(200,380));
  group_tree->AddRoot("Groups", 0, 0);

  ToAdd = new wxTextCtrl(this, ID_GROUPNAME, "", wxPoint(4,400), wxSize(200,30));
  Add = new wxButton(this, ID_GROUPADD, "Add", wxPoint(4,440), wxSize(90, 30));
  Remove = new wxButton(this, ID_GROUPREM, "Remove", wxPoint(114,440), wxSize(90, 30));

#ifdef MP_DEBUG
  fprintf(stderr, "MP_DBG: CGroupManager: leaving constructor\n");
#endif
}

CGroupManager::~CGroupManager(){
#ifdef MP_DEBUG
  fprintf(stderr, "MP_DBG: CGroupManager: Entering Destructor\n");
#endif

#ifdef MP_DEBUG
  fprintf(stderr, "MP_DBG: CGroupManager: Leaving Destructor\n");
#endif  
}

void CGroupManager::OnGroupAdd(wxCommandEvent& event){
#ifdef MP_DEBUG
  fprintf(stderr, "MP_DBG: CGroupManager: Entering OnGroupAdd\n");
#endif

  wxString GroupName;
  
  // get the group name to add and add it if it is non-empty
  GroupName = ToAdd->GetValue();
  if (!GroupName.IsEmpty()){
    // test that group is not already present !
    TParameterItem *it_add;
    bool is_present=false;

    it_add = parent->groups;
    while (it_add!=NULL){
      if (strcmp(GroupName.c_str(), it_add->ParameterName)==0) is_present=true;
      it_add = it_add->next;
    };

    if (is_present){
      wxMessageBox("A grup with the same name already exists", "Note");
      return;
    }      

    // append item
    // 1. To the list of groups in the program core
    //printf("append item\n");
    //printf("1. To the list of groups in the program core\n");
    it_add = new TParameterItem;
    it_add->next = parent->groups;
    parent->groups = it_add;
    strcpy(it_add->ParameterName, GroupName.c_str());
    strcpy(it_add->ParameterDescription, GroupName.c_str());
    AssignDefaultValues(it_add);
    it_add->NumberOfSubParams = 0;

    // 2. To the group tree
    //printf("2. To the group tree\n");
    it_add->id = group_tree->AppendItem(group_tree->GetRootItem(), GroupName);
    group_tree->Expand(group_tree->GetRootItem());

    // 3. To the group combo below parameter list
    //printf("3. To the group combo below parameter list\n");
    parent->page_param->AppendGroup(GroupName);
    //printf("Done\n");
  }

#ifdef MP_DEBUG
  fprintf(stderr, "MP_DBG: CGroupManager: Leaving OnGroupAdd\n");
#endif
}

void CGroupManager::OnGroupRemove(wxCommandEvent& event){
#ifdef MP_DEBUG
  fprintf(stderr, "MP_DBG: CGroupRemove: Entering OnGroupRemove\n");
#endif

  wxTreeItemId sel;

  // retreive selection and check its validity
  sel = group_tree->GetSelection();
  if ((sel.IsOk()) && (sel!=group_tree->GetRootItem())){
    // search the item in the list.
    TParameterItem *it = parent->groups;
    TParameterItem *it_prev = NULL;
    TParameterItem *it_param = NULL;
    bool stop=false;
    int i;

    do {
      if (sel==it->id) 
	stop=true;
      else{
	it_prev = it;
	it = it->next;
      }
    } while ((!stop) && (it!=NULL));
    if (!stop){
      wxMessageBox("Consistency Problem: that group does not seem registered.\nPlease report the problem to Axoris", "Error");
      return;
    }

    // remove the group from the combo box in the param page
    int index;
    wxString sel_txt;

    sel_txt = group_tree->GetItemText(sel);
    index = parent->page_param->GroupAssign->FindString(sel_txt);
    if (index==-1){
      wxMessageBox("Consistency Problem: that group does not seem registered in the parameter window.\nPlease report the problem to Axoris", "Error");
      return;
    }
    parent->page_param->GroupAssign->Remove(index,index);

    // restore all the params associated to that group to
    // normal parameters
    for (i=0;i<it->NumberOfSubParams;i++){
      it_param = it->SubParams[i];
      it->InGroup = 0;
      it->GroupName = NULL;
    }

    // remove the item
    if (it_prev!=NULL)
      it_prev->next = it->next;
    delete it;
    group_tree->Delete(sel);
  } else {
    wxMessageBox("Please select a group in the list before", "Removing note");
  }

#ifdef MP_DEBUG
  fprintf(stderr, "MP_DBG: CGroupManager: Leaving OnGroupAdd\n");
#endif
}

//-----------------------------------------------------
// implementation of CMPPreProjectDialog
//-----------------------------------------------------

BEGIN_EVENT_TABLE(CMPPreProjectDialog, wxDialog)
  EVT_BUTTON(ID_STORE, CMPPreProjectDialog::OnStore)
  EVT_BUTTON(ID_ADDTOCBO, CMPPreProjectDialog::OnAddToCombo)
  EVT_BUTTON(ID_REMOVECBO, CMPPreProjectDialog::OnRemoveCombo)
  EVT_BUTTON(ID_DEFAULTCBO, CMPPreProjectDialog::OnDefaultCombo)
  EVT_TREE_SEL_CHANGED(ID_PARAMLIST, CMPPreProjectDialog::OnSelChanged)
  EVT_TREE_SEL_CHANGED(ID_GROUPLIST, CMPPreProjectDialog::OnGroupChanged)
  EVT_COMBOBOX(ID_COMP, CMPPreProjectDialog::OnCompChanged)
  EVT_COMBOBOX(ID_TYPE, CMPPreProjectDialog::OnTypeChanged)
  EVT_CHECKBOX(ID_ENABLE, CMPPreProjectDialog::OnEnable)
  EVT_CHECKBOX(ID_GROUPTXT, CMPPreProjectDialog::OnAssignGroup)
  EVT_COMBOBOX(ID_GROUPASSIG, CMPPreProjectDialog::OnChangeAssignGroup)
END_EVENT_TABLE()

CMPPreProjectDialog::CMPPreProjectDialog(CglModuleList *i_mods, wxWindow* parent, wxWindowID id)
  : wxDialog(parent, id, "MPPreProjectDialog", wxDefaultPosition, wxSize(250,170), wxDEFAULT_DIALOG_STYLE){
#ifdef MP_DEBUG
  fprintf(stderr, "MP_DBG: MPPreProjectDialog: Entering constructor\n");
#endif

  wxString txt;

  l_mods = i_mods;
  mods = i_mods->List;
  groups = i_mods->mpgroups;
  
  actual_prop = NULL;
  SetClientSize(640,600);
  
  BtnOk = new wxButton(this, wxID_OK, "Ok", wxPoint(45,550), wxSize(60, 24));
  BtnCancel = new wxButton(this, wxID_CANCEL, "Cancel", wxPoint(145,550), wxSize(60, 24));

  // create the parameter list
  pager = new wxNotebook(this, ID_PAGER, wxPoint(20,20), wxSize(210,510), wxNB_TOP | wxNB_FIXEDWIDTH);
  page_param = new CParameterManager(pager, ID_PARAMPAGE, wxPoint(0,0), wxSize(210,475));
  page_group = new CGroupManager(pager, ID_PARAMPAGE, wxPoint(0,0), wxSize(210,475));
  page_group->parent = this;
  pager->AddPage(page_param, "Parameters");
  pager->AddPage(page_group, "Groups");
  
  //tree = new CParameterTree(this, ID_PARAMLIST, wxPoint(24,44), wxSize(200,450));
  tree = page_param->param_tree;
  tree->GetParamList(mods);

  //EnableParam = new wxCheckBox(this, ID_ENABLE, "Enable parameter", wxPoint(24,480), wxSize(200, 30));
  EnableParam = page_param->enable;
  EnableParam->Enable(false);

  // create the parameter properties
  txt << "Properties for parameter ?? ";
  TxtIntro = new wxStaticText(this, ID_STATIC, txt , wxPoint(250, 24), wxSize(380, 20));

  TxtDescr = new wxStaticText(this, ID_STATIC, "Description", wxPoint(250, 65), wxSize(130,20));

  TxtComp = new wxStaticText(this, ID_STATIC, "Computation type", wxPoint(250, 95), wxSize(170,20));
  TxtType = new wxStaticText(this, ID_STATIC, "Type",             wxPoint(250,125), wxSize(170,20));
  TxtProg = new wxStaticText(this, ID_STATIC, "Progression type", wxPoint(250,155), wxSize(170,20));

  TxtActual  = new wxStaticText(this, ID_STATIC, "Internal", wxPoint(360,185), wxSize(100,20));
  TxtDisplay = new wxStaticText(this, ID_STATIC, "Display",  wxPoint(470,185), wxSize(100,20));
  TxtVal = new wxStaticText(this, ID_STATIC, "Value"  , wxPoint( 250,215), wxSize(100, 20));
  TxtMin = new wxStaticText(this, ID_STATIC, "Minimum", wxPoint( 250,245), wxSize(100, 20));
  TxtMax = new wxStaticText(this, ID_STATIC, "Maximum", wxPoint( 250,275), wxSize(100, 20));

  TxtDisp    = new wxStaticText(this, ID_STATIC, "Display type", wxPoint( 250,305), wxSize(130, 20));
  TxtNbSteps = new wxStaticText(this, ID_STATIC, "Nb steps",     wxPoint( 450,305), wxSize(100, 20));

  TxtTableName = new wxStaticText(this, ID_STATIC, "Table file", wxPoint( 250,340), wxSize(130, 20));

  EdtDescr =  new wxTextCtrl(this, ID_DESCR, "", wxPoint(360, 63), wxSize(220, 24));

  EdtMin  = new wxTextCtrl(this, ID_MINIMUM , "0.0", 
			      wxPoint(350,243), wxSize(80, 24));
  EdtMax  = new wxTextCtrl(this, ID_MAXIMUM , "1.0", 
			      wxPoint(350,273), wxSize(80, 24));
  EdtDVal = new wxTextCtrl(this, ID_DVALUE  , "0.5", 
			      wxPoint(460,213), wxSize(80, 24));
  EdtDMin = new wxTextCtrl(this, ID_DMINIMUM, "0.0", 
			      wxPoint(460,243), wxSize(80, 24));
  EdtDMax = new wxTextCtrl(this, ID_DMAXIMUM, "1.0", 
			      wxPoint(460,273), wxSize(80, 24));

  EdtNbSteps = new wxTextCtrl(this, ID_STEPS, "20", 
			      wxPoint(535,303), wxSize(80, 24));
  
  EdtTableName = new wxTextCtrl(this, ID_TABNAME, "", 
				wxPoint(380, 340), wxSize(200,24));

  EdtComboLabelToAdd = new wxTextCtrl(this, ID_CBOLAB, "",
				      wxPoint(275, 523), wxSize(80,24));
  EdtComboValueToAdd = new wxTextCtrl(this, ID_CBOVAL, "",
				      wxPoint(405, 523), wxSize(80,24));

  CompChoices[0] << "auto";
  CompChoices[1] << "table";
  CboComp = new wxComboBox(this, ID_COMP, "auto",   wxPoint(380, 93), wxSize(80, 24),
			   2, CompChoices, wxCB_DROPDOWN | wxCB_READONLY);

  TypeChoices[0] << "slider";
  TypeChoices[1] << "combo";
  CboType = new wxComboBox(this, ID_TYPE, "slider", wxPoint(380,123), wxSize(80, 24),
			   2, TypeChoices, wxCB_DROPDOWN | wxCB_READONLY);

  ProgChoices[0] << "lin";
  ProgChoices[1] << "log";
  ProgChoices[2] << "exp";
  CboProg = new wxComboBox(this, ID_PROG, "lin",    wxPoint(380,153), wxSize(80, 24),
			   3, ProgChoices, wxCB_DROPDOWN | wxCB_READONLY);

  DispChoices[0] << "int";
  DispChoices[1] << "float";
  CboDisp = new wxComboBox(this, ID_DISP, "float",  wxPoint(360,303), wxSize(80, 24),
			   2, DispChoices, wxCB_DROPDOWN | wxCB_READONLY);

  ComboTable = new wxListCtrl(this, ID_CBOTABLE, wxPoint(250,375), wxSize(260, 140), wxLC_REPORT | wxLC_SINGLE_SEL | wxLC_HRULES | wxLC_VRULES);
  ComboTable->InsertColumn(0, "Label", wxLIST_FORMAT_LEFT, 159);
  ComboTable->InsertColumn(1, "Value", wxLIST_FORMAT_CENTER, 99);

  AddToCombo = new wxButton(this, ID_ADDTOCBO, "Add", wxPoint(550,383), wxSize(70, 24));
  RemoveFromCombo = new wxButton(this, ID_REMOVECBO, "Remove", wxPoint(550,413), wxSize(70, 24));
  DefaultCombo = new wxButton(this, ID_DEFAULTCBO, "Default", wxPoint(550,443), wxSize(70, 24));

  EnableControls(false);
  BtnStore = new wxButton(this, ID_STORE, "Store", wxPoint(400,560), wxSize(54, 24));

#ifdef MP_DEBUG
  fprintf(stderr, "MP_DBG: MPPreProjectDialog: Leaving constructor\n");
#endif
}

CMPPreProjectDialog::~CMPPreProjectDialog(){
#ifdef MP_DEBUG
  fprintf(stderr, "MP_DBG: MPPreProjectDialog: Entering destructor\n");
#endif

  l_mods->mpgroups = groups;

#ifdef MP_DEBUG
  fprintf(stderr, "MP_DBG: MPPreProjectDialog: Leaving destructor\n");
#endif
}

int CMPPreProjectDialog::EnableControls(bool flag){
#ifdef MP_DEBUG
  fprintf(stderr, "MP_DBG: MPPreProjectDialog: Entering EnableControls\n");
#endif

  TxtIntro->Enable(flag);
  TxtDescr->Enable(flag);
  EdtDescr->Enable(flag);
  TxtActual->Enable(flag);
  TxtDisplay->Enable(flag);
  TxtVal->Enable(flag);
  TxtMin->Enable(flag);
  TxtMax->Enable(flag);
  TxtComp->Enable(flag);
  TxtType->Enable(flag);
  TxtProg->Enable(flag);
  TxtDisp->Enable(flag);
  EdtMin->Enable(flag);
  EdtMax->Enable(flag);
  EdtDVal->Enable(flag);
  EdtDMin->Enable(flag);
  EdtDMax->Enable(flag);
  CboProg->Enable(flag);
  CboComp->Enable(flag);
  CboType->Enable(flag);
  CboDisp->Enable(flag);
  TxtNbSteps->Enable(flag);
  EdtNbSteps->Enable(flag);
  TxtTableName->Enable(flag);
  EdtTableName->Enable(flag);
  EdtComboLabelToAdd->Enable(flag);
  EdtComboValueToAdd->Enable(flag);
  ComboTable->Enable(flag);
  AddToCombo->Enable(flag);
  RemoveFromCombo->Enable(flag);
  DefaultCombo->Enable(flag);
  EnableParam->Enable(flag);

#ifdef MP_DEBUG
  fprintf(stderr, "MP_DBG: MPPreProjectDialog: Leaving EnableControls\n");
#endif
  return 0;
}

int CMPPreProjectDialog::UpdateControls(TParameterItem *prop){
#ifdef MP_DEBUG
  fprintf(stderr, "MP_DBG: MPPreProjectDialog: Entering UpdateControls\n");
#endif

  wxString tmp;
  wxString type;
  wxString prog;
  wxString comp;
  char s[20];
  int i;

  actual_prop = prop;

  EdtTableName->Clear();
  ComboTable->DeleteAllItems();

  EnableParam->SetValue(actual_prop->Enabled);
  if (!actual_prop->Enabled){
    EnableControls(false);
    EnableParam->Enable(true);
    BtnStore->Enable(false);
    return 0;
  }
  EnableParam->Enable(true);
  BtnStore->Enable(true);

  //page_param->TxtAssign->Enable(true);

  if (actual_prop->InGroup!=0){
    EnableControls(false);
    BtnStore->Enable(true);
    EnableParam->Enable(true);
    BtnStore->Enable(false);
    page_param->TxtAssign->SetValue(true);
    page_param->GroupAssign->Enable(true);
    page_param->GroupAssign->SetSelection(page_param->GroupAssign->FindString(actual_prop->GroupName->ParameterName));
    return 0;
  }    

  tmp << "Properties for parameter " << actual_prop->ParameterName;
  TxtIntro->SetLabel(tmp);
  
  page_param->TxtAssign->SetValue(false);
  page_param->GroupAssign->Enable(false);

  EdtDescr->SetValue(actual_prop->ParameterDescription);

  switch (actual_prop->ProcessType){
  case AUTO_SLIDER_LIN:  comp="auto";  type="slider"; prog="lin"; break;
  case AUTO_SLIDER_LOG:  comp="auto";  type="slider"; prog="log"; break;
  case AUTO_SLIDER_EXP:  comp="auto";  type="slider"; prog="exo"; break;
  case TABLE_SLIDER_LIN: comp="table"; type="slider"; prog="lin"; break;
  case TABLE_SLIDER_LOG: comp="table"; type="slider"; prog="log"; break;
  case TABLE_SLIDER_EXP: comp="table"; type="slider"; prog="exp"; break;
  case TABLE_COMBO:      comp="table"; type="combo";  prog="lin"; break;
  };
  CboComp->SetSelection(CboComp->FindString(comp));
  CboType->SetSelection(CboType->FindString(type));
  CboProg->SetSelection(CboProg->FindString(prog));

  if (actual_prop->ProcessType!=TABLE_COMBO){
    switch (actual_prop->DisplayType){
    case DISPLAY_INT:   comp="int";   break;
    case DISPLAY_FLOAT: comp="float"; break;
    };
    CboDisp->SetValue(comp);

    sprintf(s, "%d", actual_prop->NumberOfSteps);
    tmp = s;
    EdtNbSteps->SetValue(tmp);

    sprintf(s, "%f", actual_prop->DisplayInitialValue);
    tmp = s;
    EdtDVal->SetValue(tmp);
    sprintf(s, "%f", actual_prop->DisplayMinValue);
    tmp = s;
    EdtDMin->SetValue(tmp);
    sprintf(s, "%f", actual_prop->DisplayMaxValue);
    tmp = s;
    EdtDMax->SetValue(tmp);
 
    if ((actual_prop->ProcessType==AUTO_SLIDER_LIN) || (actual_prop->ProcessType==AUTO_SLIDER_LOG) || (actual_prop->ProcessType==AUTO_SLIDER_EXP)){
      actual_prop->NumberOfSubParams = 1;
      sprintf(s, "%f", actual_prop->ParamMinValue[0]);
      tmp = s;
      EdtMin->SetValue(tmp);
      sprintf(s, "%f", actual_prop->ParamMaxValue[0]);
      tmp = s;
      EdtMax->SetValue(tmp);
    } else {
      EdtTableName->SetValue(actual_prop->TableFileName[0]);
    }
  } else {
    wxString val;
    char tmp[80];
    wxListItem li;
    li.m_mask = wxLIST_MASK_TEXT;
    for (i=0;i<actual_prop->NumberOfSteps;i++){
      li.m_itemId = i;
      li.m_col = 0;
      li.m_text = actual_prop->ComboLabels[i];
      ComboTable->InsertItem(li);
      sprintf(tmp, "%d", actual_prop->ParamTable[0][i]);
      val = tmp;
      ComboTable->SetItem(ComboTable->GetItemCount()-1, 1, val);
    }
    // set default value
    SetComboDefault();    
  }

  wxCommandEvent evt;
  OnCompChanged(evt);
  OnTypeChanged(evt);

#ifdef MP_DEBUG
  fprintf(stderr, "MP_DBG: MPPreProjectDialog: Leaving UpdateControls\n");
#endif
  return 0;
}

// handle double click mouse events
void CMPPreProjectDialog::OnSelChanged(wxCommandEvent& event){
#ifdef MP_DEBUG
  fprintf(stderr, "MP_DBG: MPPreProjectDialog: Entering OnSelChanged\n");
#endif

  if (event.m_id != ID_PARAMLIST){
    event.Skip();
    return;
  }

  wxTreeItemId SelId = tree->GetSelection();
  TglModuleList *ml;
  TParameterItem *il;

  if (!SelId){
    return;
  }

  if (tree->ItemHasChildren(SelId)){
    EnableControls(false);
    event.Skip();
    return;
  }
  
  ml = mods;
  while (ml!=NULL){
    il = ml->Mod->mpparam;
    while (il!=NULL){
      if (il->id == SelId){
	UpdateControls(il);
	return;
      }
      il = il->next;
    };
    ml = (TglModuleList*) ml->Next;
  };

#ifdef MP_DEBUG
  fprintf(stderr, "MP_DBG: MPPreProjectDialog: Leaving OnSelChanged\n");
#endif
}

// handle double click mouse events
void CMPPreProjectDialog::OnGroupChanged(wxCommandEvent& event){
#ifdef MP_DEBUG
  fprintf(stderr, "MP_DBG: MPPreProjectDialog: Entering OnGroupChanged\n");
#endif
    
  if (event.m_id != ID_GROUPLIST){
    event.Skip();
    return;
  }

  wxTreeItemId SelId = page_group->group_tree->GetSelection();

  if (!SelId){
    return;
  }

  if (tree->ItemHasChildren(SelId)){
    EnableControls(false);
    event.Skip();
    return;
  }
  
  TParameterItem *it;
  it = groups;

  do{
    if (it->id = SelId){
      UpdateControls(it);
      return;
    }
    it = it->next;
  } while (it!=NULL);

  wxMessageBox("Consistency Problem:\nSelected group not registered\n\nPlease contact Axoris", "Error");
#ifdef MP_DEBUG
  fprintf(stderr, "MP_DBG: MPPreProjectDialog: Leaving OnGroupChanged\n");
#endif
}

void CMPPreProjectDialog::OnEnable(wxCommandEvent& event){
#ifdef MP_DEBUG
  fprintf(stderr, "MP_DBG: MPPreProjectDialog: Entering OnEnable\n");
#endif
  
  if (event.m_id != ID_ENABLE){
    event.Skip();
    return;
  }

  actual_prop->Enabled = EnableParam->GetValue();
  UpdateControls(actual_prop);
#ifdef MP_DEBUG
  fprintf(stderr, "MP_DBG: MPPreProjectDialog: Leaving OnEnable\n");
#endif
}

void CMPPreProjectDialog::OnAssignGroup(wxCommandEvent& event){
#ifdef MP_DEBUG
  fprintf(stderr, "MP_DBG: MPPreProjectDialog: Entering OnAssignGroup\n");
#endif

  if (event.m_id != ID_GROUPTXT){
    event.Skip();
    return;
  }

  bool on = page_param->TxtAssign->GetValue();
  TParameterItem *g;
  int i;

  if (on){ // want to assign a group
    // search in the list of groups if  there is available place
    bool found=false;
    bool detect;
    wxString gname;
    int n;
    
    i=0;
    n=page_param->GroupAssign->GetCount();
    while ((!found) && (i<n)){
      gname=page_param->GroupAssign->GetString(i);
      // get the associated TParameterItem from groups
      detect=false;
      g = groups;
      while ((g!=NULL) && (!detect)){
	if (strcmp(g->ParameterName, gname.c_str())==0)
	  detect=true;
	else
	  g=g->next;
      };

      // check that group has been detected
      if (!detect){
	wxMessageBox("Consistency error:\n one of the groups listed seems unregistered\n\nPlease contact Axoris", "Error");
	return;
      }
      
      // check that some place is available
      if (g->NumberOfSubParams<5)
	found=true;
      else
	i++;
    };

    // deal with the case where no group has been found
    if (!found){
      wxMessageBox("No available group.\nThis means either that there are no defined group \nor that all the groups already have 5 parameters", "Error");
      page_param->TxtAssign->SetValue(false);
      return;
    }

    // pre-select the given group
    page_param->GroupAssign->SetValue(page_param->GroupAssign->GetString(i));
    actual_prop->InGroup=g->NumberOfSubParams+1;
    actual_prop->GroupName=g;
    g->SubParams[g->NumberOfSubParams]=actual_prop;
    g->NumberOfSubParams++;

    UpdateControls(actual_prop);

  } else { // want to recover the standalone mode
    // reassign all param in the group list
    g = actual_prop->GroupName;
    for (i=actual_prop->InGroup;i<g->NumberOfSubParams;i++){
      g->SubParams[i]->InGroup--;
      g->SubParams[i-1]=g->SubParams[i];
    }
    g->NumberOfSubParams--;

    // remove assignment to the group
    actual_prop->GroupName = NULL;
    actual_prop->InGroup = 0;
  }

#ifdef MP_DEBUG
  fprintf(stderr, "MP_DBG: MPPreProjectDialog: Leaving OnAssignGroup\n");
#endif
}

void CMPPreProjectDialog::OnChangeAssignGroup(wxCommandEvent& event){
#ifdef MP_DEBUG
  fprintf(stderr, "MP_DBG: MPPreProjectDialog: Entering OnChangeAssignGroup\n");
#endif

  fprintf(stdout, "In assig\n");

  if (event.m_id != ID_GROUPASSIG){
    event.Skip();
    return;
  }

  wxString gname;
  TParameterItem *g;
  TParameterItem *g_old;
  int i;

  fprintf(stdout, "Get Selection\n");
  // retrieve the selection
  if (page_param->GroupAssign->GetSelection()==-1){
    return;
  }
  gname = page_param->GroupAssign->GetStringSelection();

  fprintf(stdout, "Search in list\n");
  // search the name in the group list
  g = groups;
  while ((g!=NULL) && (strcmp(g->ParameterName, gname.c_str())!=0)) g = g->next;

  if (g==NULL){
    wxMessageBox("Consistency error:\n one of the groups listed seems unregistered\n\nPlease contact Axoris", "Error");
    return;
  }
  
  fprintf(stdout, "Compare\n");
  // if the selected group is the same as the previous one, do nothing
  if (strcmp(g->ParameterName, actual_prop->GroupName->ParameterName)==0){
    return;
  }

  fprintf(stdout, "Check place\n");
  // check that there remain place in the selected group
  if (g->NumberOfSubParams==5){
    //wxMessageBox("The selected group already has the maximal number of\nparameters allowed.", "Error");
    //UpdateControls(actual_prop);
    page_param->GroupAssign->SetSelection(page_param->GroupAssign->FindString(actual_prop->GroupName->ParameterName));
    //event.Skip();
    return;
  }

  fprintf(stdout, "Remove old\n");
  // reassign all param in the previously selected group
  g_old = actual_prop->GroupName;
  for (i=actual_prop->InGroup;i<g_old->NumberOfSubParams;i++){
    g_old->SubParams[i]->InGroup--;
    g_old->SubParams[i-1]=g->SubParams[i];
  }
  g_old->NumberOfSubParams--;

  fprintf(stdout, "Reassign\n");
  // reassign the parameter to the new group
  actual_prop->InGroup=g->NumberOfSubParams+1;
  actual_prop->GroupName=g;
  g->SubParams[g->NumberOfSubParams]=actual_prop;
  g->NumberOfSubParams++;

  UpdateControls(actual_prop);

#ifdef MP_DEBUG
  fprintf(stderr, "MP_DBG: MPPreProjectDialog: Leaving OnChangeAssignGroup\n");
#endif
}

void CMPPreProjectDialog::OnCompChanged(wxCommandEvent &WXUNUSED(event)){
#ifdef MP_DEBUG
  fprintf(stderr, "MP_DBG: MPPreProjectDialog: Entering OnCompChanged\n");
#endif

  wxString tmp;
  tmp = CboComp->GetValue();

  EnableControls(true);

  if (strcmp(tmp.c_str(), "auto")==0){
    // one must set the type to "slider"
    CboType->SetValue("slider");
    CboType->Enable(false);
    TxtTableName->Enable(false);
    EdtTableName->Enable(false);
  } else if (strcmp(tmp.c_str(), "table")==0){
    EdtNbSteps->SetValue("128");
    EdtNbSteps->Enable(false);
    EdtMin->Enable(false);
    EdtMax->Enable(false);
    TxtActual->Enable(false);
  }

  EdtComboLabelToAdd->Enable(false);
  EdtComboValueToAdd->Enable(false);
  ComboTable->Enable(false);
  AddToCombo->Enable(false);
  RemoveFromCombo->Enable(false);
  DefaultCombo->Enable(false);

#ifdef MP_DEBUG
  fprintf(stderr, "MP_DBG: MPPreProjectDialog: Leaving OnCompChanged\n");
#endif
}

void CMPPreProjectDialog::OnTypeChanged(wxCommandEvent &WXUNUSED(event)){
#ifdef MP_DEBUG
  fprintf(stderr, "MP_DBG: MPPreProjectDialog: Entering OnTypeChanged\n");
#endif

  wxString tmp;
  bool tabflag = true;
  tmp = CboComp->GetValue();
  if (strcmp(tmp.c_str(), "table")!=0){
    return;
  }

  tmp = CboType->GetValue();

  if (strcmp(tmp.c_str(), "slider")==0){ 
    tabflag=true;
  } else if (strcmp(tmp.c_str(), "combo")==0){
    tabflag=false;
  } else {
    EnableControls(true);
    return;
  }

  TxtProg->Enable(tabflag);
  CboProg->Enable(tabflag);
  TxtDisp->Enable(tabflag);
  CboDisp->Enable(tabflag);
  TxtNbSteps->Enable(tabflag);
  TxtTableName->Enable(tabflag);
  EdtTableName->Enable(tabflag);
  TxtDisplay->Enable(tabflag);
  TxtVal->Enable(tabflag);
  TxtMin->Enable(tabflag);
  TxtMax->Enable(tabflag);
  EdtDVal->Enable(tabflag);
  EdtDMin->Enable(tabflag);
  EdtDMax->Enable(tabflag);
  EdtComboLabelToAdd->Enable(!tabflag);
  EdtComboValueToAdd->Enable(!tabflag);
  ComboTable->Enable(!tabflag);
  AddToCombo->Enable(!tabflag);
  RemoveFromCombo->Enable(!tabflag);
  DefaultCombo->Enable(!tabflag);

#ifdef MP_DEBUG
  fprintf(stderr, "MP_DBG: MPPreProjectDialog: Entering OnTypeChanged\n");
#endif
};

void CMPPreProjectDialog::OnAddToCombo(wxCommandEvent &WXUNUSED(event)){
#ifdef MP_DEBUG
  fprintf(stderr, "MP_DBG: MPPreProjectDialog: Entering OnAddToCombo\n");
#endif

  wxString lab, val;
  wxString tmp;
  int value;
  char valtmp[80];
  long item;

  lab = EdtComboLabelToAdd->GetValue();
  val = EdtComboValueToAdd->GetValue();

  // check if some data is given
  if ((lab.IsEmpty()) || (val.IsEmpty()))
    return;

  // check if the label is not already present
  item = -1;
  do{
    item = ComboTable->GetNextItem(item);
    tmp = ComboTable->GetItemText(item);
    if (strcmp(tmp.c_str(), lab.c_str())==0) return;
  } while (item!=-1);

  // save data
  wxListItem li;
  li.m_mask = wxLIST_MASK_TEXT;
  li.m_itemId = ComboTable->GetItemCount();
  li.m_col = 0;
  li.m_text = lab;
  ComboTable->InsertItem(li);
  sscanf(val.c_str(), "%d", &value);
  sprintf(valtmp, "%d", value);
  ComboTable->SetItem(ComboTable->GetItemCount()-1, 1, valtmp);

#ifdef MP_DEBUG
  fprintf(stderr, "MP_DBG: MPPreProjectDialog: Leaving OnAddToCombo\n");
#endif
}

void CMPPreProjectDialog::OnRemoveCombo(wxCommandEvent &event){
#ifdef MP_DEBUG
  fprintf(stderr, "MP_DBG: MPPreProjectDialog: Entering OnRemoveCombo\n");
#endif

  long item = -1;

  // retreive the selected item
  item = ComboTable->GetNextItem(item, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED);
  if (item == -1) return;

  // remove it
  ComboTable->DeleteItem(item);
}

int CMPPreProjectDialog::SetComboDefault(bool state){
  wxColour colour;
  
  if (state) 
    colour.Set(255,255,127);
  else
    colour.Set(255,255,255);
  
  wxListItem li;
  li.m_mask = wxLIST_MASK_TEXT;
  li.m_itemId = actual_prop->DefaultValue;
  li.m_col = 0;
  ComboTable->GetItem(li);
  li.SetBackgroundColour(colour);
  ComboTable->SetItem(li);
  li.m_col = 1;
  ComboTable->GetItem(li);
  li.SetBackgroundColour(colour);
  ComboTable->SetItem(li);

#ifdef MP_DEBUG
  fprintf(stderr, "MP_DBG: MPPreProjectDialog: Leaving OnAddToCombo\n");
#endif

  return 0;
}

void CMPPreProjectDialog::OnDefaultCombo(wxCommandEvent &event){
#ifdef MP_DEBUG
  fprintf(stderr, "MP_DBG: MPPreProjectDialog: Entering OnDefaultCombo\n");
#endif

  long item = -1;
  wxListItem li;

  // retreive the selected item
  item = ComboTable->GetNextItem(item, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED);
  if (item == -1) return;

  // remove first defaut value
  if (actual_prop->DefaultValue!=-1){
    SetComboDefault(false);
  }

  actual_prop->DefaultValue=item;
  // set it as default
  SetComboDefault(true);

#ifdef MP_DEBUG
  fprintf(stderr, "MP_DBG: MPPreProjectDialog: Leaving OnDefaultCombo\n");
#endif
}

void CMPPreProjectDialog::OnStore(wxCommandEvent &WXUNUSED(event)){
#ifdef MP_DEBUG
  fprintf(stderr, "MP_DBG: MPPreProjectDialog: Entering OnStore\n");
#endif

  wxString tmp;
  long l;
  int n,i;
  wxListItem li;

  strcpy(actual_prop->ParameterDescription, EdtDescr->GetValue().c_str());

  tmp = EdtDVal->GetValue();
  tmp.ToDouble(&(actual_prop->DisplayInitialValue));

  tmp = EdtDMin->GetValue();
  tmp.ToDouble(&(actual_prop->DisplayMinValue));

  tmp = EdtDMax->GetValue();
  tmp.ToDouble(&(actual_prop->DisplayMaxValue));

  tmp = EdtMin->GetValue();
  tmp.ToDouble(&(actual_prop->ParamMinValue[0]));

  tmp = EdtMax->GetValue();
  tmp.ToDouble(&(actual_prop->ParamMaxValue[0]));

  tmp = CboDisp->GetStringSelection();
  if (strcmp(tmp.c_str(),"int")==0)
    actual_prop->DisplayType = DISPLAY_INT;
  else
    actual_prop->DisplayType = DISPLAY_FLOAT;

  tmp = EdtNbSteps->GetValue();
  tmp.ToLong(&l);
  if (l<2) l=20;
  actual_prop->NumberOfSteps = (int) l;

  strcpy(actual_prop->TableFileName[0], EdtTableName->GetValue().c_str());

  tmp = CboComp->GetStringSelection();
  if (strcmp(tmp.c_str(),"auto")==0){
    // Type must be slider ! We expect it is without test !
    tmp = CboProg->GetStringSelection();
    if (strcmp(tmp.c_str(),"log")==0)
      actual_prop->ProcessType=AUTO_SLIDER_LOG;
    else if (strcmp(tmp.c_str(),"exp")==0)
      actual_prop->ProcessType=AUTO_SLIDER_EXP;
    else
      actual_prop->ProcessType=AUTO_SLIDER_LIN;
  } else {
    // table type ... slider or combo
    tmp = CboType->GetStringSelection();
    if (strcmp(tmp.c_str(),"slider")==0){
      tmp = CboProg->GetStringSelection();
      if (strcmp(tmp.c_str(),"log")==0)
	actual_prop->ProcessType=TABLE_SLIDER_LOG;
      else if (strcmp(tmp.c_str(),"exp")==0)
	actual_prop->ProcessType=TABLE_SLIDER_EXP;
      else
	actual_prop->ProcessType=TABLE_SLIDER_LIN;
    } else     
      actual_prop->ProcessType=TABLE_COMBO;
  }

  // retreive the combo values
  if (actual_prop->ParamTable[0]!=NULL){
    delete[] actual_prop->ParamTable[0];
    actual_prop->ParamTable[0]=NULL;
  }
  if (actual_prop->ComboLabels!=NULL){
    delete[] actual_prop->ComboLabels;
    actual_prop->ComboLabels=NULL;
  }

  n=ComboTable->GetItemCount();
  if (n!=0){
    actual_prop->NumberOfSteps=n;
    actual_prop->ParamTable[0]=new int[n];
    actual_prop->ComboLabels=new TComboLabel[n];

    for (i=0;i<n;i++){
      li.m_mask = wxLIST_MASK_TEXT;
      li.m_itemId = i;
      li.m_col = 0;
      ComboTable->GetItem(li);
      tmp=li.GetText();
      strcpy(actual_prop->ComboLabels[i],tmp.c_str());

      li.m_itemId = i;
      li.m_col = 1;
      ComboTable->GetItem(li);
      tmp = li.GetText();
      sscanf(tmp.c_str(), "%d", &(actual_prop->ParamTable[0][i]));
    }
  }

#ifdef MP_DEBUG
  fprintf(stderr, "MP_DBG: MPPreProjectDialog: Leaving OnStore\n");
#endif
}

////////////////////////////////////////
// function used to load a project
////////////////////////////////////////

int LoadMPProject(char *fname, TglModuleList *mods){
#ifdef MP_DEBUG
  fprintf(stderr, "MP_DBG: Entering LoadMPProject\n");
#endif

  TProjectParameters *pl, *pl_tmp;
  TParameterItem *il;
  TParameterItem *ip;
  TglModuleList *ml;
  bool match;
  int i;
  wxFileName fn(fname);
  char prjname[512];
  
  fn.SetExt("prj");
  strcpy(prjname, fn.GetFullPath());
  if (!wxFileExists(prjname)){
    //fprintf(stdout, "No project file found\n");
    //wxLogMessage("No project file found\n");
    return 1;
  }

  // load the project into a Parameter list 
  pl = NULL;
  if (LoadProject(prjname, &pl)!=0){
    //fprintf(stdout, "Wrong project format\n");
    //wxLogMessage("Wrong project format\n");
    return 1;
  }

  // assign the parameter 
  // we browse the parameters one by one and try to find if they correspond
  // to a module of our structure.
  pl_tmp = pl;
  while (pl_tmp!=NULL){
    match = false;
    ml = mods;
    while ((!match) && (ml!=NULL)){
      if (strcmp(ml->Mod->Label, pl_tmp->ModName)==0) match = true;
      if (!match) ml = (TglModuleList*) ml->Next;
    };
    if (match){
      // we have found a valid module, we now need to find matching properties ...
      ip = ml->Mod->mpparam;
      while (ip!=NULL){
	match = false;
	il = pl_tmp->param;
	while ((!match) && (il!=NULL)){
	  if (strcmp(il->ParameterName, ip->ParameterName)==0) match = true;
	  if (!match) il = il->next;
	};
	if (match){
	  // got a matching parameter, 
	  //store the config in the module parameter list
	  strcpy(ip->ParameterDescription, il->ParameterDescription);
	  ip->ProcessType = il->ProcessType;
	  ip->DisplayType = il->DisplayType;
	  ip->NumberOfSteps = il->NumberOfSteps;
	  ip->DisplayMinValue = il->DisplayMinValue;
	  ip->DisplayMaxValue = il->DisplayMaxValue;
	  ip->DisplayInitialValue = il->DisplayInitialValue;
	  ip->NumberOfSubParams = il->NumberOfSubParams;
	  ip->Enabled = true;
	  ip->DefaultValue=il->DefaultValue;
	  if (il->ComboLabels!=NULL){
	    ip->ComboLabels=new TComboLabel[il->NumberOfSteps];
	    memcpy(ip->ComboLabels, il->ComboLabels, il->NumberOfSteps*sizeof(TComboLabel));
	  }
	  for (i=0;i<il->NumberOfSubParams;i++){
	    ip->ParamMinValue[i] = il->ParamMinValue[i];
	    ip->ParamMaxValue[i] = il->ParamMaxValue[i];
	    ip->ParamValue[i] = il->ParamValue[i];
	    //  int *ParamTable[MAXSUBPARAM];
	    ip->ParamAddress[i] = il->ParamAddress[i]-pl_tmp->AddressOffset;
	    strcpy(ip->TableFileName[i], il->TableFileName[i]);
	    if (il->ParamTable[i]!=NULL){
	      ip->ParamTable[i]=new int[il->NumberOfSteps];
	      memcpy(ip->ParamTable[i], il->ParamTable[i], il->NumberOfSteps*sizeof(int));
	    }
	  }
	}
	// go to the next module param
	ip=ip->next;
      }
    }
    pl_tmp = pl_tmp->next;
  };

  // delete the param list
  while (pl!=NULL){
    pl_tmp = pl->next;
    DeleteParamList(&(pl->param));
    delete pl;
    pl = pl_tmp;
  };

#ifdef MP_DEBUG
  fprintf(stderr, "MP_DBG: Leaving LoadMPProject\n");
#endif
  return 0;
}

////////////////////////////////////////
// function used to buils the project
////////////////////////////////////////

int BuildMPProject(char *fname, char *name, TglModuleList *mods, bool map){
#ifdef MP_DEBUG
  fprintf(stderr, "MP_DBG: Entering BuildMPProject\n");
#endif

  char objname[255];
  char mapname[255];
  char offname[255];
  char line[512];
  char tmp[40];
  int len, nbmods, idx, i, j;
  int *offsets;
  FILE *offile;
  TglModuleList *ml;
  TProjectParameters *pl, *pl_tmp;

  // build the object, offset and map file names
  strcpy(objname,fname);
  len = strlen(objname);
  if (objname[len-4]=='.'){
    objname[len-3]='o';
    objname[len-2]='b';
    objname[len-1]='j';
  } else
    strcat(objname, ".obj");
  len = strlen(objname);

  strcpy(mapname, objname);
  mapname[len-3]='m';
  mapname[len-2]='a';
  mapname[len-1]='p';

  strcpy(offname, objname);
  offname[len-3]='o';
  offname[len-2]='f';
  offname[len-1]='f';

  // count the number of modules
  nbmods=0;
  ml = mods;
  while (ml!=NULL){
    ml = (TglModuleList*) ml->Next;
    nbmods++;
  };

  // read the offset table
  offsets = new int[nbmods];  
  for (i=0;i<nbmods;i++) offsets[i]=0;

  offile = fopen(offname, "r");
  if (offile==NULL)
    return 1;

  for (i=0;i<nbmods-2;i++){
    fgets(line, 511, offile);
    len=strlen(line);
    while ((line[len-1]=='\n') || (line[len-1]=='\r')){
      line[len-1] = '\0';
      len--;
    };
    if (len==0) continue;
    j=0;
    while (line[j]!=':') j++;
    strncpy(tmp, line, j);
    tmp[j]='\0';
    sscanf(tmp, "%d", &idx);
    // search the second and the third ':'
    j++;    
    while (line[j]!=':') j++;
    j++;    
    while (line[j]!=':') j++;
    j++;    
    strncpy(tmp, line+j, len-j+1);
    sscanf(tmp, "%d", &(offsets[idx-1]));
  }
  fclose(offile);
  
  // build the parameter list
  pl = NULL;
  ml = mods;
  while (ml!=NULL){
    if ((strcmp(ml->Mod->Name, "Input")!=0) && (strcmp(ml->Mod->Name, "Output")!=0)){
      pl_tmp = new TProjectParameters;
      pl_tmp->next = pl;
      pl = pl_tmp;
      strcpy(pl->ModName, ml->Mod->Label);
      strcpy(pl->ModDescr, ml->Mod->Name);
      pl->AddressOffset = offsets[ml->Index-1];
      pl->param = ml->Mod->mpparam;
    }
    ml = (TglModuleList*) ml->Next;
  };

  // really write the project
  WriteProject(objname, name, map ? mapname : NULL, pl);

  // release memory:
  // 1. offset table
  delete[] offsets;
  // 2. parameter list
  while (pl!=NULL){
    pl_tmp = pl->next;
    delete pl;
    pl = pl_tmp;
  };

#ifdef MP_DEBUG
  fprintf(stderr, "MP_DBG: Leaving BuildMPProject\n");
#endif

  return 0;
}
