/////////////////////////////////////////////////////////////////////////////
// Name:        theme.cpp
// Project:     GUI for Axoris AL3101 application generator
// Author:      Gregor(y) Soyez
// Version:     1.1
// Created:     30/12/03
// Copyright:   (c) Axoris
/////////////////////////////////////////////////////////////////////////////

#include "theme.h"
#include "Config.h"
#include <wx/bitmap.h>
#include <wx/font.h>

// ID's
#define ID_STATIC  1
#define ID_THEMES  2
#define ID_BTN_PV  3
#define ID_AUTO_PV 4
#define ID_PREVIEW 5

#define THEME_FILE_PATTERN "*.agt"

int LoadTheme(wxString Name, wxString Path, TTheme *t){ 
  FILE *flux;
  char tmp[255];

  strcpy(t->Name,  Name.c_str());

  sprintf(tmp,"%s/%s.agt", Path.c_str(), t->Name);
  flux=fopen(tmp, "r");
  if (flux==NULL){
    wxLogMessage("Theme not found, trying the default one ...");
    sprintf(tmp,"%s/default.agt", Path.c_str());
    flux=fopen(tmp, "r");
    if (flux==NULL){
      wxLogMessage("Default theme not found, Check your configuration or your installation.");
      return 1;
    }
  }

  fscanf(flux, "%s", t->TexMod);
  fscanf(flux, "%s", t->TexPin);
  fscanf(flux, "%d %d %d", &(t->NameColor[0]), &(t->NameColor[1]), &(t->NameColor[2]));
  fscanf(flux, "%d %d %d", &(t->LinkColor[0]), &(t->LinkColor[1]), &(t->LinkColor[2]));

  fclose(flux);

  return 0;
}

int GetThemeFromUser(CConfig *cfg, TTheme *t){
  wxString s=wxFileSelector("Choose one theme", cfg->SkinDir, "", "", "*.agt", wxOPEN);
  if (!(s))
    return 1;

  // we need to split the string in Path + ThemeName
  char str[512];
  int n, len;

  sprintf(str, "%s", s.c_str());
  len = strlen(str);
  n=len-1;
  while ((n>=0) && (str[n]!='/') && (str[n]!='\\')) n--;

  if (n>=0){
    delete cfg->SkinDir;
    cfg->SkinDir=new char[n+1];
    strncpy(cfg->SkinDir,str,n);
    cfg->SkinDir[n]='\0';
  }
  
  if (len-n-5>=0){
    delete cfg->SkinName;
    cfg->SkinName=new char[len-n-4];
    strncpy(cfg->SkinName,str+n+1,len-n-5);
    cfg->SkinName[len-n-5]='\0';
  }

  return LoadTheme(cfg->SkinName,cfg->SkinDir,t);
}

//-----------------------------------------------------
// CThemeSelector implementation
//-----------------------------------------------------

BEGIN_EVENT_TABLE(CThemePreview, wxScrolledWindow)
  EVT_PAINT(CThemePreview::OnPaint)
END_EVENT_TABLE()

CThemePreview::CThemePreview(TTheme *i_theme, wxWindow* parent, wxWindowID id, const wxPoint& pos, const wxSize& size)
  : wxScrolledWindow(parent, id, pos, size){
  theme = i_theme;
  IsThemeSelected=0;
}

void CThemePreview::OnPaint(wxPaintEvent& event){
  wxPaintDC dc(this); 
  PrepareDC(dc);
  Render(&dc);
}

void CThemePreview::Redraw(){
  // update the bitmaps
  char tmp[255];

  sprintf(tmp, "../resources/AgALag/textures/%s", theme->TexMod);
  mod.LoadFile(tmp, wxBITMAP_TYPE_BMP);
  mod = mod.Rescale(120, 30);

  sprintf(tmp, "../resources/AgALag/textures/%s", theme->TexPin);
  pin.LoadFile(tmp, wxBITMAP_TYPE_BMP);
  pin = pin.Rescale(15, 15);

  IsThemeSelected = 1;

  Refresh();
}

void CThemePreview::Render(wxDC *dc){
  int i, j;

  // clear dc
  SetBackgroundColour(wxT("BLACK"));
  dc->SetBackground(wxBrush(wxColour(0, 0, 0), wxSOLID));
  dc->SetBackgroundMode(wxTRANSPARENT);
  dc->Clear();
  
  // draws the grid
  //  wxColour PtColour(255, 255, 0);
  dc->SetPen(wxPen(wxColour(255,255,0), 1, wxSOLID));
  for (i=0;i<=13;i++)
    for (j=0;j<=16;j++){
      dc->DrawPoint(i*15, j*15);
    }

  dc->SetTextForeground(wxColour(theme->NameColor[0],
				 theme->NameColor[1],
				 theme->NameColor[2]));
  dc->SetFont(wxFont(16, wxROMAN, wxNORMAL, wxNORMAL));

  if (!IsThemeSelected) return;

  // draw Input
  dc->DrawBitmap(mod.ConvertToBitmap(),15,15);
  for (i=0;i<8;i++){
    dc->DrawBitmap(pin.ConvertToBitmap(),15*(1+i),45);
  }
  dc->DrawText("Input", 20, 18);

  // draw Output
  dc->DrawBitmap(mod.ConvertToBitmap(),15,195);
  for (i=0;i<8;i++){
    dc->DrawBitmap((pin.Mirror(FALSE)).ConvertToBitmap(),15*(1+i),180);
  }
  dc->DrawText("Output", 20, 198);

  // draw module
  dc->DrawBitmap(mod.ConvertToBitmap(),15,105);
  for (i=0;i<3;i++){
    dc->DrawBitmap((pin.Mirror(FALSE)).ConvertToBitmap(),15*(1+i),90);
  }
  for (i=0;i<3;i++){
    dc->DrawBitmap(pin.ConvertToBitmap(),15*(1+i),135);
  }

  // draw links 
  dc->SetPen(wxPen(wxColour(theme->LinkColor[0],
			    theme->LinkColor[1],
			    theme->LinkColor[2]), 
		   2, wxSOLID));
  
  dc->DrawLine(23,52,23,98);
  dc->DrawLine(38,52,53,98);
  dc->DrawLine(23,142,23,188);
  dc->DrawLine(53,142,38,188);

}

//-----------------------------------------------------
// CThemeSelector implementation
//-----------------------------------------------------

BEGIN_EVENT_TABLE(CThemeSelector, wxDialog)
  EVT_CHECKBOX(ID_AUTO_PV, CThemeSelector::OnAutoPreview)
  EVT_LISTBOX(ID_THEMES, CThemeSelector::OnThemeSelected)
  EVT_BUTTON(ID_BTN_PV, CThemeSelector::OnPreview)
  EVT_BUTTON(wxID_OK, CThemeSelector::OnOK)
END_EVENT_TABLE()

CThemeSelector::CThemeSelector(wxWindow* parent, wxWindowID id)
 : wxDialog(parent, id, "Theme selector", wxDefaultPosition, wxSize(250,170), wxDEFAULT_DIALOG_STYLE){
#ifdef DEBUG
  fprintf(stderr, "DBG: theme selector: Entering constructor\n");
#endif
  SetClientSize(425,330);

  TxtIntro = new wxStaticText(this, ID_STATIC, "Select your theme", wxPoint( 15, 10), wxSize(300, 20));

  ThemeList = new wxListBox(this, ID_THEMES, wxPoint(15,40), wxSize(170, 205));
  BtnPreview = new wxButton(this, ID_BTN_PV, "Preview", wxPoint(15,255), wxSize(90, 25));
  AutoPreview = new wxCheckBox(this, ID_AUTO_PV, "Auto", wxPoint(120,255), wxSize(70, 25));
  AutoPreview->SetValue(false);

  Preview = new CThemePreview(&NewTheme, this, ID_PREVIEW, wxPoint(210,40), wxSize(195,240));

  BtnOk = new wxButton(this, wxID_OK, "Ok", wxPoint(90,295), wxSize(75, 25));
  BtnCancel = new wxButton(this, wxID_CANCEL, "Cancel", wxPoint(260,295), wxSize(75, 25));

  ListThemes();
}

CThemeSelector::~CThemeSelector(){
#ifdef DEBUG
  fprintf(stderr, "DBG: theme selector: Entering destructor\n");
#endif
}

int CThemeSelector::ListThemes(){
#ifdef DEBUG
  fprintf(stderr, "DBG: theme selector: Entering ListThemes\n");
#endif

  CConfig *cfg;
  cfg=new CConfig;
  wxDir dir(cfg->SkinDir);
  wxString filename;
  bool cont;

  // enumerates files in directory
  cont = dir.GetFirst(&filename, THEME_FILE_PATTERN);
  while (cont){
    filename.Remove(filename.Len()-4);
    ThemeList->Append(filename);
    cont = dir.GetNext(&filename);
  };

  delete cfg;

  return 0;
}

int CThemeSelector::FillTheme(TTheme *t){
  int nsel=ThemeList->GetSelection();
  int rc;
  CConfig *cfg;

  if ((nsel<0) || (nsel>=ThemeList->GetCount()))
    return 1;

  sprintf(t->Name, "%s", ThemeList->GetString(nsel).c_str());

  cfg=new CConfig;
  rc=LoadTheme(t->Name, cfg->SkinDir, t);
  delete cfg;

  return rc;
}

void CThemeSelector::OnOK(wxCommandEvent &event){
 
  if (FillTheme(&NewTheme)!=0){
    wxMessageBox("No valid theme selected.", "Error", wxOK|wxICON_ERROR);
    return;
  } else {
    CConfig *cfg;
    cfg=new CConfig;
    delete cfg->SkinName;
    cfg->SkinName=new char[strlen(NewTheme.Name)+1];
    strcpy(cfg->SkinName, NewTheme.Name);
    cfg->SkinName[strlen(NewTheme.Name)]='\0';
    delete cfg;
  }

  event.Skip();
}

void CThemeSelector::OnPreview(wxCommandEvent &event){
  wxBitmap bmp;
  
  if (FillTheme(&NewTheme)!=0){
    wxMessageBox("No valid theme selected.", "Error", wxOK|wxICON_ERROR);
    return;
  }

  Preview->Redraw();
}

void CThemeSelector::OnAutoPreview(wxCommandEvent &event){
  IsAutoPreview = AutoPreview->IsChecked();
  BtnPreview->Enable(!IsAutoPreview);
  OnPreview(event);
}

void CThemeSelector::OnThemeSelected(wxCommandEvent &event){
  if (IsAutoPreview){
    OnPreview(event);
  }

  event.Skip();
}
