//--------------------------------------------------------------------//
//  AgALag : Axoris group AL3101 application generator                //
//                                                                    //
//                 _         _    _      _                            //
//                / \____   / \  | |    / \____                       //
//               / _ \  _\ / _ \ | |   / _ \  _\                      //
//              / /_\ \  __ /_\ \| |  / /_\ \  __                     //
//             / _____ \ \ \____ \ |_/______ \ \ \                    //
//            /_/     \_\___\   \_\____|    \_\___\                   //
//                                                                    //
//  File: iomapping.cpp                                               //
//  Description: Manager for temporary IO addresses between modules   //
//  Version: 1.0                                                      //
//  Author: Gregor                                                    //
//  e-mail: Gregor.Soyez@swing.be                                     //
//                                                                    //
//  Copyright (C) 2002 Axoris                                         //
//--------------------------------------------------------------------//

#include "iomapping.h"

int _MappingOffset;
int *_MappingUsage;
int _MappingMax, _NbMapped;

// initialize temporary io mapping
int InitIOMapping(int i_offset){
  int i;

  _MappingOffset = i_offset;
  _MappingMax = 1023 - _MappingOffset;
  _MappingUsage = new int[_MappingMax];
  _NbMapped = 0;

  for (i=0;i<_MappingMax;i++) _MappingUsage[i]=0;

  return IO_MAPPING_OK;
}

int FinishIOMapping(){
  delete[] _MappingUsage;

  return IO_MAPPING_OK;
}


// retreive a temporary free address
// return IO_MAPPING_OK if an address
// can be returned through the parameter 
// and IO_MAPPING_FAILED if there is no more 
// address free.
// see header file for details concerning request_at_end
int QueryIO(int *_io, bool request_at_end){
  int i=0;
  
  if (request_at_end){
    i=_NbMapped;
  }

  while ((i<_MappingMax) && (_MappingUsage[i])) i++;

  if (i==_MappingMax)
    return IO_MAPPING_FAILED;

  _MappingUsage[i]=1;
  if (i==_NbMapped) _NbMapped++;

  (*_io)=i;
  return IO_MAPPING_OK;
}

// free an address. Returns OK if the address
// was used and FAILED otherwise.
int FreeIO(int _io){
  if (!_MappingUsage[_io])
    return IO_MAPPING_FAILED;

  _MappingUsage[_io]=0;

  return IO_MAPPING_OK;
}

// retreive the number of current mappings
int GetNbMapped(){
  return _NbMapped;
}

// write the mappings in the final file
int WriteMappingDefs(FILE *flux){
  int i;

  // writes a small comment to start
  fprintf(flux, ";; temporary IO for modules intercommunications\n");
  
  // write preasm definitions
  for (i=0;i<_NbMapped;i++){
    fprintf(flux, "ABS\tTMP%X\t0x%X\n", i, _MappingOffset+i);
  }

  return IO_MAPPING_OK;
}
