//--------------------------------------------------------------------//
//  AgALag : Axoris group AL3101 application generator                //
//                                                                    //
//                 _         _    _      _                            //
//                / \____   / \  | |    / \____                       //
//               / _ \  _\ / _ \ | |   / _ \  _\                      //
//              / /_\ \  __ /_\ \| |  / /_\ \  __                     //
//             / _____ \ \ \____ \ |_/______ \ \ \                    //
//            /_/     \_\___\   \_\____|    \_\___\                   //
//                                                                    //
//  File: iomapping.h                                                 //
//  Description: anager for temporary IO addresses between modules    //
//  Version: 1.0                                                      //
//  Author: Gregor                                                    //
//  e-mail: Gregor.Soyez@swing.be                                     //
//                                                                    //
//  Copyright (C) 2002 Axoris                                         //
//--------------------------------------------------------------------//

#ifndef __PROJ_AL_IO_MAPPING_H__
#define __PROJ_AL_IO_MAPPING_H__

#include "ProjAL.h"
#include <stdio.h>

#define IO_MAPPING_OK     0
#define IO_MAPPING_FAILED 1

// initialize and finish temporary io mapping
int InitIOMapping(int i_offset);
int FinishIOMapping();

// retreive a temporary free address
// return IO_MAPPING_OK if an address
// can be returned through the parameter 
// and IO_MAPPING_FAILED if there is no more 
// address free.
// if request_at_end is true, we force a new temp_io (not reuse a freed one)
// this feature is used in feedback lines because freeing process cannot
// take into account back linking.
int QueryIO(int*, bool request_at_end=false);

// free an address. Returns OK if the address
// was used and FAILED otherwise.
int FreeIO(int);

// get the current number of mappings
int GetNbMapped();

// write the pre-asm definitions in the final file
int WriteMappingDefs(FILE *flux);

#endif
