#ifndef __AL3101_PREASM_H__
#define __AL3101_PREASM_H__

#define PREASM_OK       0   
#define PREASM_MAX_CST  1
#define PREASM_SYNTAX   2
#define PREASM_FORMAT   3
#define PREASM_EXISTS   4
#define PREASM_NOTHING  5

#define CST_EQU   0
#define CST_ABS   1
#define CST_LABEL 2

// structure definition for Preassembler EQU and ABS commands
typedef struct{
  char *Name;       // name of the defined constant
  char *StrValue;   // string as it appears in the .asm file
  int Type;         // CST_ABS or CST_EQU
} PAsmCst;

// srtucture definition for preassembler labels
typedef struct{
  char *Name;       // Name of the label (without :)
  int CmdIndex;     // index of the next command line after the label definition
} PAsmLabel;

#ifndef EXTERN_PREASM_DEFINED
// Preassembler variables.
extern PAsmCst *AL3101PreAsmCstTable;      // ABS and EQU tables
extern PAsmLabel *AL3101PreAsmLabelTable;  // labels table
extern int AL3101PreAsmNbCst;              // Number of predefined constants
extern int AL3101PreAsmNbLabel;            // number of labels
#endif

// Initialize the preassembler
// This add the preasm constants always defined by default
// It returns PREASM_OK when successful.
int InitPreAsm();

// Preassemble a command
// This should be called by the main assembler loop before treating
// the command as a assembler one.
// The function returns PREASM_OK if it is a valid preasm command, 
// PREASM_SYNTAX if it is a preasm command with wrong syntax, PREASM_FORMAT
// if the name of the preasm-defined string is not valid, PREASM_EXISTS if
// the preassembler-defined string already exists and PREASM_NOTHING
// if the command doesn't correspond to any preassembler command.
// parameters are 
//  - number of tokens
//  - array of tokens given by parsing
//  - index of the current commande line.
//    it is used in case of label definition and it points to the first line
//    after the label.
int PreAssemble(int, char*[], int);

// update the Preasm PC (increment it by 1)
// it has to be called AFTER each command analysis
// during the second step of assembling
// returns PREASM_OK
int PreAsmStepPC();

// given a string, check if it correspond to a given predefined constant. 
// If it is the case, put the sting corresponding to the searched value 
// into strCst and return 0. Otherwise, return 1.
// Last parameter is the type of constant requested (CST_*)
// Function returns PREASM_OK if value returned successfull and PREASM_NOTHING
// if requested constant doesn't exists.
int PreAsmGetConstant(char*, char*, int);

// Close Preasm
// This function basically release all allocated memory
// returns PREASM_OK
int PreAsmClose();


#endif
