/********************************************************************/
/*                                                                  */
/*                        A X O R I S                               */
/*                                                                  */
/*                Alesis AL3101/2 DSP Simulator                     */
/*                                                                  */
/********************************************************************/
/*                                                                  */
/* Origin Date    : June 2001                                       */
/* Project Version: v0.1                                            */
/* FileName       : Commands.c                                      */
/* File Version   : v0.1                                            */
/* Author         : Fabian Duchesne                                 */
/* Last Modif     : September 2001                                  */
/*                                                                  */
/********************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <math.h>		// for the pow() function in show command

#include "./../MessageMacros.h"
#include "Commands.h"
#include "./../Simul_func/AL3101_Core.h"
#include "./../Simul_func/AL3101_IOs.h" 
#include "./../1K_func/AL3101Asm.h"
#include "./../1K_func/AL3101Disasm.h"
#include "./../1K_func/AL3101Format.h"
#include "./../1K_func/AL3101AsmDisasmUtils.h"
#include "./../Debug.h"
#include "./../Error.h"
#include "./../HelperFunctions.h"

/********************************************************************/
extern AL3101IoMode    AL3101InputModesList[];
extern AL3101IoMode    AL3101OutputModesList[];
extern AL3101IoMode    AL3101SignalModesList[];
extern AL3101IoMode    AL3101TraceModesList[];
extern INT32           NumOfInputModesInList;
extern INT32           NumOfOutputModesInList;
extern INT32           NumOfSignalModesInList;
extern INT32           NumOfTraceModesInList;


/********************************************************************/
/*                  C O M M A N D   L I S T                         */
/********************************************************************/
INT32 NumOfCommandsInList = 64;

AL3101SimCommand AL3101SimCommandList[] = {
	{ "exit"		,4	, CMD_QUIT					, CmdQuit				, "- exits the simulator" },
    { "quit"		,4	, CMD_QUIT					, CmdQuit				, "- exits the simulator" },
	{ "load"		,4	, CMD_LOAD					, CmdLoad				, "filename [complete] - loads an asm, obj or cmd filename (see User manual for details)" },
	{ "help"		,4	, CMD_HELP					, CmdHelp				, "[command] - prints the list of commands or help on a specific command" },
	{ "loadscript"	,10	, CMD_LOADSCRIPT			, CmdLoadScript			, "filename - loads and execute a script filename" },

    { "version"		,7	, CMD_VERSION				, CmdVersion			, "- displays the version of the simulator" },
    { "copyright"	,9	, CMD_COPYRIGHT				, CmdCopyright			, "- displays the copyright information for the simulator" },
    
    { "start"		,5	, CMD_START					, CmdStart				, "[blocking] [addr] - starts [blocking] simulation at given PC address (address = current by default)" },
	{ "go"			,2	, CMD_START					, CmdStart				, "[blocking] [addr] - starts [blocking] simulation at given PC address (address = current by default)" },
    { "step"		,4	, CMD_STEP					, CmdStep				, "[blocking] [n] - steps [blocking] n times (n=1 by default)" },
    { "stop"        ,4	, CMD_STOP					, CmdStop               , "- stops the simulation" },
	{ "reset"		,5	, CMD_RESET					, CmdReset				, "[type] - resets according to [type] (see User manual for details)" },
	
	{ "+input"		,6	, CMD_SET_INPUT				, CmdSetInput			, "sets an input (see User manual for details)"},
	{ "-input"		,6	, CMD_DEL_INPUT				, CmdDelInput			, "index - clears the input given by the index from the input list"},
	{ "-allinput"	,9	, CMD_DEL_ALLINPUT			, CmdDelAllInput		, "clears all the inputs"},
	{ "showinput"	,9	, CMD_SHOW_INPUT			, CmdShowInput			, "show all inputs (input list for -input)"},
	{ "loopinput"	,9	, CMD_LOOP_INPUT			, CmdLoopInput			, "loops on the specified input"},
	{ "+output"		,7	, CMD_SET_OUTPUT			, CmdSetOutput			, "sets an output (see User manual for details)"},
	{ "-output"		,7	, CMD_DEL_OUTPUT			, CmdDelOutput			, "index - clears the output given by the index from the output list"},
	{ "-alloutput"	,10	, CMD_DEL_ALLOUTPUT			, CmdDelAllOutput		, "clears all the outputs"},
	{ "showoutput"	,10	, CMD_SHOW_OUTPUT			, CmdShowOutput			, "show all outputs (output list for -output)"},
	{ "+signal"		,7	, CMD_SET_SIGNAL			, CmdSetSignal			, "sets a signal (see User manual for details)"},
	{ "-signal"		,7	, CMD_DEL_SIGNAL			, CmdDelSignal			, "index - clears the signal given by the index from the signal list"},
	{ "-allsignal"	,10	, CMD_DEL_ALLSIGNAL			, CmdDelAllSignal		, "clears all the signals"},
	{ "showsignal"	,10	, CMD_SHOW_SIGNAL			, CmdShowSignal			, "show all signals (signal list for -signal)"},
	{ "+trace"		,6	, CMD_SET_TRACE				, CmdSetTrace			, "sets a trace (see User manual for details)"},
	{ "-trace"		,6	, CMD_DEL_TRACE				, CmdDelTrace			, "index - clears the trace given by the index from the trace list"},
	{ "-alltrace"	,9	, CMD_DEL_ALLTRACE			, CmdDelAllTrace		, "clears all the traces"},
	{ "showtrace"	,9	, CMD_SHOW_TRACE			, CmdShowTrace			, "show all traces (trace list for -trace)"},
	{ "+tool"		,5	, CMD_SET_TOOL				, CmdSetTool			, "sets a tool (see User manual for details)"},
	{ "-tool"		,5	, CMD_DEL_TOOL				, CmdDelTool			, "index - clears the tool given by the index from the tool list"},
	{ "-alltool"	,8	, CMD_DEL_ALLTOOL			, CmdDelAllTool			, "clears all the tools"},
	{ "showtool"	,8	, CMD_SHOW_TOOL				, CmdShowTool			, "show all tools (tool list for -tool)"},
	
	{ "+break"		,6	, CMD_SET_INSTRBREAKPOINT	, CmdSetInstrBreakPoint	, "[addr] - sets break point at addr (pc by default)" },
	{ "+tempbreak"	,10	, CMD_SET_TEMP_BREAKPOINT	, CmdSetTempBreakPoint	, "HitNumber [addr] - sets a break at address addr (pc by default) to survive HitNumber hits" },
    { "-break"		,6	, CMD_DEL_INSTRBREAKPOINT	, CmdDelInstrBreakPoint	, "[addr] - clears break point at address addr (pc by default)" },
	{ "+valbreak"	,9	, CMD_SET_VALBREAKPOINT		, CmdSetValBreakPoint	, "Value MemType [location] - sets break point on value at address MemType:Location" },
    { "-valbreak"	,9	, CMD_DEL_VALBREAKPOINT		, CmdDelValBreakPoint	, "MemType [location] - clears break point on value at address MemType:Location" },
    { "-allbreak"	,9	, CMD_DEL_ALLBREAKPOINTS	, CmdDelAllBreakPoints	, "- clears all break points" },
    { "showbreak"	,9	, CMD_DISP_BREAKPOINTS		, CmdDisplayBreakPoints , "- shows the break points" },
  
	{ "dasm"		,4	, CMD_DASM_N				, CmdDasmN				, "[addr [m]] [addr [n]] - disassemble from adress m (pc by default) to address n (address m by default)." },
    { "+dasm"		,5	, CMD_DASM_ON				, CmdDasmOn				, "- turns on printing of instructions when stepping into the code." },
    { "-dasm"		,5	, CMD_DASM_OFF				, CmdDasmOff			, "- turns off printing of instructions when stepping into the code." },
	{ "show"		,4	, CMD_SHOW					, CmdShow				, "shows a register or memory element (see User manual for details)" },
	{ "setval"		,6	, CMD_SET_VAL				, CmdSetVal				, "sets the value of a register of memory element (see User manual for details)"},
    
	{ "+watch"		,6	, CMD_SET_WATCH				, CmdSetWatch			, "sets watch on a register or memory element (check manual for more details)" },
    { "-watch"		,6	, CMD_DEL_WATCH				, CmdDelWatch			, "index - clear watch with given index from the watch list" },
    { "-allwatch"	,9	, CMD_DEL_ALLWATCHES		, CmdDelAllWatches		, "clear all watches" },
	{ "showwatch"	,9	, CMD_DISP_WATCHES			, CmdDisplayWatches		, "show all watches (watch list for -watch)" },
        
	{ "+alias"		,6	, CMD_SET_ALIAS				, CmdSetAlias			, "CmdName AliasName - sets AliasName as being an alias for the CmdName command" },
    { "-alias"		,6	, CMD_DEL_ALIAS				, CmdDelAlias			, "index - clears the alias given by the index from the alias list" },
    { "-allalias"	,9	, CMD_DEL_ALLALIASES		, CmdDelAllAliases		, "clears all aliases" },
	{ "showalias"	,9	, CMD_DISP_ALIASES			, CmdDisplayAliases		, "shows all aliases (alias list for -alias)" },
		
	{ "setuser"		,7	, CMD_SET_USERPARAMETER		, CmdSetUserParameter	, "param val - sets the user definable parameter to the given value (see User manual for details)" },

	{ "setstream"	,9	, CMD_SET_STREAM			, CmdSetStream			, "set parameters of the wavedevice stream (see User manual for details)" },
	{ "startstream"	,11	, CMD_START_STREAM			, CmdStartStream		, "start simulation with streaming (see User manual for details)" },
	{ "stopstream"	,10	, CMD_STOP_STREAM			, CmdStopStream			, "stop simulation running with streaming (see User manual for details)" },
	
// commands not implemented yet	
	{ "cycle"		,5	, CMD_DISP_CYCLES			, CmdDisplayCycles		, "- prints number of instruction cycles executed since the last reset." },
    { "1kstate"		,7	, CMD_DISP_DSP_STATE		, CmdDisplayDSPState	, "- displays full information about the state of the DSP." },
	
	{ "registers"	,9	, CMD_DISP_REGISTERS		, CmdDisplayRegisters	, "- shows the contents of registers" },
	{ "progmem"		,7	, CMD_DISP_PROG_MEMORY		, CmdDisplayProgMemory	, "addr n - displays the first n values of program memory at address addr" },
	{ "datamem"		,7	, CMD_DISP_DATA_MEMORY		, CmdDisplayDataMemory	, "addr n - displays the first n values of data memory at address addr" },
// end of commands not implemented yet

// this next command should (has to) stay at the end of the table.
//  otherwise, help command will not work properly anymore!!!
	{ ""			,1	, CMD_NOP					, CmdNop				,  "do nothing" },
};


/********************************************************************/
/*                D S P   C O R E   I N S T A N C E                 */
/********************************************************************/
AL3101 DspInstance;
AL3101* pDspInstance = &DspInstance;
extern const CHAR* PROMPT_1KSM;

/********************************************************************/
/*              F U N C T I O N   P R O T O T Y P E S               */
/********************************************************************/
extern INT32 ExecuteScript(CHAR *ScriptFileName);

/********************************************************************/
/*       H E L P E R    F U N C T I O N   P R O T O T Y P E S       */
/********************************************************************/
INT32 LoadMemoryMap(CHAR *FileName);
INT32 GetAddressAddressNumbers(CHAR *Options, INT32 *FirstAddress, INT32 *Size);
INT32 GetAddressAddressNumber(CHAR *Options, INT32 *FirstAddress);
INT32 GetResetType(CHAR *Options);
INT32 ApplyResetFromType(INT32 ResetType);


/********************************************************************/
/*    C O M M A N D S    F U N C T I O N S    P R O T O T Y P E S   */
/********************************************************************/

/************************************************************/
/* NAME       :												*/
/* DESCRIPTION:												*/
/* INPUT      :												*/
/* OUTPUT     :												*/
/************************************************************/
INT32 CmdQuit(CHAR* Options)
{
	DBG_PRINT("In CmdQuit\n");

	AL3101_Stop(pDspInstance);
	// Clean implemented in main
	//AL3101_Clean(pDspInstance);

	DBG_PRINT("Out CmdQuit\n");

	return 0;
};

/************************************************************/
/* NAME       :												*/
/* DESCRIPTION:												*/
/* INPUT      :												*/
/* OUTPUT     :												*/
/************************************************************/
INT32 CmdNop(CHAR* Options)
{
	return 0;
};

/************************************************************/
/* NAME       :												*/
/* DESCRIPTION:												*/
/* INPUT      :												*/
/* OUTPUT     :												*/
/************************************************************/
INT32 CmdHelp(CHAR* Options)
{
	INT32 ii;
	INT32 Count;
	INT32 index;
	INT32 CommandID;
	CHAR Option[CMD_MAX_LEN];
	
	DBG_PRINT("In CmdHelp\n");

	
	Count = GetNextWord(Option, Options);
	index = GetCommandID(Option, &CommandID);
	if (index >= NumOfCommandsInList)
	{
		OUTPUT_1("Unkown command: %s\n", Option);
	}
	if (index >= (NumOfCommandsInList - 1))
	{
		for ( ii = 0; ii < (NumOfCommandsInList - 1) ; ii++ )
		{
			OUTPUT_1("\nCommand:\n <%s>\n", AL3101SimCommandList[ii].name);
			OUTPUT_1("Usage:\n %s\n", AL3101SimCommandList[ii].help);
		}
	}
	else if (index < (NumOfCommandsInList - 1))
	{
		OUTPUT_1("\nCommand: <%s>\n", AL3101SimCommandList[index].name);
		OUTPUT_1("Usage:\n  %s\n\n", AL3101SimCommandList[index].help);
	}
	

	DBG_PRINT("Out CmdHelp\n");
	
	return 0;
};

/************************************************************/
/* NAME       :												*/
/* DESCRIPTION:												*/
/* INPUT      :												*/
/* OUTPUT     :												*/
/************************************************************/
INT32 CmdVersion(CHAR* Options)
{
	DBG_PRINT("In CmdVersion\n");

	DBG_PRINT("Out CmdVersion\n");
	
	return 0;
};

/************************************************************/
/* NAME       :												*/
/* DESCRIPTION:												*/
/* INPUT      :												*/
/* OUTPUT     :												*/
/************************************************************/
INT32 CmdCopyright(CHAR* Options)
{
	DBG_PRINT("In CmdCopyright\n");

	OUTPUT("AXORIS (c) 2002 Copyright - All rights reserved.\n");

	DBG_PRINT("Out CmdCopyright\n");
	
	return 0;
};

/************************************************************/
/* NAME       :												*/
/* DESCRIPTION:												*/
/* INPUT      :												*/
/* OUTPUT     :												*/
/************************************************************/
INT32 CmdStart(CHAR* Options)
{
	CHAR		TempStr[CMD_MAX_LEN];
	INT32		BlockingMode = 0;
	INT32		StartAddress;

	DBG_PRINT("In CmdStart\n");

	// Get the possible blocking mode from the Options string 
	GetNextWord(TempStr, Options);
	// If option string is "blocking", use blocking mode
	if (strcmp(TempStr, "") != 0)
	{
		if (strcmp(TempStr, "blocking") == 0)
		{
			BlockingMode = 1;
			// Get the possible start address from the Options string 
			GetNextWord(TempStr, Options);
		}
		if (strcmp(TempStr, "") != 0)
		{
			StartAddress = atoi(TempStr);
			if ((StartAddress < 0) || (StartAddress > 1023)) WARNING("Invalid address value: should be between 0 and 1023 included.\n");
			else pDspInstance->State.PC = StartAddress;
		}
	}

	// Launch the simulation
	AL3101_Start(pDspInstance, 0, BlockingMode);

	DBG_PRINT("Out CmdStart\n");
	
	return 0;
};

/************************************************************/
/* NAME       :												*/
/* DESCRIPTION:												*/
/* INPUT      :												*/
/* OUTPUT     :												*/
/************************************************************/
INT32 CmdStop(CHAR* Options)
{
	DBG_PRINT("In CmdStop\n");
	
	AL3101_Stop(pDspInstance);

	DBG_PRINT("Out CmdStop\n");
	
	return 0;
};

/************************************************************/
/* NAME       :												*/
/* DESCRIPTION:												*/
/* INPUT      :												*/
/* OUTPUT     :												*/
/************************************************************/
INT32 CmdStep(CHAR* Options)
{
	CHAR		TempStr[CMD_MAX_LEN];
	INT32		NumberOfSteps;
	INT32		BlockingMode = 0;

	DBG_PRINT("In CmdStep\n");

	// Get the possible blocking mode from the Options string 
	GetNextWord(TempStr, Options);
	// If option string is "blocking", use blocking mode
	if (strcmp(TempStr, "") != 0)
	{
		if (strcmp(TempStr, "blocking") == 0)
		{
			BlockingMode = 1;
			// Get the possible start address from the Options string 
			GetNextWord(TempStr, Options);
		}
		if (strcmp(TempStr, "") != 0) NumberOfSteps = atoi(TempStr);
		else NumberOfSteps = 1;
	}
	else NumberOfSteps = 1;

	// StepIt
	AL3101_Start(pDspInstance, NumberOfSteps);
	
	DBG_PRINT("Out CmdStep\n");
	
	return 0;
};

/************************************************************/
/* NAME       :												*/
/* DESCRIPTION:												*/
/* INPUT      :												*/
/* OUTPUT     :												*/
/************************************************************/
INT32 CmdReset(CHAR* Options)
{
//	CHAR		TempStr[CMD_MAX_LEN];
	INT32		ResetType;

	DBG_PRINT("In CmdReset\n");

	ResetType = GetResetType(Options);
	ApplyResetFromType(ResetType);

/*	// Check first what should be reset
	// Get the reset type
	GetNextWord(TempStr, Options);
	String_ToLower_String(TempStr, TempStr);

	// Check if the IOs should be reset
	if (strcmp(TempStr, "io") == 0)
	{
		AL3101_Reset_IO(pDspInstance, 0);
	}
	// Check if the IOs should be reset and reinitialized again.
	else if (strcmp(TempStr, "io_reopen") == 0)
	{
		AL3101_Reset_IO(pDspInstance, 1);
	}
	// Check if the program memory and breakpoints should be reset.
	else if (strcmp(TempStr, "prog") == 0)
	{
		AL3101_Reset_Prog(pDspInstance);
		AL3101_Reset_Breakpoints(pDspInstance);
	}
	// Check if the program memory only should be reset.
	else if (strcmp(TempStr, "prog_only") == 0)
	{
		AL3101_Reset_Prog(pDspInstance);
	}
	// Check if the data memory and watches should be reset.
	else if (strcmp(TempStr, "data") == 0)
	{
		AL3101_Reset_Data(pDspInstance);
		AL3101_Reset_Watches(pDspInstance);
	}
	// Check if the data memory only should be reset.
	else if (strcmp(TempStr, "data_only") == 0)
	{
		AL3101_Reset_Data(pDspInstance);
	}
	// Check if a hardware reset should take place
	//		Meaning Program and data memory.
	else if (strcmp(TempStr, "dsp") == 0)
	{
		AL3101_Reset_Prog(pDspInstance);
		AL3101_Reset_Data(pDspInstance);
	}
	// Check if everything should be reset and IOs should be reinitialized
	else if (strcmp(TempStr, "all_reopen") == 0)
	{
		AL3101_Reset_IO(pDspInstance, 1);
		AL3101_Reset_Prog(pDspInstance);
		AL3101_Reset_Data(pDspInstance);
		AL3101_Reset_Breakpoints(pDspInstance);
		AL3101_Reset_Watches(pDspInstance);
	}
	// Check if everything should be reset (program, data memories and IOs).
	else //if (strcmp(TempStr, "all") == 0)		// this is the default case
	{
		AL3101_Reset_IO(pDspInstance, 0);
		AL3101_Reset_Prog(pDspInstance);
		AL3101_Reset_Data(pDspInstance);
		AL3101_Reset_Breakpoints(pDspInstance);
		AL3101_Reset_Watches(pDspInstance);
	}

	AL3101_Reset_State(pDspInstance);
*/

	DBG_PRINT("Out CmdReset\n");
	
	return 0;
};

/************************************************************/
/* NAME       :												*/
/* DESCRIPTION:												*/
/* INPUT      :												*/
/* OUTPUT     :												*/
/************************************************************/
INT32 CmdLoad(CHAR *Options)
{
	CHAR		TempStr[CMD_MAX_LEN];
	CHAR		ProgFileName[CMD_MAX_LEN];
	INT32		NbLines;
	INT32		index;
	INT32		LoadingAsmObj = 0;
	
	DBG_PRINT("In CmdLoad\n");

	// Get the file Name from the Options string 
	GetNextFilenameWord(ProgFileName, Options);
	if (!strcmp(ProgFileName, ""))
	{
		ERROR("A file name must be provided\n");
		return -1;
	};
	for(index = strlen(ProgFileName); index >= 0; index--) TempStr[index] = tolower(ProgFileName[index]);

	CHAR *pCharTemp = strrchr(TempStr, '.');
	if (pCharTemp == NULL)
	{
		ERROR("Unable to determine wether an assembly or disassembly is needed.\n");
		ERROR("Please use a file name with the .asm or .obj extension.\n");
		return -1;
	}
	pCharTemp++;
	if (strcmp(pCharTemp, "asm") == 0)			// The input file is an assembly file
	{
		INFO("Assembling ...\n");
		NbLines=Assemble(ProgFileName, pDspInstance->CurrentProgram);
	
		if (NbLines==ASM_FILE_NOT_EXISTS)
		{
			ERROR(".asm file does not exists. Giving up.\n");
			return -1;
		}
		else if (NbLines==ASM_READ_ERROR)
		{
			ERROR("Errors encountered while reading .asm file. Giving up.\n");
			return -1;
		}
		else if (NbLines==ASM_ERROR)
		{
			ERROR("Errors encountered during assembling. Giving up.\n");
			return -1;
		}
		else
		{
			INFO_1("... Sucess assembling : %d lines\n", NbLines);
		}
		LoadingAsmObj = 1;
	}
	else if (strcmp(pCharTemp, "obj") == 0)	// The input file is an object file
	{
		INFO("Disassembling ...\n");
		NbLines=Disassemble(ProgFileName, "1Kdisasm.asm", pDspInstance->CurrentProgram);

		if (NbLines==DISASM_FILE_NOT_EXISTS)
		{
			ERROR(".obj file does not exists. Giving up.\n");
			return -1;
		}
		else if (NbLines==DISASM_READ_ERROR)
		{
			ERROR("Errors encountered while reading .obj file. Giving up.\n");
			return -1;
		}
		else if (NbLines==DISASM_ERRORS)
		{
			ERROR("Errors encountered during disassembling. Giving up.\n");
			return -1;
		}
		else
		{
			INFO_1("... Sucess disassembling: %d lines\n", NbLines);
		}
		LoadingAsmObj = 2;
	}
	else if (strcmp(pCharTemp, "cmd") == 0)	// The input file is a command script file
	{
		INFO("Executing script ...\n");
		ExecuteScript(ProgFileName);
	}
	else
	{
		ERROR("Unable to determine wether an assembly or disassembly is needed.\n");
		ERROR("Please use a file name with the .asm or .obj extension.\n");
		return -1;
	}

	// We should initialize memory only on load asm or load obj.
	// For script, it does not make sense
	if (LoadingAsmObj != 0)
	{
		GetNextWord(TempStr, Options);
		if (strcmp(TempStr, "unchanged") != 0)		// The rest of the program memory should be cleared if "unchanged" specified
		{
			for (index = NbLines; index < 1024; index++)
			{
				// Setting of these lines is absolutely necessary
				pDspInstance->CurrentProgram[index].op = 0;			// 1st opcode is used as default one
				pDspInstance->CurrentProgram[index].ln = -1;		// line number in the assembler file
				pDspInstance->CurrentProgram[index].s_index = 0;	// index in the AL3101Table (for simulation call)
				pDspInstance->CurrentProgram[index].lObj = 0;		// Object code (binary): bits  0 to 31 
				pDspInstance->CurrentProgram[index].wObj = 0;		//                       bits 32 to 40
				strcpy(pDspInstance->CurrentProgram[index].CmdStr, "CM 0.0 0x00  ; Added by the simulator."); // string with the assembler code.
				// These lines are added just to clear everything when needed
				pDspInstance->CurrentProgram[index].m = 0;			// memory adress
				pDspInstance->CurrentProgram[index].c = 0;			// constant (aligned to S3.24 by default)
				pDspInstance->CurrentProgram[index].d = 0;			// constant (aligned to S3.24 by default)
			}
		}
	
		// Setting the NumberOfLines to the correct value depending on the "truecyle" user parameter
		if (pDspInstance->Config.TrueCycleSimulation == 0) pDspInstance->State.NumberOfLines = NbLines;
		else  pDspInstance->State.NumberOfLines = 1024;

		//AL3101_Init(&DspInstance);	// line commented because it reinitialize everything which is not that wanted
										// problem being that memory is not reset.

		// Load the memory map file
		pCharTemp = strrchr(ProgFileName, '.');
		if (pCharTemp == NULL)
		{
			ERROR("Unable to determine wether an assembly or disassembly is needed.\n");
			ERROR("Please use a file name with the .asm or .obj extension.\n");
			return -1;
		}
		pCharTemp++;
		strcpy(pCharTemp, "map");
		LoadMemoryMap(ProgFileName);

	}

	DBG_PRINT("Out CmdLoad\n");
	return 0;
};


/************************************************************/
/* NAME       :												*/
/* DESCRIPTION:												*/
/* INPUT      :												*/
/* OUTPUT     :												*/
/************************************************************/
INT32 CmdLoadScript(CHAR *Options)
{
	CHAR		ScriptFileName[CMD_MAX_LEN];
	
	DBG_PRINT("In CmdLoadScript\n");

	// Get the file Name from the Options string 
	GetNextFilenameWord(ScriptFileName, Options);
	if (!strcmp(ScriptFileName, ""))
	{
		ERROR("A file name must be provided\n");
		return -1;
	};

	// execute the script
	ExecuteScript(ScriptFileName);

	DBG_PRINT("Out CmdLoadScript\n");
	return 0;
};

/************************************************************/
/* NAME       :												*/
/* DESCRIPTION:												*/
/* INPUT      :												*/
/* OUTPUT     :												*/
/************************************************************/
INT32 DelIO(IOTable *IO, INT32 IO_index, INT32 *NumberOfIO)
{
	INT32 index;
	INT32 ErrorCode;

	DBG_PRINT("In DelIO\n");

	if (IO_index < *NumberOfIO)
	{
		// Clean the IO given by the index
		ErrorCode = (IO[IO_index].f_Clean)(IO[IO_index].pInstance);

		// Move the rest of the list one step
		for (index = IO_index; index < (*NumberOfIO - 1); index++)
		{
			IO[index].pInstance = IO[index + 1].pInstance;
			IO[index].f_Action = IO[index + 1].f_Action;
			IO[index].f_Clean = IO[index + 1].f_Clean;
			IO[index].f_GetPosition = IO[index + 1].f_GetPosition;
			IO[index].f_GetLength = IO[index + 1].f_GetLength;
			strcpy(IO[index].CmdLine, IO[index + 1].CmdLine);
		}
				
		// We have one IO less now
		*NumberOfIO = (*NumberOfIO) - 1;
	}
	else
	{
		OUTPUT_2("Index provided: %d exceeds limit: %d\n", IO_index, ((*NumberOfIO) - 1));
	}
	

	DBG_PRINT("Out DelIO\n");
	
	return 0;
}

/************************************************************/
/* NAME       :												*/
/* DESCRIPTION:												*/
/* INPUT      :												*/
/* OUTPUT     :												*/
/************************************************************/
INT32 DelAllIO(IOTable *IO, INT32 *NumberOfIO)
{
	INT32 ErrorCode;
	INT32 index;

	DBG_PRINT("In DelAllIO\n");

	// Clean all the IO
	for (index = 0; index < (*NumberOfIO); index++)
	{
		ErrorCode = (IO[index].f_Clean)(IO[index].pInstance);
	}

	// We don't have IO anymore
	*NumberOfIO = 0;

	DBG_PRINT("Out DelAllIO\n");

	return 0;
}

/************************************************************/
/* NAME       :												*/
/* DESCRIPTION:												*/
/* INPUT      :												*/
/* OUTPUT     :												*/
/************************************************************/
INT32 CmdSetInput(CHAR* Options)
{
	CHAR Mode[CMD_MAX_LEN];
	CHAR DummyString[CMD_MAX_LEN];
	INT32 index, ModeIndex = -1;

	DBG_PRINT("In CmdSetInput\n");

	// Check if the maximum number of input instances has been reached
	if (pDspInstance->NumberOfInputInstances == pDspInstance->Config.MaxNumberOfInputs)
	{
		ERROR("Maximum number of inputs reached: command not taken in account.\nPlease change the upper limit with the setuser command.\n");

		DBG_PRINT("Out CmdSetInput\n");
	
		return -1;
	}

	// Process the command
	strcpy(DummyString, Options);
	GetNextWord(Mode, DummyString);
	if (!strcmp(Mode, ""))
	{
		ERROR("An Input type must be specified\n");
		return 1;	
	};
	
	for (index = 0; index < NumOfInputModesInList; index++)
	{
		if (strcmp(AL3101InputModesList[index].name, Mode) == 0) ModeIndex = index;
	}
	
	if (ModeIndex == -1)
	{
		ERROR_1("Input type not known: %s\n", Mode);
		DBG_PRINT("Out CmdSetInput\n");
		return -1;
	}
	DBG_PRINT("Out CmdSetInput\n");

	return(AL3101InputModesList[ModeIndex].Init)(pDspInstance, Options, pDspInstance->IO_Inputs, &pDspInstance->NumberOfInputInstances);
};

/************************************************************/
/* NAME       :												*/
/* DESCRIPTION:												*/
/* INPUT      :												*/
/* OUTPUT     :												*/
/************************************************************/
INT32 CmdDelInput(CHAR* Options)
{
	CHAR TempStr[CMD_MAX_LEN];
	INT32 IO_index = -1;
	INT32 ErrorCode;

	DBG_PRINT("In CmdDelInput\n");

	// Process the command
	GetNextWord(TempStr, Options);
	IO_index = atoi(TempStr);

	ErrorCode = DelIO(pDspInstance->IO_Inputs, IO_index, &(pDspInstance->NumberOfInputInstances));
	

	DBG_PRINT("Out CmdDelInput\n");
	
	return ErrorCode;
};

/************************************************************/
/* NAME       :												*/
/* DESCRIPTION:												*/
/* INPUT      :												*/
/* OUTPUT     :												*/
/************************************************************/
INT32 CmdDelAllInput(CHAR* Options)
{
	INT32 ErrorCode;

	DBG_PRINT("In CmdDelAllInput\n");

	// Process the command
	ErrorCode = DelAllIO(pDspInstance->IO_Inputs, &(pDspInstance->NumberOfInputInstances));

	DBG_PRINT("Out CmdDelAllInput\n");
	
	return ErrorCode;
};

/************************************************************/
/* NAME       :												*/
/* DESCRIPTION:												*/
/* INPUT      :												*/
/* OUTPUT     :												*/
/************************************************************/
INT32 CmdShowInput(CHAR* Options)
{
	INT32 index;

	DBG_PRINT("In CmdShowInput\n");

	// Process the command

	for (index = 0; index < pDspInstance->NumberOfInputInstances; index++)
	{
		OUTPUT_2("Input #%i command: +input%s", index, pDspInstance->IO_Inputs[index].CmdLine);
	}
	

	DBG_PRINT("Out CmdShowInput\n");
	
	return 0;
};

/************************************************************/
/* NAME       :												*/
/* DESCRIPTION:												*/
/* INPUT      :												*/
/* OUTPUT     :												*/
/************************************************************/
INT32 CmdLoopInput(CHAR* Options)
{
	CHAR TempStr[CMD_MAX_LEN];
	INT32 IO_index = -1;
	INT32 LoopingMode = 1;

	DBG_PRINT("In CmdLoopInput\n");

	// Process the command
	GetNextWord(TempStr, Options);
	IO_index = atoi(TempStr);

	// Check that the index used is OK
	if (IO_index < 0)
	{
		OUTPUT_1("Index provided should be positive: %d\n", IO_index);
	}
	if (IO_index > pDspInstance->NumberOfInputInstances)
	{
		OUTPUT_2("Index provided: %d exceeds limit: %d\n", IO_index, (pDspInstance->NumberOfInputInstances - 1));
	}

	// "lower" the option string
	String_ToLower_String(Options, Options);
	// Get the looping mode
	GetNextWord(TempStr, Options);
	if (strcmp(TempStr, "off") == 0) LoopingMode = 0;

	// set looping mode now
	pDspInstance->IO_Inputs[IO_index].f_SetLooping(pDspInstance->IO_Inputs[IO_index].pInstance, LoopingMode);
	
	DBG_PRINT("Out CmdLoopInput\n");
	
	return 0;
};

/************************************************************/
/* NAME       :												*/
/* DESCRIPTION:												*/
/* INPUT      :												*/
/* OUTPUT     :												*/
/************************************************************/
INT32 CmdSetOutput(CHAR* Options)
{
	CHAR Mode[CMD_MAX_LEN];
	CHAR DummyString[CMD_MAX_LEN];
	INT32 index, ModeIndex = -1;

	DBG_PRINT("In CmdSetOutput\n");
	
	// Check if the maximum number of output instances has been reached
	if (pDspInstance->NumberOfOutputInstances == pDspInstance->Config.MaxNumberOfOutputs)
	{
		ERROR("Maximum number of outputs reached: command not taken in account.\nPlease change the upper limit with the setuser command.\n");

		DBG_PRINT("Out CmdSetOutput\n");
	
		return -1;
	}

	// Process the command
	strcpy(DummyString, Options);
	GetNextWord(Mode, DummyString);
	if (!strcmp(Mode, ""))
	{
		ERROR("An Output type must be specified\n");
		return 1;	
	};

	for (index = 0; index < NumOfOutputModesInList; index++)
	{
		if (strcmp(AL3101OutputModesList[index].name, Mode) == 0) ModeIndex = index;
	}
	
	if (ModeIndex == -1)
	{
		ERROR_1("Output type not known: %s\n", Mode);
		DBG_PRINT("Out CmdSetOutput\n");
		return -1;
	}
	DBG_PRINT("Out CmdSetOutput\n");

	return (AL3101OutputModesList[ModeIndex].Init)(pDspInstance, Options, pDspInstance->IO_Outputs, &pDspInstance->NumberOfOutputInstances);
};

/************************************************************/
/* NAME       :												*/
/* DESCRIPTION:												*/
/* INPUT      :												*/
/* OUTPUT     :												*/
/************************************************************/
INT32 CmdDelOutput(CHAR* Options)
{
	CHAR TempStr[CMD_MAX_LEN];
	INT32 IO_index = -1;
	INT32 ErrorCode;

	DBG_PRINT("In CmdDelOutput\n");

	// Process the command
	GetNextWord(TempStr, Options);
	IO_index = atoi(TempStr);

	ErrorCode = DelIO(pDspInstance->IO_Outputs, IO_index, &(pDspInstance->NumberOfOutputInstances));

	DBG_PRINT("Out CmdDelOutput\n");
	
	return ErrorCode;
};

/************************************************************/
/* NAME       :												*/
/* DESCRIPTION:												*/
/* INPUT      :												*/
/* OUTPUT     :												*/
/************************************************************/
INT32 CmdDelAllOutput(CHAR* Options)
{
	INT32 ErrorCode;

	DBG_PRINT("In CmdDelAllOutput\n");

	// Process the command
	ErrorCode = DelAllIO(pDspInstance->IO_Outputs, &(pDspInstance->NumberOfOutputInstances));

	DBG_PRINT("Out CmdDelAllOutput\n");
	
	return ErrorCode;
};

/************************************************************/
/* NAME       :												*/
/* DESCRIPTION:												*/
/* INPUT      :												*/
/* OUTPUT     :												*/
/************************************************************/
INT32 CmdShowOutput(CHAR* Options)
{
	INT32 index;

	DBG_PRINT("In CmdShowOutput\n");

	// Process the command
	for (index = 0; index < pDspInstance->NumberOfOutputInstances; index++)
	{
		OUTPUT_2("Output #%i command: +output%s", index, pDspInstance->IO_Outputs[index].CmdLine);
	}
	
	DBG_PRINT("Out CmdShowOutput\n");
	
	return 0;
};

/************************************************************/
/* NAME       :												*/
/* DESCRIPTION:												*/
/* INPUT      :												*/
/* OUTPUT     :												*/
/************************************************************/
INT32 CmdSetSignal(CHAR* Options)
{
	CHAR Mode[CMD_MAX_LEN];
	CHAR DummyString[CMD_MAX_LEN];
	INT32 index, ModeIndex = -1;

	DBG_PRINT("In CmdSetSignal\n");

	// Check if the maximum number of signal instances has been reached
	if (pDspInstance->NumberOfSignalInstances == pDspInstance->Config.MaxNumberOfSignals)
	{
		ERROR("Maximum number of signals reached: command not taken in account.\nPlease change the upper limit with the setuser command.\n");

		DBG_PRINT("Out CmdSetSignals\n");
	
		return -1;
	}

	// Process the command
	strcpy(DummyString, Options);
	GetNextWord(Mode, DummyString);
	if (!strcmp(Mode, ""))
	{
		ERROR("An Signal type must be specified\n");
		return 1;	
	};
	
	for (index = 0; index < NumOfSignalModesInList; index++)
	{
		if (strcmp(AL3101SignalModesList[index].name, Mode) == 0) ModeIndex = index;
	}
	
	if (ModeIndex == -1)
	{
		ERROR_1("Signal type not known: %s\n", Mode);
		DBG_PRINT("Out CmdSetSignal\n");
		return -1;
	}
	DBG_PRINT("Out CmdSetSignal\n");

	return (AL3101SignalModesList[ModeIndex].Init)(pDspInstance, Options, pDspInstance->Signals, &pDspInstance->NumberOfSignalInstances);
};

/************************************************************/
/* NAME       :												*/
/* DESCRIPTION:												*/
/* INPUT      :												*/
/* OUTPUT     :												*/
/************************************************************/
INT32 CmdDelSignal(CHAR* Options)
{
	CHAR TempStr[CMD_MAX_LEN];
	INT32 IO_index = -1;
	INT32 ErrorCode;

	DBG_PRINT("In CmdDelSignal\n");

	// Process the command
	GetNextWord(TempStr, Options);
	IO_index = atoi(TempStr);

	ErrorCode = DelIO(pDspInstance->Signals, IO_index, &(pDspInstance->NumberOfSignalInstances));
	
	DBG_PRINT("Out CmdDelSignal\n");
	
	return ErrorCode;
};

/************************************************************/
/* NAME       :												*/
/* DESCRIPTION:												*/
/* INPUT      :												*/
/* OUTPUT     :												*/
/************************************************************/
INT32 CmdDelAllSignal(CHAR* Options)
{
	INT32 ErrorCode;

	DBG_PRINT("In CmdDelAllSignal\n");

	// Process the command
	ErrorCode = DelAllIO(pDspInstance->Signals, &(pDspInstance->NumberOfSignalInstances));

	DBG_PRINT("Out CmdDelAllSignal\n");
	
	return ErrorCode;
};

/************************************************************/
/* NAME       :												*/
/* DESCRIPTION:												*/
/* INPUT      :												*/
/* OUTPUT     :												*/
/************************************************************/
INT32 CmdShowSignal(CHAR* Options)
{
	INT32 index;

	DBG_PRINT("In CmdShowSignal\n");

	// Process the command
	for (index = 0; index < pDspInstance->NumberOfSignalInstances; index++)
	{
		OUTPUT_2("Signal #%i command: +signal%s", index, pDspInstance->Signals[index].CmdLine);
	}	

	DBG_PRINT("Out CmdShowSignal\n");
	
	return 0;
};

/************************************************************/
/* NAME       :												*/
/* DESCRIPTION:												*/
/* INPUT      :												*/
/* OUTPUT     :												*/
/************************************************************/
INT32 CmdSetTrace(CHAR* Options)
{
	CHAR Mode[CMD_MAX_LEN];
	CHAR DummyString[CMD_MAX_LEN];
	INT32 index, ModeIndex = -1;

	DBG_PRINT("In CmdSetTrace\n");
	
	// Check if the maximum number of trace instances has been reached
	if (pDspInstance->NumberOfTraceInstances == pDspInstance->Config.MaxNumberOfTraces)
	{
		ERROR("Maximum number of traces reached: command not taken in account.\nPlease change the upper limit with the setuser command.\n");

		DBG_PRINT("Out CmdSetTrace\n");
	
		return -1;
	}

	// Process the command
	strcpy(DummyString, Options);
	GetNextWord(Mode, DummyString);
	if (!strcmp(Mode, ""))
	{
		ERROR("An Trace type must be specified\n");
		return 1;	
	};

	for (index = 0; index < NumOfTraceModesInList; index++)
	{
		if (strcmp(AL3101TraceModesList[index].name, Mode) == 0) ModeIndex = index;
	}
	
	if (ModeIndex == -1)
	{
		ERROR_1("Trace type not known: %s\n", Mode);
		DBG_PRINT("Out CmdSetTrace\n");
		return -1;
	}
	DBG_PRINT("Out CmdSetTrace\n");

	return (AL3101TraceModesList[ModeIndex].Init)(pDspInstance, Options, pDspInstance->Traces, &pDspInstance->NumberOfTraceInstances);
};

/************************************************************/
/* NAME       :												*/
/* DESCRIPTION:												*/
/* INPUT      :												*/
/* OUTPUT     :												*/
/************************************************************/
INT32 CmdDelTrace(CHAR* Options)
{
	CHAR TempStr[CMD_MAX_LEN];
	INT32 IO_index = -1;
	INT32 ErrorCode;

	DBG_PRINT("In CmdDelTrace\n");

	// Process the command
	GetNextWord(TempStr, Options);
	IO_index = atoi(TempStr);

	ErrorCode = DelIO(pDspInstance->Traces, IO_index, &(pDspInstance->NumberOfTraceInstances));
	
	DBG_PRINT("Out CmdDelTrace\n");
	
	return ErrorCode;
};

/************************************************************/
/* NAME       :												*/
/* DESCRIPTION:												*/
/* INPUT      :												*/
/* OUTPUT     :												*/
/************************************************************/
INT32 CmdDelAllTrace(CHAR* Options)
{
	INT32 ErrorCode;

	DBG_PRINT("In CmdDelAllTrace\n");

	// Process the command
	ErrorCode = DelAllIO(pDspInstance->Traces, &(pDspInstance->NumberOfTraceInstances));

	DBG_PRINT("Out CmdDelAllTrace\n");
	
	return ErrorCode;
};

/************************************************************/
/* NAME       :												*/
/* DESCRIPTION:												*/
/* INPUT      :												*/
/* OUTPUT     :												*/
/************************************************************/
INT32 CmdShowTrace(CHAR* Options)
{
	INT32 index;

	DBG_PRINT("In CmdShowTrace\n");

	// Process the command
	for (index = 0; index < pDspInstance->NumberOfTraceInstances; index++)
	{
		OUTPUT_2("Trace #%i command: +trace%s", index, pDspInstance->Traces[index].CmdLine);
	}
	
	DBG_PRINT("Out CmdShowTrace\n");
	
	return 0;
};

/************************************************************/
/* NAME       :												*/
/* DESCRIPTION:												*/
/* INPUT      :												*/
/* OUTPUT     :												*/
/************************************************************/
INT32 CmdSetTool(CHAR* Options)
{
	DBG_PRINT("In CmdSetTool\n");
	
	// Check if the maximum number of tool instances has been reached
	if (pDspInstance->NumberOfToolInstances == pDspInstance->Config.MaxNumberOfTools)
	{
		ERROR("Maximum number of tools reached: command not taken in account.\nPlease change the upper limit with the setuser command.\n");

		DBG_PRINT("Out CmdSetTool\n");
	
		return -1;
	}

	// Process the command
	DBG_PRINT("Out CmdSetTool\n");

	return Tool_Init(pDspInstance, Options);
};

/************************************************************/
/* NAME       :												*/
/* DESCRIPTION:												*/
/* INPUT      :												*/
/* OUTPUT     :												*/
/************************************************************/
INT32 CmdDelTool(CHAR* Options)
{
	CHAR TempStr[CMD_MAX_LEN];
	INT32 index, IO_index = -1;
	INT32 ErrorCode;

	DBG_PRINT("In CmdDelTool\n");

	// Process the command
	GetNextWord(TempStr, Options);
	IO_index = atoi(TempStr);

	if (IO_index < pDspInstance->NumberOfToolInstances)
	{
		// Clean the Tool given by the index
		ErrorCode = (pDspInstance->Tools[IO_index].f_Clean)(pDspInstance->Tools[IO_index].pInstance);

		// Move the rest of the list one step
		for (index = IO_index; index < pDspInstance->NumberOfToolInstances - 1; index++)
		{
			pDspInstance->Tools[index].pInstance = pDspInstance->Tools[index + 1].pInstance;
			pDspInstance->Tools[index].f_Read = pDspInstance->Tools[index + 1].f_Read;
			pDspInstance->Tools[index].f_Write = pDspInstance->Tools[index + 1].f_Write;
			pDspInstance->Tools[index].f_Clean = pDspInstance->Tools[index + 1].f_Clean;
			pDspInstance->Tools[index].f_GetPosition = pDspInstance->Tools[index + 1].f_GetPosition;
			pDspInstance->Tools[index].f_GetLength = pDspInstance->Tools[index + 1].f_GetLength;
			strcpy(pDspInstance->Tools[index].CmdLine, pDspInstance->Tools[index + 1].CmdLine);
		}
				
		// We have one Tool less now
		pDspInstance->NumberOfToolInstances = pDspInstance->NumberOfToolInstances - 1;
	}
	else
	{
		OUTPUT_2("Index provided: %d exceeds limit: %d\n", IO_index, (pDspInstance->NumberOfToolInstances - 1));
	}
	

	DBG_PRINT("Out CmdDelTool\n");
	
	return ErrorCode;
};

/************************************************************/
/* NAME       :												*/
/* DESCRIPTION:												*/
/* INPUT      :												*/
/* OUTPUT     :												*/
/************************************************************/
INT32 CmdDelAllTool(CHAR* Options)
{
	INT32 index;
	INT32 ErrorCode;

	DBG_PRINT("In CmdDelAllTool\n");

	// Process the command
	// Clean all the Tools
	for (index = 0; index < pDspInstance->NumberOfToolInstances; index++)
	{
		ErrorCode = (pDspInstance->Tools[index].f_Clean)(pDspInstance->Tools[index].pInstance);
	}

	// We don't have Tools anymore
	pDspInstance->NumberOfToolInstances = 0;

	DBG_PRINT("Out CmdDelAllTool\n");
	
	return ErrorCode;
};

/************************************************************/
/* NAME       :												*/
/* DESCRIPTION:												*/
/* INPUT      :												*/
/* OUTPUT     :												*/
/************************************************************/
INT32 CmdShowTool(CHAR* Options)
{
	INT32 index;

	DBG_PRINT("In CmdShowTool\n");

	// Process the command
	for (index = 0; index < pDspInstance->NumberOfToolInstances; index++)
	{
		OUTPUT_2("Tool #%i command: +tool%s", index, pDspInstance->Tools[index].CmdLine);
	}
	
	DBG_PRINT("Out CmdShowTool\n");
	
	return 0;
};

/************************************************************/
/* NAME       :												*/
/* DESCRIPTION:												*/
/* INPUT      :												*/
/* OUTPUT     :												*/
/************************************************************/
INT32 CmdSetInstrBreakPoint(CHAR* Options)
{
	CHAR		TempStr[CMD_MAX_LEN];
	INT32		BreakpointAddress;

	DBG_PRINT("In CmdSetBreakPoint\n");
	
	// Get the breakpoint address from the Options string 
	GetNextWord(TempStr, Options);
	
	// If option string is empty, use PC as breakpoint address
	if (strcmp(TempStr, "") == 0) BreakpointAddress = pDspInstance->State.PC;
	else BreakpointAddress = atoi(TempStr);	// Transform string into breakpoint address
	
	// Check for value consistency
	if ((BreakpointAddress < 0) || (BreakpointAddress >= 1024))
	{
		ERROR("The address of a breakpoint should be between 0 and 1023 included.\n");
		ERROR_1(" Parameter provided is out of range: %i\n", BreakpointAddress);
		return -1;
	}
	
	// Mark line as being "break pointed"
	pDspInstance->Break.InstrBreakpoint[BreakpointAddress] = 1;

	DBG_PRINT("Out CmdSetBreakPoint\n");
	
	return 0;
};

/************************************************************/
/* NAME       :												*/
/* DESCRIPTION:												*/
/* INPUT      :												*/
/* OUTPUT     :												*/
/************************************************************/
INT32 CmdDelInstrBreakPoint(CHAR* Options)
{
	CHAR		TempStr[CMD_MAX_LEN];
	INT32		BreakpointAddress;

	DBG_PRINT("In CmdDelBreakPoint\n");
	
	// Get the breakpoint address from the Options string 
	GetNextWord(TempStr, Options);
	
	if (strcmp(TempStr, "") == 0) BreakpointAddress = pDspInstance->State.PC;
	else BreakpointAddress = atoi(TempStr);	// Transform string into breakpoint address

	// Check for value consistency
	if ((BreakpointAddress < 0) || (BreakpointAddress >= 1024))
	{
		ERROR("The address of a breakpoint should be between 0 and 1023 included.\n");
		ERROR_1(" Parameter provided is out of range: %i\n", BreakpointAddress);
		return -1;
	}
	
	// Mark line as being "break cleared"
	if (pDspInstance->Break.InstrBreakpoint[BreakpointAddress] == 0) WARNING_1("No breakpoint set at given address: %i\n", BreakpointAddress);
	pDspInstance->Break.InstrBreakpoint[BreakpointAddress] = 0;

	DBG_PRINT("Out CmdDelBreakPoint\n");
	
	return 0;
};

/************************************************************/
/* NAME       :												*/
/* DESCRIPTION:												*/
/* INPUT      :												*/
/* OUTPUT     :												*/
/************************************************************/
INT32 CmdSetTempBreakPoint(CHAR* Options)
{
	CHAR		TempStr[CMD_MAX_LEN];
	INT32		BreakpointAddress;
	INT32		NumberOfHits;

	DBG_PRINT("In CmdSetTempBreakPoint\n");

	// Get the number of hits from the Options string 
	GetNextWord(TempStr, Options);
	// If option string is empty, return an error
	if (strcmp(TempStr, "") == 0)
	{
		ERROR("Number of hits not given. Aborting command.\n");
		DBG_PRINT("Out CmdSetTempBreakPoint\n");
		return -1;
	}
	else NumberOfHits = atoi(TempStr);	// Transform string into number of hits
	// Check consistency of Number of hits value
	if (NumberOfHits >  32765)
	{
		ERROR("You don't really want to break more than 32765 times? Aborting command.\n");
		DBG_PRINT("Out CmdSetTempBreakPoint\n");
		return -2;
	}
	if (NumberOfHits < 1)
	{
		ERROR("You don't really want to break less than 1 tims? Aborting command.\n");
		DBG_PRINT("Out CmdSetTempBreakPoint\n");
		return -3;
	}

	// Get the breakpoint address from the Options string 
	GetNextWord(TempStr, Options);
	
	// If option string is empty, use PC as breakpoint address
	if (strcmp(TempStr, "") == 0) BreakpointAddress = pDspInstance->State.PC;
	else BreakpointAddress = atoi(TempStr);	// Transform string into breakpoint address
	
	// Check for value consistency
	if ((BreakpointAddress < 0) || (BreakpointAddress >= 1024))
	{
		ERROR("The address of a breakpoint should be between 0 and 1023 included.\n");
		ERROR_1(" Parameter provided is out of range: %i\n", BreakpointAddress);
		return -1;
	}
	
	// Mark line as being "break pointed"
	pDspInstance->Break.InstrBreakpoint[BreakpointAddress] = 1 + NumberOfHits;

	DBG_PRINT("Out CmdSetTempBreakPoint\n");
	
	return 0;
};

/************************************************************/
/* NAME       :												*/
/* DESCRIPTION:												*/
/* INPUT      :												*/
/* OUTPUT     :												*/
/************************************************************/
INT32 CmdSetValBreakPoint(CHAR* Options)
{
	CHAR		TempStr[CMD_MAX_LEN];
	INT32		*BreakAddress;
	INT32		BreakValue;
	AL3101_Value_Breakpoint	*Breakpoint;
	// memory related variables
	CHAR		IdentifierString[CMD_MAX_LEN];
	INT32		RegMemSpecifier;
	INT32		FirstItemToDisplay;

	DBG_PRINT("In CmdSetValBreakPoint\n");
	

	// "lower" the option string
	String_ToLower_String(Options, Options);
	// Get the value from the Options string 
	GetNextWord(TempStr, Options);
	BreakValue = atoi(TempStr);		// Transform string into break value
	

	// Get the register or memory identifier to display
	GetNextWord(TempStr, Options);

	GetRegMemSpecifier(TempStr, &RegMemSpecifier);
	if ((RegMemSpecifier > 0) && (RegMemSpecifier < REGMEM_SPECIFIER_MAX))
	{
		GetAddressStringFromSpecifier(RegMemSpecifier, IdentifierString, &BreakAddress);
		if ((RegMemSpecifier > REGMEM_SPECIFIER_LIMIT) && (RegMemSpecifier < REGMEM_SPECIFIER_MAX))
		{
			// If option string is empty, use 0 as location
			GetNextWord(TempStr, Options);
			if (strcmp(TempStr, "") == 0)
			{
				FirstItemToDisplay = 0;
				strcmp(TempStr, "0");
			}
			else FirstItemToDisplay = atoi(TempStr);	// Transform string into breakpoint address
			BreakAddress = BreakAddress + FirstItemToDisplay;
			// Format the string
			strcat(IdentifierString, "[");
			strcat(IdentifierString, TempStr);
			strcat(IdentifierString, "]");
		}
	}
	else 
	{
		ERROR_1("Identifier not known: %s\n", TempStr);
		DBG_PRINT("Out CmdSetValBreakPoint\n");
		return -1;
	}

	// Entering the list update part
	Breakpoint = new AL3101_Value_Breakpoint;
	Breakpoint->Address = BreakAddress;
	Breakpoint->TestValue = BreakValue;
	strcpy(Breakpoint->DisplayString, IdentifierString);

	pDspInstance->Break.BreakpointProcessingInProgress.Enter();
	pDspInstance->Break.ValBreakpoints.Append(Breakpoint);
	pDspInstance->Break.BreakpointProcessingInProgress.Leave();

	DBG_PRINT("Out CmdSetValBreakPoint\n");
	
	return 0;
};

/************************************************************/
/* NAME       :												*/
/* DESCRIPTION:												*/
/* INPUT      :												*/
/* OUTPUT     :												*/
/************************************************************/
INT32 CmdDelValBreakPoint(CHAR* Options)
{
	CHAR		TempStr[CMD_MAX_LEN];
	INT32		*BreakAddress;
	// memory related variables
	CHAR		IdentifierString[CMD_MAX_LEN];
	INT32		RegMemSpecifier;
	INT32		FirstItemToDisplay;

	DBG_PRINT("In CmdDelValBreakPoint\n");
	

	// "lower" the option string
	String_ToLower_String(Options, Options);

	// Get the register or memory identifier to display
	GetNextWord(TempStr, Options);

	GetRegMemSpecifier(TempStr, &RegMemSpecifier);
	if ((RegMemSpecifier > 0) && (RegMemSpecifier < REGMEM_SPECIFIER_MAX))
	{
		GetAddressStringFromSpecifier(RegMemSpecifier, IdentifierString, &BreakAddress);
		if ((RegMemSpecifier > REGMEM_SPECIFIER_LIMIT) && (RegMemSpecifier < REGMEM_SPECIFIER_MAX))
		{
			// If option string is empty, use 0 as location
			GetNextWord(TempStr, Options);
			if (strcmp(TempStr, "") == 0)
			{
				FirstItemToDisplay = 0;
				strcmp(TempStr, "0");
			}
			else FirstItemToDisplay = atoi(TempStr);	// Transform string into breakpoint address
			BreakAddress = BreakAddress + FirstItemToDisplay;
			// Format the string
			strcat(IdentifierString, "[");
			strcat(IdentifierString, TempStr);
			strcat(IdentifierString, "]");
		}
	}
	else 
	{
		ERROR_1("Identifier not known: %s\n", TempStr);
		DBG_PRINT("Out CmdDelValBreakPoint\n");
		return -1;
	}

	// Entering the list update part
	INT32 BreakCount, index;
	AL3101_Value_Breakpoint	*Breakpoint;
	ValBreakpoint_List::Node *Node;
	INT32 DeletedABreakpoint = 0;

	pDspInstance->Break.BreakpointProcessingInProgress.Enter();
	BreakCount = pDspInstance->Break.ValBreakpoints.GetCount();
	for (index = 0; index < BreakCount; index++)
	{
		Node = pDspInstance->Break.ValBreakpoints.Item(index);
		Breakpoint = Node->GetData();
		if (Breakpoint->Address == BreakAddress)
		{
			delete Breakpoint;
			if (pDspInstance->Break.ValBreakpoints.DeleteNode(Node) != TRUE) ERROR("Error in deleting breakpoint element.\n");
			DeletedABreakpoint++;
		}
	}
	pDspInstance->Break.BreakpointProcessingInProgress.Leave();

	if (DeletedABreakpoint == 0) WARNING("Breakpoint not found. Try Showbreak to know what are the breakpoints.\n");

	DBG_PRINT("Out CmdDelValBreakPoint\n");
	
	return 0;
};

/************************************************************/
/* NAME       :												*/
/* DESCRIPTION:												*/
/* INPUT      :												*/
/* OUTPUT     :												*/
/************************************************************/
INT32 CmdDelAllBreakPoints(CHAR* Options)
{
	INT32		index;

	DBG_PRINT("In CmdDelAllBreakPoints\n");

	for (index = 0; index < 1024; index++)
		pDspInstance->Break.InstrBreakpoint[index] = 0;

	// Delete the value breakpoints
	INT32 BreakCount;
	AL3101_Value_Breakpoint	*Breakpoint;
	ValBreakpoint_List::Node *Node;

	pDspInstance->Break.BreakpointProcessingInProgress.Enter();
	BreakCount = pDspInstance->Break.ValBreakpoints.GetCount();
	for (index = 0; index < BreakCount; index++)
	{
		Node = pDspInstance->Break.ValBreakpoints.GetFirst();
		Breakpoint = Node->GetData();
		delete Breakpoint;
		if (pDspInstance->Break.ValBreakpoints.DeleteNode(Node) != TRUE) ERROR("Error in deleting breakpoint element.\n");
	}
	pDspInstance->Break.BreakpointProcessingInProgress.Leave();

	DBG_PRINT("Out CmdDelAllBreakPoints\n");
	
	return 0;
};

/************************************************************/
/* NAME       :												*/
/* DESCRIPTION:												*/
/* INPUT      :												*/
/* OUTPUT     :												*/
/************************************************************/
INT32 CmdDisplayBreakPoints(CHAR* Options)
{
	INT32 BreakPointNumber = 1;
	INT32 TemporaryBreakPointNumber = 1;
	INT32 index;

	DBG_PRINT("In CmdDisplayBreakPoints\n");
	
	// Display the instruction breakpoints
	for (index = 0; index < 1024; index++)
	{
		if (pDspInstance->Break.InstrBreakpoint[index] == 1)
		{
			OUTPUT_2("Breakpoint #%i at address %i\n", BreakPointNumber, index);
			BreakPointNumber++;
		}
		if (pDspInstance->Break.InstrBreakpoint[index] > 1)
		{
			OUTPUT_3("Temporary breakpoint #%i at address %i. %i breaks remaining.\n", TemporaryBreakPointNumber, index, (pDspInstance->Break.InstrBreakpoint[index] - 1));
			TemporaryBreakPointNumber++;
		}
	}

	// Display the value breakpoints
	INT32 BreakCount;
	AL3101_Value_Breakpoint	*Breakpoint;
	ValBreakpoint_List::Node *Node;

	pDspInstance->Break.BreakpointProcessingInProgress.Enter();
	BreakCount = pDspInstance->Break.ValBreakpoints.GetCount();
	for (index = 0; index < BreakCount; index++)
	{
		Node = pDspInstance->Break.ValBreakpoints.Item(index);
		Breakpoint = Node->GetData();
		OUTPUT_3("Breakpoint on value #%i on %s = %i\n", index, Breakpoint->DisplayString, Breakpoint->TestValue);
	}
	pDspInstance->Break.BreakpointProcessingInProgress.Leave();

	DBG_PRINT("Out CmdDisplayBreakPoints\n");
	
	return 0;
};

/************************************************************/
/* NAME       :												*/
/* DESCRIPTION:												*/
/* INPUT      :												*/
/* OUTPUT     :												*/
/************************************************************/
INT32 CmdShow(CHAR* Options)
{
	CHAR		IdentifierString[CMD_MAX_LEN];
	INT32		RegMemSpecifier;
	INT32		*ItemToDisplay;
	INT32		*LastAllowedItem;
	INT32		FirstItemToDisplay;
	INT32		NumberOfItemsToDisplay;
	INT32		DisplayType;
	
	CHAR		TempStr[CMD_MAX_LEN];
	INT32		index;

	DBG_PRINT("In CmdShow\n");

	// "lower" the option string
	String_ToLower_String(Options, Options);
	// Get the register or memory identifier to display
	GetNextWord(TempStr, Options);

	NumberOfItemsToDisplay = 0;				// by default, there's only one item to display
	GetRegMemSpecifier(TempStr, &RegMemSpecifier);
	if ((RegMemSpecifier > 0) && (RegMemSpecifier < REGMEM_SPECIFIER_MAX))
	{
		GetAddressStringsFromSpecifier(RegMemSpecifier, IdentifierString, &ItemToDisplay, &LastAllowedItem);
		if ((RegMemSpecifier > REGMEM_SPECIFIER_LIMIT) && (RegMemSpecifier < REGMEM_SPECIFIER_MAX))
		{
			// Get the "Address" start given as offset
			GetAddressAddressNumbers(Options, &FirstItemToDisplay, &NumberOfItemsToDisplay);
			ItemToDisplay = ItemToDisplay + FirstItemToDisplay;
		}
	}
	else 
	{
		ERROR_1("Identifier not known in show command: %s\n", TempStr);
		DBG_PRINT("Out CmdShow\n");
		return -1;
	}

	// Get the type of display to use
	GetNextWord(TempStr, Options);
	
	if (strcmp(TempStr, "float") == 0)			DisplayType = AL3101_DISPLAY_FLOAT;
	else if (strcmp(TempStr, "ds3.24") == 0)	DisplayType = AL3101_DISPLAY_DS324;
	else if (strcmp(TempStr, "ds3.18") == 0)	DisplayType = AL3101_DISPLAY_DS318;
	else if (strcmp(TempStr, "hs3.24") == 0)	DisplayType = AL3101_DISPLAY_HS324;
	else if (strcmp(TempStr, "hs3.18") == 0)	DisplayType = AL3101_DISPLAY_HS318;
	else if (strcmp(TempStr, "bs3.24") == 0)	DisplayType = AL3101_DISPLAY_BS324;
	else if (strcmp(TempStr, "bs3.18") == 0)	DisplayType = AL3101_DISPLAY_BS318;
	else if (strcmp(TempStr, "dec") == 0)		DisplayType = AL3101_DISPLAY_DS324;		// same as the ds3.24 type
	else if (strcmp(TempStr, "hex") == 0)		DisplayType = AL3101_DISPLAY_HS324;		// same as the hs3.24 type
	else if (strcmp(TempStr, "bin") == 0)		DisplayType = AL3101_DISPLAY_BS324;		// same as the bs3.24 type
	else DisplayType = pDspInstance->Config.DefaultDisplayTypeInShow;					// default type used

	if (NumberOfItemsToDisplay == 0)			// only one register to show
	{
		InternalValue_To_String(TempStr, *ItemToDisplay, DisplayType);
		OUTPUT_2("%s = %s\n", IdentifierString, TempStr);
	}
	else										// memory bunch (even of size 1) to show
	{
		for (index = 0; index < NumberOfItemsToDisplay; index++)
		{
			if (ItemToDisplay >= LastAllowedItem)
			{
				OUTPUT("Limits reached, cannot display further\n");
				break;
			}
			InternalValue_To_String(TempStr, *ItemToDisplay, DisplayType);
			OUTPUT_3("%s[%i] = %s\n", IdentifierString, FirstItemToDisplay, TempStr);
			ItemToDisplay++;
			FirstItemToDisplay++;
		}
	}
		
	DBG_PRINT("Out CmdShow\n");
	
	return 0;
};


/************************************************************/
/* NAME       :												*/
/* DESCRIPTION:												*/
/* INPUT      :												*/
/* OUTPUT     :												*/
/************************************************************/
INT32 CmdSetVal(CHAR* Options)
{
	CHAR		IdentifierString[CMD_MAX_LEN];
	INT32		RegMemSpecifier;
	INT32		*ItemToDisplay;
	INT32		*LastAllowedItem;
	INT32		FirstItemToDisplay;
	INT32		Value;
	
	CHAR		TempStr[CMD_MAX_LEN];

	DBG_PRINT("In CmdVal\n");

	// "lower" the option string
	String_ToLower_String(Options, Options);
	// Get the register or memory identifier to display
	GetNextWord(TempStr, Options);

	GetRegMemSpecifier(TempStr, &RegMemSpecifier);
	if ((RegMemSpecifier > 0) && (RegMemSpecifier < REGMEM_SPECIFIER_MAX))
	{
		GetAddressStringsFromSpecifier(RegMemSpecifier, IdentifierString, &ItemToDisplay, &LastAllowedItem);
		if ((RegMemSpecifier > REGMEM_SPECIFIER_LIMIT) && (RegMemSpecifier < REGMEM_SPECIFIER_MAX))
		{
			// Get the "Address" start given as offset
			GetAddressAddressNumber(Options, &FirstItemToDisplay);
			ItemToDisplay = ItemToDisplay + FirstItemToDisplay;
		}
	}
	else 
	{
		ERROR_1("Identifier not known in show command: %s\n", TempStr);
		DBG_PRINT("Out CmdShow\n");
		return -1;
	}

	// Get now the values to be set in the chosen location
	String_ToUpper_String(Options, Options);
	if ((RegMemSpecifier > 3) && (RegMemSpecifier < 10))
	{
		GetNextWord(TempStr, Options);
		while(TempStr[0] != 0)
		{
			if (ItemToDisplay >= LastAllowedItem)
			{
				OUTPUT("Limits reached, cannot set values further.\n");
				OUTPUT_2("Following values will be ignored: %s%s\n", TempStr, Options);
				break;
			}
			if (ExtractHexa(TempStr, &Value, 24) != 0)
				ERROR_1("Error in converting value: %s\n", TempStr);
			ExtendTo32Bits(&Value);
			*ItemToDisplay = Value;
			ItemToDisplay++;
			// get the next value
			GetNextWord(TempStr, Options);
		}
	}
	else
	{
		GetNextWord(TempStr, Options);
		if (ExtractHexa(TempStr, &Value, 24) != 0)
			ERROR_1("Error in converting value: %s\n", TempStr);
		ExtendTo32Bits(&Value);
		*ItemToDisplay = Value;
	}

	DBG_PRINT("Out CmdVal\n");
	
	return 0;
};

/************************************************************/
/* NAME       :												*/
/* DESCRIPTION:												*/
/* INPUT      :												*/
/* OUTPUT     :												*/
/************************************************************/
INT32 CmdSetWatch(CHAR* Options)
{
	CHAR		IdentifierString[CMD_MAX_LEN];
	INT32		RegMemSpecifier;
	INT32		*ItemToDisplay;
	INT32		*CopyItemToDisplay;
	INT32		FirstItemToDisplay;
	INT32		NumberOfItemsToDisplay;
	INT32		DisplayType;
	CHAR		TempStr[CMD_MAX_LEN];


	DBG_PRINT("In CmdSetWatch\n");
	
	// "lower" the option string
	String_ToLower_String(Options, Options);
	// Get first the register or memory identifier to display
	GetNextWord(TempStr, Options);

	NumberOfItemsToDisplay = 0;				// by default, there's only one item to display
	GetRegMemSpecifier(TempStr, &RegMemSpecifier);
	if ((RegMemSpecifier > 0) && (RegMemSpecifier < REGMEM_SPECIFIER_MAX))
	{
		GetAddressStringFromSpecifier(RegMemSpecifier, IdentifierString, &ItemToDisplay);
		if ((RegMemSpecifier > REGMEM_SPECIFIER_LIMIT) && (RegMemSpecifier < REGMEM_SPECIFIER_MAX))
		{
			// Get the "Address" start given as offset
			GetAddressAddressNumbers(Options, &FirstItemToDisplay, &NumberOfItemsToDisplay);
			ItemToDisplay = ItemToDisplay + FirstItemToDisplay;
		}
	}
	else 
	{
		ERROR_1("Identifier not known in show command: %s\n", TempStr);
		DBG_PRINT("Out CmdSetWatch\n");
		return -1;
	}

	// Get the then the type of display to use
	GetNextWord(TempStr, Options);
	
	if (strcmp(TempStr, "float") == 0)			DisplayType = AL3101_DISPLAY_FLOAT;
	else if (strcmp(TempStr, "ds3.24") == 0)	DisplayType = AL3101_DISPLAY_DS324;
	else if (strcmp(TempStr, "ds3.18") == 0)	DisplayType = AL3101_DISPLAY_DS318;
	else if (strcmp(TempStr, "hs3.24") == 0)	DisplayType = AL3101_DISPLAY_HS324;
	else if (strcmp(TempStr, "hs3.18") == 0)	DisplayType = AL3101_DISPLAY_HS318;
	else if (strcmp(TempStr, "bs3.24") == 0)	DisplayType = AL3101_DISPLAY_BS324;
	else if (strcmp(TempStr, "bs3.18") == 0)	DisplayType = AL3101_DISPLAY_BS318;
	else if (strcmp(TempStr, "dec") == 0)		DisplayType = AL3101_DISPLAY_DS324;		// same as the ds3.24 type
	else if (strcmp(TempStr, "hex") == 0)		DisplayType = AL3101_DISPLAY_HS324;		// same as the hs3.24 type
	else if (strcmp(TempStr, "bin") == 0)		DisplayType = AL3101_DISPLAY_BS324;		// same as the bs3.24 type
	else DisplayType = pDspInstance->Config.DefaultDisplayTypeInShow;					// default type used

	// Derive from the Parameters the pointer to the copy of the memory
	INT32 *TempPtr1 = (INT32 *)(&pDspInstance->State);
	INT32 *TempPtr2 = (INT32 *)(&pDspInstance->Watch.StateCopy);
	INT32 Temp = TempPtr2 - TempPtr1;
	CopyItemToDisplay = ItemToDisplay + Temp;

	// At this point:
	//  - ItemToDisplay is the pointer to the element(s) to watch
	//  - CopyItemToDisplay is the pointer to the copy of the memory pointing on the element(s) to watch
	//  - RegMemSpecifier tells if the watch is on a register (if 0< <3) or on a memory (if 3< <10)
	//  - FirstItemToDisplay is the offset in the memory bank
	//  - NumberOfItemsToDisplay is the number of items to display for memory watches
	//  - DisplayType contains the type of display which should be used
	//  - IdentifierString contains the string describing the element to show
	if (NumberOfItemsToDisplay == 0)
	{
		AL3101_RegisterWatch *watch = new AL3101_RegisterWatch;
		if (watch == NULL)
		{
			ERROR("Unable to allocate memory for the watch\n");
			DBG_PRINT("Out CmdSetWatch\n");
			return -1;
		}
		// initialize watch related variables
		watch->Address = ItemToDisplay;
		watch->CopyAddress = CopyItemToDisplay;
		// initialize display related variables
		watch->DisplayFormat = DisplayType;
		strcpy(watch->DisplayString, IdentifierString);
		
		// Add the watch
		pDspInstance->Watch.WatchProcessingInProgress.Enter();
		pDspInstance->Watch.RegisterWatches.Append(watch);
		pDspInstance->Watch.WatchProcessingInProgress.Leave();
	}
	else
	{
		AL3101_MemoryWatch *watch = new AL3101_MemoryWatch;
		if (watch == NULL)
		{
			ERROR("Unable to allocate memory for the watch\n");
			DBG_PRINT("Out CmdSetWatch\n");
			return -1;
		}
		// initialize watch related variables
		watch->Address = ItemToDisplay;
		watch->CopyAddress = CopyItemToDisplay;
		watch->BeginIndex = FirstItemToDisplay;
		watch->NumberOfElements = NumberOfItemsToDisplay;
		// initialize display related variables
		watch->DisplayFormat = DisplayType;
		strcpy(watch->PreDisplayString, IdentifierString);
		strcat(watch->PreDisplayString, "[");
		strcpy(watch->PostDisplayString, "]");
		
		// Add the watch
		pDspInstance->Watch.WatchProcessingInProgress.Enter();
		pDspInstance->Watch.MemoryWatches.Append(watch);
		pDspInstance->Watch.WatchProcessingInProgress.Leave();
	}
	

	DBG_PRINT("Out CmdSetWatch\n");
	
	return 0;
};

/************************************************************/
/* NAME       :												*/
/* DESCRIPTION:												*/
/* INPUT      :												*/
/* OUTPUT     :												*/
/************************************************************/
INT32 CmdDelWatch(CHAR* Options)
{
	CHAR		TempStr[CMD_MAX_LEN];
	INT32		ReturnValue = 0;
	INT32		WatchIndex = 0;
	INT32		RegWatchCount, MemWatchCount;
	AL3101_RegisterWatch	 *RegWatch;
	RegisterWatch_List::Node *RegNode;
	AL3101_MemoryWatch		 *MemWatch;
	MemoryWatch_List::Node	 *MemNode;

	DBG_PRINT("In CmdDelWatch\n");

	// Get the value from the Options string 
	GetNextWord(TempStr, Options);
	WatchIndex = atoi(TempStr);		// Transform string into watch index value
	if (WatchIndex == 0)
	{
		ERROR_1("The parameter entered is not a valid index: %s\n", WatchIndex);
		ReturnValue = -1;
		goto CmdDelWatch_EndOfFunction;
	}

	// Get the count of the different watches
	RegWatchCount = pDspInstance->Watch.RegisterWatches.GetCount();
	MemWatchCount = pDspInstance->Watch.MemoryWatches.GetCount();
	if (WatchIndex > RegWatchCount + MemWatchCount)
	{
		ERROR_2("The index given (%i) is out the range: %i\n", WatchIndex, RegWatchCount + MemWatchCount);
		ReturnValue = -2;
		goto CmdDelWatch_EndOfFunction;
	}

	WatchIndex--;					// The rest of the function will work 0-based
	pDspInstance->Watch.WatchProcessingInProgress.Enter();
	if (WatchIndex < RegWatchCount)
	{
		RegNode = pDspInstance->Watch.RegisterWatches.Item(WatchIndex);
		RegWatch = RegNode->GetData();
		delete RegWatch;
		if (pDspInstance->Watch.RegisterWatches.DeleteNode(RegNode) != TRUE) ERROR("Error in deleting watch element.\n");
	}
	else
	{
		WatchIndex = WatchIndex - RegWatchCount;
		MemNode = pDspInstance->Watch.MemoryWatches.Item(WatchIndex);
		MemWatch = MemNode->GetData();
		delete MemWatch;
		if (pDspInstance->Watch.MemoryWatches.DeleteNode(MemNode) != TRUE) ERROR("Error in deleting watch element.\n");
	}
	pDspInstance->Watch.WatchProcessingInProgress.Leave();

	
CmdDelWatch_EndOfFunction:
	DBG_PRINT("Out CmdDelWatch\n");
	
	return ReturnValue;
};

/************************************************************/
/* NAME       :												*/
/* DESCRIPTION:												*/
/* INPUT      :												*/
/* OUTPUT     :												*/
/************************************************************/
INT32 CmdDelAllWatches(CHAR* Options)
{
	INT32 WatchCount;
	INT32 index, WatchIndex = 0;
	AL3101_RegisterWatch	 *RegWatch;
	RegisterWatch_List::Node *RegNode;
	AL3101_MemoryWatch		 *MemWatch;
	MemoryWatch_List::Node	 *MemNode;
	DBG_PRINT("In CmdDelAllWatches\n");

	pDspInstance->Watch.WatchProcessingInProgress.Enter();
	// Delete the register watches
	WatchCount = pDspInstance->Watch.RegisterWatches.GetCount();
	for (index = 0; index < WatchCount; index++)
	{
		RegNode = pDspInstance->Watch.RegisterWatches.GetFirst();
		RegWatch = RegNode->GetData();
		delete RegWatch;
		if (pDspInstance->Watch.RegisterWatches.DeleteNode(RegNode) != TRUE) ERROR("Error in deleting watch element.\n");
	}
	// Delete the register watches
	WatchCount = pDspInstance->Watch.MemoryWatches.GetCount();
	for (index = 0; index < WatchCount; index++)
	{
		MemNode = pDspInstance->Watch.MemoryWatches.GetFirst();
		MemWatch = MemNode->GetData();
		delete MemWatch;
		if (pDspInstance->Watch.MemoryWatches.DeleteNode(MemNode) != TRUE) ERROR("Error in deleting watch element.\n");
	}
	pDspInstance->Watch.WatchProcessingInProgress.Leave();

	DBG_PRINT("Out CmdDelAllWatches\n");
	
	return 0;
};

/************************************************************/
/* NAME       :												*/
/* DESCRIPTION:												*/
/* INPUT      :												*/
/* OUTPUT     :												*/
/************************************************************/
INT32 CmdDisplayWatches(CHAR* Options)
{
	INT32 WatchCount;
	INT32 index, WatchIndex = 1;
	AL3101_RegisterWatch	 *RegWatch;
	RegisterWatch_List::Node *RegNode;
	AL3101_MemoryWatch		 *MemWatch;
	MemoryWatch_List::Node	 *MemNode;

	DBG_PRINT("In CmdDisplayWatches\n");


	pDspInstance->Watch.WatchProcessingInProgress.Enter();
	// Display the register breakpoints
	WatchCount = pDspInstance->Watch.RegisterWatches.GetCount();
	for (index = 0; index < WatchCount; index++)
	{
		RegNode = pDspInstance->Watch.RegisterWatches.Item(index);
		RegWatch = RegNode->GetData();
		OUTPUT_2("Watch #%i on %s\n", WatchIndex, RegWatch->DisplayString);
		WatchIndex++;
	}
	// Display the memory breakpoints
	WatchCount = pDspInstance->Watch.MemoryWatches.GetCount();
	for (index = 0; index < WatchCount; index++)
	{
		MemNode = pDspInstance->Watch.MemoryWatches.Item(index);
		MemWatch = MemNode->GetData();
		if (MemWatch->NumberOfElements == 1) OUTPUT_4("Watch #%i on %s%i%s\n", WatchIndex, MemWatch->PreDisplayString, MemWatch->BeginIndex, MemWatch->PostDisplayString);
		else OUTPUT_7("Watch #%i on %s%i%s to %s%i%s\n", WatchIndex, MemWatch->PreDisplayString, MemWatch->BeginIndex, MemWatch->PostDisplayString, MemWatch->PreDisplayString, MemWatch->BeginIndex + MemWatch->NumberOfElements, MemWatch->PostDisplayString);
		WatchIndex++;
	}
	pDspInstance->Watch.WatchProcessingInProgress.Leave();

	DBG_PRINT("Out CmdDisplayWatches\n");
	
	return 0;
};

/************************************************************/
/* NAME       :												*/
/* DESCRIPTION:												*/
/* INPUT      :												*/
/* OUTPUT     :												*/
/************************************************************/
INT32 CmdSetUserParameter(CHAR* Options)
{
	DBG_PRINT("In CmdSetUserParameter\n");
	CHAR ParameterString[CMD_MAX_LEN];
	CHAR ValueString[CMD_MAX_LEN];

	String_ToLower_String(Options, Options);
	GetNextWord(ParameterString, Options);
	GetNextWord(ValueString, Options);

	if (strcmp(ParameterString, "dasm") == 0)
	{
		if (strcmp(ValueString, "on") == 0) pDspInstance->Config.ShowDisasmWhenStepping = 1;
		else if (strcmp(ValueString, "off") == 0) pDspInstance->Config.ShowDisasmWhenStepping = 0;
		else ERROR_1("Parameter value not recognized: %s\n", ValueString);
	}
	else if (strcmp(ParameterString, "showformat") == 0)
	{
		if (strcmp(ValueString, "float") == 0) pDspInstance->Config.DefaultDisplayTypeInShow = AL3101_DISPLAY_FLOAT;
		else if (strcmp(ValueString, "ds3.24") == 0) pDspInstance->Config.DefaultDisplayTypeInShow = AL3101_DISPLAY_DS324;
		else if (strcmp(ValueString, "ds3.18") == 0) pDspInstance->Config.DefaultDisplayTypeInShow = AL3101_DISPLAY_DS318;
		else if (strcmp(ValueString, "hs3.24") == 0) pDspInstance->Config.DefaultDisplayTypeInShow = AL3101_DISPLAY_HS324;
		else if (strcmp(ValueString, "hs3.18") == 0) pDspInstance->Config.DefaultDisplayTypeInShow = AL3101_DISPLAY_HS318;
		else if (strcmp(ValueString, "bs3.24") == 0) pDspInstance->Config.DefaultDisplayTypeInShow = AL3101_DISPLAY_BS324;
		else if (strcmp(ValueString, "bs3.18") == 0) pDspInstance->Config.DefaultDisplayTypeInShow = AL3101_DISPLAY_BS318;
		else if (strcmp(ValueString, "dec") == 0) pDspInstance->Config.DefaultDisplayTypeInShow = AL3101_DISPLAY_DS324;
		else if (strcmp(ValueString, "hex") == 0) pDspInstance->Config.DefaultDisplayTypeInShow = AL3101_DISPLAY_HS324;
		else if (strcmp(ValueString, "bin") == 0) pDspInstance->Config.DefaultDisplayTypeInShow = AL3101_DISPLAY_BS324;
		else ERROR_1("Parameter value not recognized: %s\n", ValueString);
	}
	else if (strcmp(ParameterString, "truecycle") == 0)
	{
		if (strcmp(ValueString, "on") == 0) pDspInstance->Config.TrueCycleSimulation = 1;
		else if (strcmp(ValueString, "off") == 0) pDspInstance->Config.TrueCycleSimulation = 0;
		else ERROR_1("Parameter value not recognized: %s\n", ValueString);
	}
	else if (strcmp(ParameterString, "resettype") == 0)
	{
		pDspInstance->Config.DefaultResetType = GetResetType(ValueString);
	}
	else if (strcmp(ParameterString, "maxinputs") == 0)
	{
		pDspInstance->Config.MaxNumberOfInputs = atoi(ValueString);
		if (pDspInstance->Config.MaxNumberOfInputs > MAX_IO)
		{
			WARNING_1("Value entered out of range. Value set to limit: %i\n", MAX_IO);
			pDspInstance->Config.MaxNumberOfInputs = 1024;
		}
		if (pDspInstance->Config.MaxNumberOfInputs < 0)
		{
			WARNING("Value entered negative. Value set to default: 32\n");
			pDspInstance->Config.MaxNumberOfInputs = 32;
		}
		WARNING("Warning: this command will take effect at the next execution of the simulator.\n");
	}
	else if (strcmp(ParameterString, "maxoutputs") == 0)
	{
		pDspInstance->Config.MaxNumberOfOutputs = atoi(ValueString);
		if (pDspInstance->Config.MaxNumberOfInputs > MAX_IO)
		{
			WARNING_1("Value entered out of range. Value set to limit: %i\n", MAX_IO);
			pDspInstance->Config.MaxNumberOfOutputs = 1024;
		}
		if (pDspInstance->Config.MaxNumberOfOutputs < 0)
		{
			WARNING("Value entered negative. Value set to default: 32\n");
			pDspInstance->Config.MaxNumberOfOutputs = 32;
		}
		WARNING("Warning: this command will take effect at the next execution of the simulator.\n");
	}
	else if (strcmp(ParameterString, "maxsignals") == 0)
	{
		pDspInstance->Config.MaxNumberOfSignals = atoi(ValueString);
		if (pDspInstance->Config.MaxNumberOfSignals > MAX_IO)
		{
			WARNING_1("Value entered out of range. Value set to limit: %i\n", MAX_IO);
			pDspInstance->Config.MaxNumberOfSignals = 1024;
		}
		if (pDspInstance->Config.MaxNumberOfSignals < 0)
		{
			WARNING("Value entered negative. Value set to default: 32\n");
			pDspInstance->Config.MaxNumberOfSignals = 32;
		}
		WARNING("Warning: this command will take effect at the next execution of the simulator.\n");
	}
	else if (strcmp(ParameterString, "maxtraces") == 0)
	{
		pDspInstance->Config.MaxNumberOfTraces = atoi(ValueString);
		if (pDspInstance->Config.MaxNumberOfTraces > MAX_IO)
		{
			WARNING_1("Value entered out of range. Value set to limit: %i\n", MAX_IO);
			pDspInstance->Config.MaxNumberOfTraces = 1024;
		}
		if (pDspInstance->Config.MaxNumberOfTraces < 0)
		{
			WARNING("Value entered negative. Value set to default: 32\n");
			pDspInstance->Config.MaxNumberOfTraces = 32;
		}
		WARNING("Warning: this command will take effect at the next execution of the simulator.\n");
	}
	else if (strcmp(ParameterString, "maxtools") == 0)
	{
		pDspInstance->Config.MaxNumberOfTools = atoi(ValueString);
		if (pDspInstance->Config.MaxNumberOfInputs > MAX_IO)
		{
			WARNING_1("Value entered out of range. Value set to limit: %i\n", MAX_IO);
			pDspInstance->Config.MaxNumberOfTools = 1024;
		}
		if (pDspInstance->Config.MaxNumberOfTools < 0)
		{
			WARNING("Value entered negative. Value set to default: 32\n");
			pDspInstance->Config.MaxNumberOfTools = 32;
		}
		WARNING("Warning: this command will take effect at the next execution of the simulator.\n");
	}
	else ERROR_1("Parameter unknown: %s\n", ParameterString);

	
	DBG_PRINT("Out CmdSetUserParameter\n");
	
	return 0;
};

/************************************************************/
/* NAME       :												*/
/* DESCRIPTION:												*/
/* INPUT      :												*/
/* OUTPUT     :												*/
/************************************************************/
INT32 CmdDisplayCycles(CHAR* Options)
{
	DBG_PRINT("In CmdDisplayCycles\n");

	ERROR("Command not implemented yet.\n");
	
	DBG_PRINT("Out CmdDisplayCycles\n");
	
	return 0;
};

/************************************************************/
/* NAME       :												*/
/* DESCRIPTION:												*/
/* INPUT      :												*/
/* OUTPUT     :												*/
/************************************************************/
INT32 CmdDisplayDSPState(CHAR* Options)
{
	DBG_PRINT("In CmdDisplayDSPState\n");

	ERROR("Command not implemented yet.\n");
	
	DBG_PRINT("Out CmdDisplayDSPState\n");
	
	return 0;
};

/************************************************************/
/* NAME       :												*/
/* DESCRIPTION:												*/
/* INPUT      :												*/
/* OUTPUT     :												*/
/************************************************************/
INT32 CmdDisplayRegisters(CHAR* Options)
{
//	INT32 RegIndex = 0;
	
	DBG_PRINT("In CmdDisplayRegisters\n");
	
	ERROR("Command not implemented yet.\n");

	DBG_PRINT("Out CmdDisplayRegisters\n");
	return 0;
};

/************************************************************/
/* NAME       :												*/
/* DESCRIPTION:												*/
/* INPUT      :												*/
/* OUTPUT     :												*/
/************************************************************/
INT32 CmdDisplayProgMemory(CHAR* Options)
{
	DBG_PRINT("In CmdDisplayProgMemory\n");
	
	
	ERROR("Command not implemented yet.\n");
//	OUTPUT("PROGRAM MEMORY DUMP:\n");
//	OUTPUT("--------------------\n");
	//Display24BitsBuffer(DspInstance.ProgMemory, PROG_MEMORY_SIZE, 4);
//	OUTPUT("\n");

	DBG_PRINT("Out CmdDisplayProgMemory\n");
	
	return 0;
};

/************************************************************/
/* NAME       :												*/
/* DESCRIPTION:												*/
/* INPUT      :												*/
/* OUTPUT     :												*/
/************************************************************/
INT32 CmdDisplayDataMemory(CHAR* Options)
{
	DBG_PRINT("In CmdDisplayDataMemory\n");
	
	ERROR("Command not implemented yet.\n");
//	OUTPUT("DATA MEMORY DUMP:\n");
//	OUTPUT("-----------------\n");
	//Display24BitsBuffer(DspInstance.DataMemory, DATA_MEMORY_SIZE, 4);
//	OUTPUT("\n");

	DBG_PRINT("Out CmdDisplayDataMemory\n");

	return 0;
};

/************************************************************/
/* NAME       :												*/
/* DESCRIPTION:												*/
/* INPUT      :												*/
/* OUTPUT     :												*/
/************************************************************/
INT32 CmdDasmN(CHAR* Options)
{
	CHAR		TempStr[CMD_MAX_LEN];
	INT32		BeginAddress, EndAddress, CurrentAddress;

	DBG_PRINT("In CmdDasmN\n");

	// Get the begin address from the Options string 
	GetNextWord(TempStr, Options);

	// If option string is empty, use PC as address
	if (strcmp(TempStr, "") == 0) BeginAddress = pDspInstance->State.PC;
	else BeginAddress = atoi(TempStr);		// Transform string into address
	
	// Check for value consistency
	if ((BeginAddress < 0) || (BeginAddress >= 1024))
	{
		ERROR("The address for disassembly printing should be between 0 and 1023 included.\n");
		ERROR_1(" Parameter provided is out of range: %i\n", BeginAddress);
		return -1;
	}
	
	// Get the end address from the Options string 
	GetNextWord(TempStr, Options);

	// If option string is empty, use begin address as address
	if (strcmp(TempStr, "") == 0) EndAddress = BeginAddress;
	else EndAddress = atoi(TempStr);		// Transform string into address

	// Check for value consistency
	if ((EndAddress < 0) || (EndAddress >= 1024))
	{
		ERROR("The address for disassembly printing should be between 0 and 1023 included.\n");
		ERROR_1(" Parameter provided is out of range: %i\n", EndAddress);
		return -1;
	}
	
	for (CurrentAddress = BeginAddress; CurrentAddress <= EndAddress; CurrentAddress++)
		OUTPUT_2("Opcode stored in %i is %s\n", CurrentAddress, pDspInstance->CurrentProgram[CurrentAddress].CmdStr);
	
	DBG_PRINT("Out CmdDasmN\n");
	
	return 0;
};

/************************************************************/
/* NAME       :												*/
/* DESCRIPTION:												*/
/* INPUT      :												*/
/* OUTPUT     :												*/
/************************************************************/
INT32 CmdDasmOn(CHAR* Options)
{
	DBG_PRINT("In CmdDasmOn\n");

	pDspInstance->Config.ShowDisasmWhenStepping = 1;
	
	DBG_PRINT("Out CmdDasmOn\n");
	
	return 0;
};

/************************************************************/
/* NAME       :												*/
/* DESCRIPTION:												*/
/* INPUT      :												*/
/* OUTPUT     :												*/
/************************************************************/
INT32 CmdDasmOff(CHAR* Options)
{
	DBG_PRINT("In CmdDasmOff\n");

	pDspInstance->Config.ShowDisasmWhenStepping = 0;

	DBG_PRINT("Out CmdDasmOff\n");
	
	return 0;
};

/************************************************************/
/* NAME       :												*/
/* DESCRIPTION:												*/
/* INPUT      :												*/
/* OUTPUT     :												*/
/************************************************************/
INT32 CmdSetAlias(CHAR* Options)
{
	CHAR		AliasName[CMD_MAX_LEN];
	CHAR		CommandName[CMD_MAX_LEN];
	INT32		CommandID, CommandNumInArray;


	DBG_PRINT("In CmdSetAlias\n");
	
	// "lower" the option string
	String_ToLower_String(Options, Options);
	// Get first the alias name
	GetNextWord(AliasName, Options);
	// Get then the command name
	GetNextWord(CommandName, Options);

	if ((strcmp(AliasName, "") == 0) || (strcmp(CommandName, "") == 0))
	{
		ERROR("Alias or command name empty.\n");
		DBG_PRINT("Out CmdSetAlias\n");
		return -1;
	}

	CommandNumInArray = GetCommandID(CommandName, &CommandID);
	if (CommandNumInArray == NumOfCommandsInList)
	{
		ERROR("Command not recognized.\n");
		DBG_PRINT("Out CmdSetAlias\n");
		return -1;
	}


	AL3101_CommandAlias *alias = new AL3101_CommandAlias;
	if (alias == NULL)
	{
		ERROR("Unable to allocate memory for the alias\n");
		DBG_PRINT("Out CmdSetAlias\n");
		return -1;
	}
	// initialize the variables
	alias->CommandID = CommandNumInArray;
	strcpy(alias->AliasString, AliasName);
	strcpy(alias->CommandString, CommandName);
	
	// Add the watch
	pDspInstance->Config.Alias.Append(alias);


	DBG_PRINT("Out CmdSetAlias\n");
	
	return 0;
};

/************************************************************/
/* NAME       :												*/
/* DESCRIPTION:												*/
/* INPUT      :												*/
/* OUTPUT     :												*/
/************************************************************/
INT32 CmdDelAlias(CHAR* Options)
{
	CHAR		TempStr[CMD_MAX_LEN];
	INT32		ReturnValue = 0;
	INT32		AliasIndex = 0;
	INT32		AliasCount;
	AL3101_CommandAlias	 *Alias;
	CommandAlias_List::Node *Node;

	DBG_PRINT("In CmdDelAlias\n");

	// Get the value from the Options string 
	GetNextWord(TempStr, Options);
	AliasIndex = atoi(TempStr);		// Transform string into alias index value
	if (AliasIndex == 0)
	{
		ERROR_1("The parameter entered is not a valid index: %s\n", AliasIndex);
		ReturnValue = -1;
		goto CmdDelAlias_EndOfFunction;
	}

	// Get the count of the different aliases
	AliasCount = pDspInstance->Config.Alias.GetCount();
	if (AliasCount == 0)
	{
		ERROR("No alias defined; nothing to delete.\n");
		ReturnValue = -2;
		goto CmdDelAlias_EndOfFunction;
	}
	if (AliasIndex > AliasCount)
	{
		ERROR_2("The index given (%i) is out the range: %i\n", AliasIndex, AliasCount);
		ReturnValue = -3;
		goto CmdDelAlias_EndOfFunction;
	}

	AliasIndex--;					// The rest of the function will work 0-based

	Node = pDspInstance->Config.Alias.Item(AliasIndex);
	Alias = Node->GetData();
	delete Alias;
	if (pDspInstance->Config.Alias.DeleteNode(Node) != TRUE) ERROR("Error in deleting alias element.\n");

	
CmdDelAlias_EndOfFunction:
	DBG_PRINT("Out CmdDelAlias\n");
	
	return ReturnValue;
};

/************************************************************/
/* NAME       :												*/
/* DESCRIPTION:												*/
/* INPUT      :												*/
/* OUTPUT     :												*/
/************************************************************/
INT32 CmdDelAllAliases(CHAR* Options)
{
	INT32 AliasCount;
	INT32 index, AliasIndex = 0;
	AL3101_CommandAlias		 *Alias;
	CommandAlias_List::Node	 *Node;

	DBG_PRINT("In CmdDelAllAliases\n");

	// Delete the aliases
	AliasCount = pDspInstance->Config.Alias.GetCount();
	for (index = 0; index < AliasCount; index++)
	{
		Node = pDspInstance->Config.Alias.GetFirst();
		Alias = Node->GetData();
		delete Alias;
		if (pDspInstance->Config.Alias.DeleteNode(Node) != TRUE) ERROR("Error in deleting alias element.\n");
	}

	DBG_PRINT("Out CmdDelAllAliases\n");
	
	return 0;
};

/************************************************************/
/* NAME       :												*/
/* DESCRIPTION:												*/
/* INPUT      :												*/
/* OUTPUT     :												*/
/************************************************************/
INT32 CmdDisplayAliases(CHAR* Options)
{
	INT32 AliasCount;
	INT32 index, AliasIndex = 1;
	AL3101_CommandAlias		 *Alias;
	CommandAlias_List::Node  *Node;

	DBG_PRINT("In CmdDisplayAliases\n");


	// Display the aliases
	AliasCount = pDspInstance->Config.Alias.GetCount();
	for (index = 0; index < AliasCount; index++)
	{
		Node = pDspInstance->Config.Alias.Item(index);
		Alias = Node->GetData();
		OUTPUT_3("Alias #%i: %s equivalent to %s\n", AliasIndex, Alias->AliasString, Alias->CommandString);
		AliasIndex++;
	}


	DBG_PRINT("Out CmdDisplayWatches\n");
	
	return 0;
};


/************************************************************/
/* NAME       :												*/
/* DESCRIPTION:												*/
/* INPUT      :												*/
/* OUTPUT     :												*/
/************************************************************/
INT32 CmdSetStream(CHAR* Options)
{
	CHAR		TempStr[CMD_MAX_LEN];
	INT32		BlockSize, NumberOfBlocks, InputDeviceID, OutputDeviceID;

	DBG_PRINT("In CmdSetStream\n");

	// Let's first see what is the block size to be used when streaming
	GetNextWord(TempStr, Options);
	if (strcmp(TempStr, "") == 0)
	{
		ERROR("Not enough parameters. Check User Manuals for details\n");
		DBG_PRINT("Out CmdSetStream\n");
		return -1;
	}
	sscanf(TempStr, "%i", &BlockSize);
	if (BlockSize <= 0)
	{
		WARNING_1("Blocksize of %i fobidden, setting to 256\n", BlockSize);
		BlockSize = 256;
	}

	// Then let's find the number of blocks
	GetNextWord(TempStr, Options);
	if (strcmp(TempStr, "") == 0)
	{
		ERROR("Not enough parameters. Check User Manuals for details\n");
		DBG_PRINT("Out CmdSetStream\n");
		return -1;
	}
	sscanf(TempStr, "%i", &NumberOfBlocks);
	if (NumberOfBlocks < 0)
	{
		WARNING("A negative number of blocks is fobidden, setting to optimal\n");
		NumberOfBlocks = 0;
	}
	if (NumberOfBlocks == 0) INFO("Using optimal number of blocks.\n");

	// Let's then look at the input device ID
	GetNextWord(TempStr, Options);
	if (strcmp(TempStr, "") == 0) goto CmdSetStream_ExitFunction;
	sscanf(TempStr, "%i", &InputDeviceID);

	// Then finally get the output device number
	GetNextWord(TempStr, Options);
	if (strcmp(TempStr, "") == 0) goto CmdSetStream_ExitFunction;
	sscanf(TempStr, "%i", &OutputDeviceID);

CmdSetStream_ExitFunction:
	DBG_PRINT("Out CmdSetStream\n");
	
	return 0;
};
/************************************************************/
/* NAME       :												*/
/* DESCRIPTION:												*/
/* INPUT      :												*/
/* OUTPUT     :												*/
/************************************************************/
INT32 CmdStartStream(CHAR* Options)
{
	CHAR		TempStr[CMD_MAX_LEN];
	INT32		TempSamplingFrequency, TempInt;
	INT32		index;
	CHAR		IdentifierString[CMD_MAX_LEN];
	INT32		RegMemSpecifier;
	INT32		*ItemToDisplay;
	INT32		*LastAllowedItem;
	INT32		FirstItemToDisplay;
	INT32		NumberOfItemsToDisplay;

	DBG_PRINT("In CmdStartStream\n");

	// "lower" the option string
	String_ToLower_String(Options, Options);

	// Let's first get the sampling frequency
	GetNextWord(TempStr, Options);
	TempSamplingFrequency = atoi(TempStr);
	// Checking that the sampling frequency is OK with the previously initialized inputs
	if (TempSamplingFrequency != 0)						// If given sampling frequency is 0, do not take it in account
	{
		// first check the "absolute" validity of the value regarding the DSP capabilities
		if ((TempSamplingFrequency < 30000) || (TempSamplingFrequency > 50000))
		{
			WARNING("Warning: the recommended sampling frequency should be between 30kHz and 50kHz.\n");
			WARNING_1(" The sampling frequency required is %d.", TempSamplingFrequency);
			WARNING(" Simulation will go on but will perhaps not represent the real hardware operation.\n");
		}
		// then check the "relative" validity of the value regarding the inputs already set
		if (pDspInstance->SamplingFrequency == -1) pDspInstance->SamplingFrequency = TempSamplingFrequency;		// The first time, the sampling frequency used in the program is set
		else
		{
			if (pDspInstance->SamplingFrequency != TempSamplingFrequency)							// Next times, test is done
			{
				ERROR("The sampling frequency requested is different from the sampling frequency used in other inputs\n");
				ERROR(" Please check that all sampling frequencies are the same.\n");
				return -1;
			}
		}
	}
	else
	{
		if (pDspInstance->SamplingFrequency == -1)
		{
			WARNING("The sampling frequency has not been set yet. 44100Hz will be used as default.\n");
			TempSamplingFrequency = 44100;
		}
		else TempSamplingFrequency = pDspInstance->SamplingFrequency;
	}
	pDspInstance->SamplingFrequency = TempSamplingFrequency;

	// Get then the number of channels (in must be equal to out!!!)
	GetNextWord(TempStr, Options);
	if (strcmp(TempStr, "") == 0) goto CmdStartStream_StartStreaming;
	TempInt = atoi(TempStr);
	if (TempInt < 1)
	{
		ERROR("Desired number of channel smaller than 1 not allowed.\n Please enter valid number of channels\n");
		return -1;
	}
	else if (TempInt > 8)
	{
		ERROR("Desired number of channel greater than 8 not allowed.\n Please enter valid number of channels\n");
		return -1;
	}
	else pDspInstance->StreamingParams.NumberOfChannels = TempInt;
	
	// Get the IOs memory locations
	//  getting first the input locations
	for (index = 0; index < pDspInstance->StreamingParams.NumberOfChannels; index++)
	{
		// Get the register or memory identifier to display
		GetNextWord(TempStr, Options);

		NumberOfItemsToDisplay = 0;				// by default, there's only one item to display
		GetRegMemSpecifier(TempStr, &RegMemSpecifier);
		if ((RegMemSpecifier > 0) && (RegMemSpecifier < REGMEM_SPECIFIER_MAX))
		{
			GetAddressStringsFromSpecifier(RegMemSpecifier, IdentifierString, &ItemToDisplay, &LastAllowedItem);
			if ((RegMemSpecifier > REGMEM_SPECIFIER_LIMIT) && (RegMemSpecifier < REGMEM_SPECIFIER_MAX))
			{
				// Get the "Address" start given as offset
				GetAddressAddressNumbers(Options, &FirstItemToDisplay, &NumberOfItemsToDisplay);
				ItemToDisplay = ItemToDisplay + FirstItemToDisplay;
			}
		}
		else if (RegMemSpecifier == 0)
		{
			ItemToDisplay = pDspInstance->State.In;
			// Get the "Address" start given as offset
			GetAddressAddressNumbers(TempStr, &FirstItemToDisplay, &NumberOfItemsToDisplay);
			ItemToDisplay = ItemToDisplay + FirstItemToDisplay;
		}
		else 
		{
			ERROR_1("Identifier not known: %s! Aborting command.\n", TempStr);
			DBG_PRINT("Out CmdStartStream\n");
			return -1;
		}
		pDspInstance->StreamingParams.Input[index] = ItemToDisplay;
	}
	//  getting now the output locations
	for (index = 0; index < pDspInstance->StreamingParams.NumberOfChannels; index++)
	{
		// Get the register or memory identifier to display
		GetNextWord(TempStr, Options);

		NumberOfItemsToDisplay = 0;				// by default, there's only one item to display
		GetRegMemSpecifier(TempStr, &RegMemSpecifier);
		if ((RegMemSpecifier > 0) && (RegMemSpecifier < REGMEM_SPECIFIER_MAX))
		{
			GetAddressStringsFromSpecifier(RegMemSpecifier, IdentifierString, &ItemToDisplay, &LastAllowedItem);
			if ((RegMemSpecifier > REGMEM_SPECIFIER_LIMIT) && (RegMemSpecifier < REGMEM_SPECIFIER_MAX))
			{
				// Get the "Address" start given as offset
				GetAddressAddressNumbers(Options, &FirstItemToDisplay, &NumberOfItemsToDisplay);
				ItemToDisplay = ItemToDisplay + FirstItemToDisplay;
			}
		}
		else if (RegMemSpecifier == 0)
		{
			ItemToDisplay = pDspInstance->State.Out;
			// Get the "Address" start given as offset
			GetAddressAddressNumbers(TempStr, &FirstItemToDisplay, &NumberOfItemsToDisplay);
			ItemToDisplay = ItemToDisplay + FirstItemToDisplay;
		}
		else 
		{
			ERROR_1("Identifier not known: %s! Aborting command.\n", TempStr);
			DBG_PRINT("Out CmdStartStream\n");
			return -1;
		}
		pDspInstance->StreamingParams.Output[index] = ItemToDisplay;
	}

CmdStartStream_StartStreaming:
	// Launch the simulation
	AL3101_StartStream(pDspInstance);

	DBG_PRINT("Out CmdStartStream\n");
	
	return 0;
};

/************************************************************/
/* NAME       :												*/
/* DESCRIPTION:												*/
/* INPUT      :												*/
/* OUTPUT     :												*/
/************************************************************/
INT32 CmdStopStream(CHAR* Options)
{
	DBG_PRINT("In CmdStopStream\n");

	AL3101_StopStream(pDspInstance);

	DBG_PRINT("Out CmdStopStream\n");
	
	return 0;
};


/********************************************************************/
/*               U T I L I T Y    F U N C T I O N S                 */
/********************************************************************/
/************************************************************/
/* NAME       : Display32BitsBuffer							*/
/* DESCRIPTION:												*/
/* INPUT      :												*/
/* OUTPUT     :												*/
/************************************************************/
INT32 Display32BitsBuffer(INT32* pBuff, INT32 nSize)
{
	CHAR StrVal[9];
	INT32 ii; 
	
	DBG_PRINT("In Display32BitsBuffer\n");

	for ( ii = 0; ii < nSize; ii++ )
	{
		OUTPUT_1("%04d: ",ii);
		Val2Str_Hex(*pBuff++, 32, StrVal);
		OUTPUT_1("%s", StrVal);
		OUTPUT("\n");
	}
	
	DBG_PRINT("Out Display32BitsBuffer\n");

	return 0;
}

/************************************************************/
/* NAME       : Display24BitsBuffer							*/
/* DESCRIPTION:												*/
/* INPUT      :												*/
/* OUTPUT     :												*/
/************************************************************/
INT32 Display24BitsBuffer(INT32* pBuff, INT32 nSize, INT32 nAlign)
{
	CHAR StrVal[7];
	INT32 ii, jj;

	DBG_PRINT("In Display24BitsBuffer\n");
	if (( nAlign != 2) && ( nAlign != 4))
	{
		ERR_PRINT_L1("Cannot align on <%d> Word, Default Value taken\n", nAlign);
		nAlign = 2;
	}
	
	for ( ii = 0; ii < (nSize/nAlign); ii++ )
	{
		OUTPUT_1("%04d: ", ii * nAlign);
		for ( jj = 0 ; jj < nAlign ; jj++ )
		{
			Val2Str_Hex(*pBuff++, 24, StrVal);
			OUTPUT_1("%s ", StrVal);
		}
		OUTPUT("\n");
	}
	
	DBG_PRINT("Out Display24BitsBuffer\n");
	
	return 0;
}

/************************************************************/
/* NAME       : Val2Str_Hex									*/
/* DESCRIPTION:												*/
/* INPUT      :												*/
/* OUTPUT     :												*/
/************************************************************/
INT32 Val2Str_Hex(INT32 val, INT32 nBits, CHAR* Str)
{
	INT32 jj;
	CHAR* pTemp;

	DBG_PRINT("In Val2Str_Hex\n");
	
	pTemp = Str + (nBits/4);
	*pTemp-- = '\0';
	for ( jj = 0; jj < (nBits>>2); jj++ ) *pTemp-- = Val2Char_Hex((val >> 4 * jj) & 0xF);
	
	DBG_PRINT("Out Val2Str_Hex\n");

	return 0;
}

/************************************************************/
/* NAME       : Val2Char_Hex								*/
/* DESCRIPTION:												*/
/* INPUT      :												*/
/* OUTPUT     :												*/
/************************************************************/
CHAR Val2Char_Hex(INT32 val)
{
	CHAR buf;

	DBG_PRINT("In Val2Char_Hex\n");

	if ((val >= 0) && (val < 10)) {buf = '0' + (CHAR)val;}
	else if ((val >= 10) && (val < 16)) {buf = 'A' + (CHAR)val - 10;}
	else buf = 'X';

	DBG_PRINT("Out Val2Char_Hex\n");

	return(buf);
}

/************************************************************/
/* FUNCTION NAME : GetCommandID                				*/
/* DESCRIPTION   :											*/
/* INPUT         :											*/
/* OUTPUT        :											*/
/************************************************************/
INT32 GetCommandID(CHAR *CmdStr, INT32 *CmdID)
{
	INT32 index;
	
	DBG_PRINT("In GetCommandID\n");
	
	// Try to find if there's an alias
	INT32 AliasCount;
	INT32 AliasIndex = 1;
	AL3101_CommandAlias		 *Alias;
	CommandAlias_List::Node  *Node;

	// Display the aliases
	AliasCount = pDspInstance->Config.Alias.GetCount();
	for (index = 0; index < AliasCount; index++)
	{
		Node = pDspInstance->Config.Alias.Item(index);
		Alias = Node->GetData();
		if (strcmp(Alias->AliasString, CmdStr) == 0)
		{
			index = Alias->CommandID;
			goto GetCommandID_EndFunction;
		}
	}
	
	index = 0;	// This was not an alias, let's try a real command
	while ((index != NumOfCommandsInList) && (strcmp(AL3101SimCommandList[index].name, CmdStr))) index++;
	if (index == NumOfCommandsInList) *CmdID = CMD_NOP;
	else *CmdID = AL3101SimCommandList[index].CmdID;

GetCommandID_EndFunction:		
	DBG_PRINT("Out GetCommandID\n");
	
	return index;
}

/************************************************************/
/* FUNCTION NAME : GetNextFilenameOption           			*/
/* DESCRIPTION   : get the option knowing it's a file name  */
/* INPUT         :											*/
/* OUTPUT        :											*/
/************************************************************/
INT32 GetNextFilenameWord(CHAR* dst, CHAR* src)
{
	CHAR* temp1;
	
	DBG_PRINT("In GetNextFilenameWord\n");
	
	temp1 = src;
	
	while (isspace(*temp1)) temp1++;
	if (*temp1 == '"') // that's a filename with spaces
	{
		*temp1++; // let's first skip the '"'
		while (((*temp1) != '"') && ((*temp1) != '\0'))
		{
			*dst++ = *temp1++;
		}
		temp1++;
	}
	else	// let's do it the normal style
	{
		while ((!isspace(*temp1)) && (*temp1) != '\0') *dst++ = *temp1++;
	}
	*dst = '\0';
	
	strcpy(src, temp1);

	DBG_PRINT("Out GetNextFilenameWord\n");
	
	return strlen(dst);
}

/************************************************************/
/* FUNCTION NAME : GetNextOption                  			*/
/* DESCRIPTION   :											*/
/* INPUT         :											*/
/* OUTPUT        :											*/
/************************************************************/
INT32 GetNextWord(CHAR* dst, CHAR* src)
{
	CHAR* temp1;
	
	DBG_PRINT("In GetNextWord\n");
	
	temp1 = src;
	
	while (isspace(*temp1)) temp1++;
	while ((!isspace(*temp1)) && (*temp1) != '\0') *dst++ = *temp1++;
	*dst = '\0';
	
	strcpy(src, temp1);

	DBG_PRINT("Out GetNextWord\n");
	
	return strlen(dst);
}



/********************************************************************/
/*              H E L P E R     F U N C T I O N S                   */
/********************************************************************/

INT32 LoadMemoryMap(CHAR *FileName)
{
	FILE *flux;
	CHAR TempStr[255];
	INT32 Address, Value;

	flux = fopen(FileName, "rt");
	if (flux == NULL) 
	{
		OUTPUT("Cannot find a memory map file, initialization done by program itself\n");
		return -1;
	}
	else OUTPUT_1("Loading memory map file: %s\n", FileName);

	// read out the information from the file
	while(1)
	{
		// first read the information
		if (fscanf(flux, "%s", TempStr) != 1) break;
		if (fscanf(flux, "%s", TempStr) != 1) break;
		if (fscanf(flux, "%d", &Address) != 1) break;
		if (fscanf(flux, "%d\n", &Value) != 1) break;
		// and use the values
		pDspInstance->State.Data[Address] = Value;
	}

	fclose(flux);

	OUTPUT("Memory map file successfully loaded\n");

	return 0;
}

/************************************************************/
/* NAME       :	GetAddressNumbers							*/
/* DESCRIPTION:	Helper function for CmdShow function		*/
/*					helps in getting an address number		*/
/*					from the option string					*/
/* INPUT      :												*/
/* OUTPUT     :												*/
/************************************************************/
INT32 GetAddressAddressNumbers(CHAR *Options, INT32 *FirstAddress, INT32 *Size)
{
	CHAR		LocalOptions[CMD_MAX_LEN];
	CHAR		AddressStr[CMD_MAX_LEN];

	strcpy(LocalOptions, Options);		// Work on a local copy of the pointer

	// Get the supposed first address number
	GetNextWord(AddressStr, LocalOptions);
	if (strcmp(AddressStr, "0") == 0)
	{
		*FirstAddress = 0;
	}
	else
	{
		*FirstAddress = atoi(AddressStr);
		if (*FirstAddress == 0)
		{
			*Size = 1;
			return -1;
		}
	}
	GetNextWord(AddressStr, Options);

	// Get the supposed second address number
	GetNextWord(AddressStr, LocalOptions);
	if (strcmp(AddressStr, "0") == 0)
	{
		*Size = 0;
	}
	else
	{
		*Size = atoi(AddressStr);
		if (*Size == 0)
		{
			*Size = 1;
			return -2;
		}
	}
	*Size = *Size - *FirstAddress + 1;
	GetNextWord(AddressStr, Options);

	return 0;
}


/************************************************************/
/* NAME       :	GetAddressNumber							*/
/* DESCRIPTION:	Helper function for CmdSetVal function		*/
/*					helps in getting an address number		*/
/*					from the option string					*/
/* INPUT      :												*/
/* OUTPUT     :												*/
/************************************************************/
INT32 GetAddressAddressNumber(CHAR *Options, INT32 *FirstAddress)
{
	CHAR		LocalOptions[CMD_MAX_LEN];
	CHAR		AddressStr[CMD_MAX_LEN];

	strcpy(LocalOptions, Options);		// Work on a local copy of the pointer

	// Get the supposed first address number
	GetNextWord(AddressStr, LocalOptions);
	if (strcmp(AddressStr, "0") == 0)
	{
		*FirstAddress = 0;
	}
	else
	{
		*FirstAddress = atoi(AddressStr);
		if (*FirstAddress == 0)
		{
			return -1;
		}
	}
	GetNextWord(AddressStr, Options);

	return 0;
}


/************************************************************/
/* NAME       :	DisplayWatchValues							*/
/* DESCRIPTION:	Displays the variables according to the		*/
/*					watches list							*/
/* INPUT      :												*/
/* OUTPUT     :												*/
/************************************************************/
void DisplayWatchValues()
{
	CHAR TempStr[CMD_MAX_LEN];
	INT32 WatchCount, WatchIndex;
	INT32 index;
	AL3101_RegisterWatch	 *RegWatch;
	RegisterWatch_List::Node *RegNode;
	AL3101_MemoryWatch		 *MemWatch;
	MemoryWatch_List::Node	 *MemNode;

	DBG_PRINT("In DisplayWatchValues\n");

	OUTPUT("\r        \n");

	pDspInstance->Watch.WatchProcessingInProgress.Enter();
	// Display the register breakpoints
	WatchCount = pDspInstance->Watch.RegisterWatches.GetCount();
	for (index = 0; index < WatchCount; index++)
	{
		RegNode = pDspInstance->Watch.RegisterWatches.Item(index);
		RegWatch = RegNode->GetData();

		InternalValue_To_String(TempStr, *(RegWatch->Address), RegWatch->DisplayFormat);
		if (*(RegWatch->Address) == *(RegWatch->CopyAddress)) OUTPUT_2("%s = %s\n", RegWatch->DisplayString, TempStr);
		else OUTPUT_2("%s = %s *\n", RegWatch->DisplayString, TempStr);
	}
	// Display the memory breakpoints
	WatchCount = pDspInstance->Watch.MemoryWatches.GetCount();
	for (WatchIndex = 0; WatchIndex < WatchCount; WatchIndex++)
	{
		MemNode = pDspInstance->Watch.MemoryWatches.Item(WatchIndex);
		MemWatch = MemNode->GetData();
		
		INT32 *ItemToDisplay = MemWatch->Address;
		INT32 *CopyItemToDisplay = MemWatch->CopyAddress;
		INT32 FirstItemToDisplay = MemWatch->BeginIndex;
		for (index = 0; index < MemWatch->NumberOfElements; index++)
		{
			InternalValue_To_String(TempStr, *ItemToDisplay, MemWatch->DisplayFormat);
			if (*ItemToDisplay == *CopyItemToDisplay) OUTPUT_4("%s%i%s = %s\n", MemWatch->PreDisplayString, FirstItemToDisplay, MemWatch->PostDisplayString, TempStr);
			else  OUTPUT_4("%s%i%s = %s *\n", MemWatch->PreDisplayString, FirstItemToDisplay, MemWatch->PostDisplayString, TempStr);
			ItemToDisplay++;
			CopyItemToDisplay++;
			FirstItemToDisplay++;
		}

	}
	pDspInstance->Watch.WatchProcessingInProgress.Leave();

	OUTPUT("\n");

	DBG_PRINT("Out DisplayWatchValues\n");
}

/************************************************************/
/* NAME       :	GetResetType								*/
/* DESCRIPTION:	Helper function for CmdReset function		*/
/*					and the reset after simulation			*/
/*					helps in getting the type of reset to do*/
/* INPUT      :												*/
/* OUTPUT     :												*/
/************************************************************/
INT32 GetResetType(CHAR *Options)
{
	CHAR TempStr[CMD_MAX_LEN];
	INT32 ResetType;

	// Check first what should be reset
	// Get the reset type
	GetNextWord(TempStr, Options);
	String_ToLower_String(TempStr, TempStr);

	// Check if no reset should be done
	if (strcmp(TempStr, "none") == 0) ResetType = -1;
	// Check if the IOs should be reset
	else if (strcmp(TempStr, "io") == 0) ResetType = 0;
	// Check if the IOs should be reset and reinitialized again.
	else if (strcmp(TempStr, "io_reopen") == 0) ResetType = 1;
	// Check if the program memory and breakpoints should be reset.
	else if (strcmp(TempStr, "prog") == 0) ResetType = 2;
	// Check if the program memory only should be reset.
	else if (strcmp(TempStr, "prog_only") == 0) ResetType = 3;
	// Check if the data memory and watches should be reset.
	else if (strcmp(TempStr, "data") == 0) ResetType = 4;
	// Check if the data memory only should be reset.
	else if (strcmp(TempStr, "data_only") == 0) ResetType = 5;
	// Check if a hardware reset should take place
	//		Meaning Program and data memory.
	else if (strcmp(TempStr, "dsp") == 0) ResetType = 6;
	// Check if everything should be reset and IOs should be reinitialized
	else if (strcmp(TempStr, "all_reopen") == 0) ResetType = 7;
	// Check if everything should be reset (program, data memories and IOs).
	//if (strcmp(TempStr, "all") == 0)		// this is the default case 
	else ResetType = 8;

	return ResetType;
}

/************************************************************/
/* NAME       :	ApplyResetFromType							*/
/* DESCRIPTION:	Helper function for CmdReset function		*/
/*					and the reset after simulation			*/
/*					helps in making reset from type			*/
/* INPUT      :												*/
/* OUTPUT     :												*/
/************************************************************/
INT32 ApplyResetFromType(INT32 ResetType)
{
	switch (ResetType)
	{
	case -1:
		// do nothing and skip the reset instance
		return 0;
	case 0:
		// Check if the IOs should be reset
		AL3101_Reset_IO(pDspInstance, 0);
		break;
	case 1:
		// Check if the IOs should be reset and reinitialized again.
		AL3101_Reset_IO(pDspInstance, 1);
		break;
	case 2:
		// Check if the program memory and breakpoints should be reset.
		AL3101_Reset_Prog(pDspInstance);
		AL3101_Reset_Breakpoints(pDspInstance);
		break;
	case 3:
		// Check if the program memory only should be reset.
		AL3101_Reset_Prog(pDspInstance);
		break;
	case 4:
		// Check if the data memory and watches should be reset.
		AL3101_Reset_Data(pDspInstance);
		AL3101_Reset_Watches(pDspInstance);
		break;
	case 5:
		// Check if the data memory only should be reset.
		AL3101_Reset_Data(pDspInstance);
		break;
	case 6:
		// Check if a hardware reset should take place
		//		Meaning Program and data memory.
		AL3101_Reset_Prog(pDspInstance);
		AL3101_Reset_Data(pDspInstance);
		break;
	case 7:
		// Check if everything should be reset and IOs should be reinitialized
		AL3101_Reset_IO(pDspInstance, 1);
		AL3101_Reset_Prog(pDspInstance);
		AL3101_Reset_Data(pDspInstance);
		AL3101_Reset_Breakpoints(pDspInstance);
		AL3101_Reset_Watches(pDspInstance);
		break;
	default:
		// Check if everything should be reset (program, data memories and IOs).
		AL3101_Reset_IO(pDspInstance, 0);
		AL3101_Reset_Prog(pDspInstance);
		AL3101_Reset_Data(pDspInstance);
		AL3101_Reset_Breakpoints(pDspInstance);
		AL3101_Reset_Watches(pDspInstance);
		break;
	}

	AL3101_Reset_State(pDspInstance);

	return 0;
}
