/********************************************************************/
/*                                                                  */
/*                        A X O R I S                               */
/*                                                                  */
/*                Alesis AL3101/2 DSP Simulator                     */
/*                                                                  */
/********************************************************************/
/*                                                                  */
/* Origin Date    : June 2001                                       */
/* Project Version: v0.1                                            */
/* FileName       : Commands.h                                      */
/* File Version   : v0.1                                            */
/* Author         : Fabian Duchesne                                 */
/* Last Modif     : September 2001                                  */
/*                                                                  */
/********************************************************************/

#if !defined __1K_COMMANDS_INCLUDE
#define __1K_COMMANDS_INCLUDE

// 1K simulator types
#include "./../../common/1K_types.h"

/********************************************************************/
/*          C O M M A N D S   C O D E S    D E F I N E S            */
/********************************************************************/

/**** G E N E R A L   C O M M A N D S ****/
#define CMD_QUIT					0
#define CMD_HELP					1
#define CMD_VERSION					2
#define CMD_COPYRIGHT				3
#define CMD_NOP						4

/**** S I M U L A T O R   C O M M A N D S ****/
#define CMD_START					101				
#define CMD_STOP					102
#define CMD_PAUSE					103
#define CMD_RESET					104
#define CMD_LOAD					105
#define CMD_LOADSCRIPT				106
#define CMD_STEP					107

#define CMD_SET_INPUT				110
#define CMD_DEL_INPUT				111
#define CMD_DEL_ALLINPUT			112
#define CMD_SHOW_INPUT				113
#define CMD_LOOP_INPUT				114
#define CMD_SET_OUTPUT				115
#define CMD_DEL_OUTPUT				116
#define CMD_DEL_ALLOUTPUT			117
#define CMD_SHOW_OUTPUT				118
#define CMD_SET_SIGNAL				120
#define CMD_DEL_SIGNAL				121
#define CMD_DEL_ALLSIGNAL			122
#define CMD_SHOW_SIGNAL				123
#define CMD_SET_TRACE				125
#define CMD_DEL_TRACE				126
#define CMD_DEL_ALLTRACE			127
#define CMD_SHOW_TRACE				128
#define CMD_SET_TOOL				130
#define CMD_DEL_TOOL				131
#define CMD_DEL_ALLTOOL				132
#define CMD_SHOW_TOOL				133

#define CMD_SET_STREAM				150
#define CMD_START_STREAM			151
#define CMD_STOP_STREAM				152


/**** B R E A K P O I N T S   C O M M A N D S ****/
#define CMD_SET_INSTRBREAKPOINT		200
#define CMD_DEL_INSTRBREAKPOINT		201
#define CMD_SET_TEMP_BREAKPOINT		202
#define CMD_SET_VALBREAKPOINT		204
#define CMD_DEL_VALBREAKPOINT		205
#define CMD_DEL_ALLBREAKPOINTS		206

/**** W A T C H S   C O M M A N D S ****/
#define CMD_SHOW					250
#define CMD_SET_WATCH				251
#define CMD_DEL_WATCH				252
#define CMD_DEL_ALLWATCHES			253
#define CMD_DISP_WATCHES			254

/**** S E T    C O M M A N D S ****/
#define CMD_SET_VAL					300
#define CMD_SET_USERPARAMETER		301

/**** D I S P L A Y   C O M M A N D S ****/
#define CMD_DISP_CYCLES				400
#define CMD_DISP_DSP_STATE			401
#define CMD_DISP_BREAKPOINTS		402
#define CMD_DISP_REGISTERS			403
#define CMD_DISP_PROG_MEMORY		404
#define CMD_DISP_DATA_MEMORY		405

/**** D I S A S M   C O M M A N D S ****/
#define CMD_DASM_N					500
#define CMD_DASM_ON					501
#define CMD_DASM_OFF				502

/**** A S M  C O M M A N D S ****/
#define CMD_ASM						600

/**** A L I A S    C O M M A N D S ****/
#define CMD_SET_ALIAS				700
#define CMD_DEL_ALIAS				701
#define CMD_DEL_ALLALIASES			702
#define CMD_DISP_ALIASES			703

#define CMD_NO_CMD				   1000

/*****************************************/
/* Error defines                         */
/*****************************************/
// define the base error for CMD
#define	CMD_GEN_BASE_ERR					(CMD_BASE_ERR + 0x00010000)
#define CMD_FILE_BASE_ERR					(CMD_BASE_ERR + 0x00020000)
#define CMD_TOOL_BASE_ERR					(CMD_BASE_ERR + 0x00030000)
// define the errors for the CMD/FILE
#define CMD_FILE_NOERROR					(CMD_FILE_BASE_ERR +  0)
#define CMD_FILE_CANNOT_RETRIEVE_EXTENSION	(CMD_FILE_BASE_ERR +  1)
#define CMD_FILE_UNKNOWN_EXTENSION			(CMD_FILE_BASE_ERR +  2)
#define CMD_FILE_WRONG_SAMPLING_FREQUENCY	(CMD_FILE_BASE_ERR +  3)
#define CMD_FILE_WRONG_ARGUMENT				(CMD_FILE_BASE_ERR +  4)
#define CMD_FILE_CANNOT_INITIALIZE_FILE		(CMD_FILE_BASE_ERR +  5)
#define CMD_FILE_CANNOT_OPEN_FILE			(CMD_FILE_BASE_ERR +  6)
// define the errors for the CMD/TOOLS
#define CMD_TOOL_NOERROR					(CMD_FILE_BASE_ERR +  0)
//#define CMD_TOOL_CANNOT_RETRIEVE_EXTENSION	(CMD_FILE_BASE_ERR +  1)
#define CMD_TOOL_UNKNOWN					(CMD_FILE_BASE_ERR +  2)
#define CMD_TOOL_WRONG_SAMPLING_FREQUENCY	(CMD_FILE_BASE_ERR +  3)
#define CMD_TOOL_WRONG_ARGUMENT				(CMD_FILE_BASE_ERR +  4)
#define CMD_TOOL_CANNOT_INITIALIZE_TOOL		(CMD_FILE_BASE_ERR +  5)
//#define CMD_TOOL_CANNOT_OPEN_FILE			(CMD_FILE_BASE_ERR +  6)


/********************************************************************/
/*                 C O M M A N D   S T R U C T U R E                */
/********************************************************************/
typedef struct _CommandFunctions_
{
    const CHAR* name;
    INT32 length;
	INT32 CmdID;
    INT32 (*function)(CHAR*);
    const CHAR* help;

} AL3101SimCommand;

/********************************************************************/
/*    C O M M A N D S    F U N C T I O N S    P R O T O T Y P E S   */
/********************************************************************/
INT32 CmdQuit(CHAR*);
INT32 CmdNop(CHAR*);
INT32 CmdHelp(CHAR*);
INT32 CmdVersion(CHAR*);
INT32 CmdCopyright(CHAR*);

INT32 CmdStart(CHAR*);
INT32 CmdStop(CHAR*);
INT32 CmdReset(CHAR*);
INT32 CmdLoad(CHAR*);
INT32 CmdLoadScript(CHAR*);
INT32 CmdStep(CHAR*);

INT32 CmdSetInput(CHAR*);
INT32 CmdDelInput(CHAR*);
INT32 CmdDelAllInput(CHAR*);
INT32 CmdShowInput(CHAR*);
INT32 CmdLoopInput(CHAR*);
INT32 CmdSetOutput(CHAR*);
INT32 CmdDelOutput(CHAR*);
INT32 CmdDelAllOutput(CHAR*);
INT32 CmdShowOutput(CHAR*);
INT32 CmdSetSignal(CHAR*);
INT32 CmdDelSignal(CHAR*);
INT32 CmdDelAllSignal(CHAR*);
INT32 CmdShowSignal(CHAR*);
INT32 CmdSetTrace(CHAR*);
INT32 CmdDelTrace(CHAR*);
INT32 CmdDelAllTrace(CHAR*);
INT32 CmdShowTrace(CHAR*);
INT32 CmdSetTool(CHAR*);
INT32 CmdDelTool(CHAR*);
INT32 CmdDelAllTool(CHAR*);
INT32 CmdShowTool(CHAR*);

INT32 CmdSetStream(CHAR*);
INT32 CmdStartStream(CHAR*);
INT32 CmdStopStream(CHAR*);


INT32 CmdSetInstrBreakPoint(CHAR*);
INT32 CmdDelInstrBreakPoint(CHAR*);
INT32 CmdSetTempBreakPoint(CHAR*);
INT32 CmdSetValBreakPoint(CHAR*);
INT32 CmdDelValBreakPoint(CHAR*);
INT32 CmdDelAllBreakPoints(CHAR*);
INT32 CmdDisplayBreakPoints(CHAR*);

INT32 CmdShow(CHAR*);
INT32 CmdSetWatch(CHAR*);
INT32 CmdDelWatch(CHAR*);
INT32 CmdDelAllWatches(CHAR*);
INT32 CmdDisplayWatches(CHAR*);

INT32 CmdSetVal(CHAR*);
INT32 CmdSetUserParameter(CHAR*);

INT32 CmdDisplayCycles(CHAR*);
INT32 CmdDisplayDSPState(CHAR*);
INT32 CmdDisplayRegisters(CHAR*);
INT32 CmdDisplayProgMemory(CHAR*);
INT32 CmdDisplayDataMemory(CHAR*);

INT32 CmdDasmN(CHAR*);
INT32 CmdDasmOn(CHAR*);
INT32 CmdDasmOff(CHAR*);

INT32 CmdSetAlias(CHAR*);
INT32 CmdDelAlias(CHAR*);
INT32 CmdDelAllAliases(CHAR*);
INT32 CmdDisplayAliases(CHAR*);

/********************************************************************/
/*     U T I L I T Y    F U N C T I O N S    P R O T O T Y P E S    */
/********************************************************************/
INT32 Display24BitsBuffer(INT32* pBuff, INT32 nSize, INT32 nAlign);
INT32 Display32BitsBuffer(INT32* pBuff, INT32 nSize);
INT32 Val2Str_Hex(INT32 val, INT32 nBits, CHAR* Str);
CHAR Val2Char_Hex(INT32);
INT32 GetCommandID(CHAR *CmdStr, INT32 *CmdID);
INT32 GetNextWord(CHAR*, CHAR*);
INT32 GetNextFilenameWord(CHAR* dst, CHAR* src);

#endif /* __1K_COMMANDS_INCLUDE */
