// Wave.h: interface for the CWaveFile class.
//
//////////////////////////////////////////////////////////////////////

#ifndef __AL3101__FILEIO_WAV__CWAVEFILE__H__
#define __AL3101__FILEIO_WAV__CWAVEFILE__H__

#include "./../../../common/1K_types.h"		// for the simulator types
#include <stdio.h>								// for the file types, ...

#pragma pack(1)

UINT32 FourCC(CHAR *ChunkName);

struct RiffChunkHeader
{
	UINT32	ckID;				// Four-character chunk ID
	UINT32	ckSize;				// Length of data in chunk
};


struct WaveFormat_ChunkData
{
	UINT16	wFormatTag;			// Format category (PCM=1)
	UINT16	nChannels;			// Number of channels (mono=1, stereo=2)
	UINT32	nSamplesPerSec;		// Sampling rate [Hz]
	UINT32	nAvgBytesPerSec;
	UINT16	nBlockAlign;
	UINT16	nBitsPerSample;

	void Config (UINT32 NewSamplingRate  = 44100,
				 UINT32 NewBitsPerSample =    16,
				 UINT32 NewNumChannels   =     2)
	{
		nSamplesPerSec	= NewSamplingRate;
		nChannels		= (UINT16)NewNumChannels;
		nBitsPerSample	= (UINT16)NewBitsPerSample;
		nAvgBytesPerSec	= (nChannels * nSamplesPerSec * nBitsPerSample) / 8;
		nBlockAlign		= (nChannels * nBitsPerSample) / 8;
	}

	WaveFormat_ChunkData()
	{
		wFormatTag = 1;     // PCM
		Config();
	}
};


struct WaveFormat_Chunk
{
	RiffChunkHeader         header;
	WaveFormat_ChunkData    data;

	WaveFormat_Chunk()
	{
		header.ckID		=   FourCC("fmt");
		header.ckSize	=   sizeof (WaveFormat_ChunkData);
	}

	INT32 VerifyValidity()
	{
		return header.ckID == FourCC("fmt") &&

			(data.nChannels == 1 || data.nChannels == 2) &&

			data.nAvgBytesPerSec == (data.nChannels * data.nSamplesPerSec *
										data.nBitsPerSample) / 8   &&

			data.nBlockAlign == (data.nChannels * data.nBitsPerSample) / 8;
   }
};


#define  MAX_WAVE_CHANNELS   2


struct WaveFileSample
{
	INT16	chan[MAX_WAVE_CHANNELS];
};




class CWaveFile
{
	INT32				Mode;				// current file I/O mode
	FILE				*file;				// I/O stream to use
	WaveFormat_Chunk	wave_format;
	RiffChunkHeader		riff_header;		// header for whole file
	RiffChunkHeader		pcm_data;
	INT32				pcm_data_offset;	// offset of 'pcm_data' in output file
	UINT32				num_samples;

	public:
		INT32 Looping;

		CWaveFile();
		~CWaveFile();

		INT32 OpenForWrite( CHAR   *Filename,
							UINT32 SamplingRate   = 44100,
							UINT32 BytesPerSample =     2,
							UINT32 NumChannels    =     2);

		INT32 OpenForRead(CHAR *Filename);

		INT32 SeekToSample(UINT32 SampleIndex);

		// The following work only with 16-bit audio
		INT32 WriteData(INT16 *data, UINT32 numData);
		INT32 ReadData (INT16 *data, UINT32 numData);


		INT32 Close();

		UINT32 GetSamplingRate();    // [Hz]
		UINT32 GetBytesPerSample();
		UINT32 GetNumChannels();
		UINT32 GetNumSamples();


		INT32 GetCurrentFilePosition() { return ftell(file); }

		INT32 GetCurrentFileMode() { return Mode; }

		// helper functions
		INT32 WriteFunction(void *Data, UINT32 NumBytes, RiffChunkHeader *header);
		INT32 ReadFunction (void *Data, UINT32 NumBytes);
};

#pragma pack()


/*--- end of file FileIO_WAV_Wave.h ---*/

#endif // !defined(__AL3101__FILEIO_WAVE__WAVE__H__)
