/********************************************************************/
/*                                                                  */
/*                        A X O R I S                               */
/*                                                                  */
/*                Alesis AL3101/2 DSP Simulator                     */
/*                                                                  */
/********************************************************************/
/*                                                                  */
/* Origin Date    : April 2002                                      */
/* Project Version: v0.1                                            */
/* FileName       : HelperFunctions.c                               */
/* File Version   : v0.1                                            */
/* Author         : JC Denton                                       */
/* Last Modif     : April 2002                                      */
/*                                                                  */
/********************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>		// for the tolower() function
#include <string.h>		// for the string functions: strlen, strcat

#include "./MessageMacros.h"
#include "./../common/1K_types.h"		// for the standard 1K types redefinition
#include "./Simul_func/AL3101_Core.h"	// for the 1K structures
#include "./HelperFunctions.h"			// for the defines and function prototypes

extern AL3101* pDspInstance;



void String_ToLower_String(CHAR *OutputString, CHAR *InputString)
{
	INT32 index;
	INT32 StringLength;

	StringLength = strlen(InputString);
	for (index = 0; index < StringLength; index++)
		OutputString[index] = tolower(InputString[index]);

	return;
}


void String_ToUpper_String(CHAR *OutputString, CHAR *InputString)
{
	INT32 index;
	INT32 StringLength;

	StringLength = strlen(InputString);
	for (index = 0; index < StringLength; index++)
		OutputString[index] = toupper(InputString[index]);

	return;
}


void BinaryValue_To_String(CHAR *OutputString, INT32 ValueToConvert, INT32 NumberOfRelevantBits, INT32 AddBinarySign) 
{
	INT32 index;
	INT32 TempInt;
	INT32 Mask;

	OutputString[0] = 0;		// use first location of the string
	Mask = 1 << (NumberOfRelevantBits - 1);
	for (index = 0; index < NumberOfRelevantBits; index++)
	{
		TempInt = ValueToConvert & Mask;
		if (TempInt == 0) strcat(OutputString, "0");
		else strcat(OutputString, "1");
		Mask = Mask >> 1;
	}

	if (AddBinarySign != 0) strcat(OutputString, "b");
}

void InternalValue_To_String(CHAR *OutputString, INT32 ValueToConvert, INT32 DisplayType, INT32 Precision)
{
	float		TempFloat;
	INT32		TempInt;

	switch(DisplayType)
	{
	case AL3101_DISPLAY_FLOAT:
		TempFloat = (float)(ValueToConvert);
		TempFloat = TempFloat / (float)(16777216);		// TempFloat / 2^24
		//_gcvt(TempFloat, Precision, OutputString);
		sprintf(OutputString, "%f", TempFloat);
		break;
	case AL3101_DISPLAY_DS324:
		TempInt = ValueToConvert;
		//itoa(TempInt, OutputString, 10);
		sprintf(OutputString, "%i", TempInt);
		break;
	case AL3101_DISPLAY_DS318:
		TempInt = ValueToConvert / 64;
		//itoa(TempInt, OutputString, 10);
		sprintf(OutputString, "%i", TempInt);
		break;
	case AL3101_DISPLAY_HS324:
		TempInt = ValueToConvert;
		TempInt = TempInt & 0x0FFFFFFF;
		//strcpy(OutputString, "0x");
		//itoa(TempInt, OutputString + 2, 16);
		sprintf(OutputString, "0x%7.7x", TempInt);
		break;
	case AL3101_DISPLAY_HS318:
		TempInt = ValueToConvert / 64;
		TempInt = TempInt & 0x003FFFFF;
		//strcpy(OutputString, "0x");
		//itoa(TempInt, OutputString + 2, 16);
		sprintf(OutputString, "0x%6.6x", TempInt);
		break;
	case AL3101_DISPLAY_BS324:
		TempInt = ValueToConvert;
		TempInt = TempInt & 0x0FFFFFFF;
		//itoa(TempInt, OutputString, 2);
		//strcat(OutputString, "b");
		BinaryValue_To_String(OutputString, TempInt, 28, 1);
		break;
	case AL3101_DISPLAY_BS318:
		TempInt = ValueToConvert / 64;
		TempInt = TempInt & 0x003FFFFF;
		//itoa(TempInt, OutputString, 2);
		//strcat(OutputString, "b");
		BinaryValue_To_String(OutputString, TempInt, 22, 1);
		break;
	default:
		BUG_REPORT("Type not known!\n");
	}
}



/************************************************************/
/* NAME       :	GetRegMemSpecifier							*/
/* DESCRIPTION:	Helper function for CmdShow function		*/
/*					helps in getting the register or memory	*/
/*					specifier from the option string		*/
/* INPUT      :												*/
/* OUTPUT     :												*/
/************************************************************/
void GetRegMemSpecifier(CHAR *TempStr, INT32 *RegMemSpecifier)
{
	if (strcmp(TempStr, "a") == 0) *RegMemSpecifier = 1;			// Wants to display the A register
	else if (strcmp(TempStr, "b") == 0)  *RegMemSpecifier = 2;		// Wants to display the B register
	else if (strcmp(TempStr, "u") == 0) *RegMemSpecifier = 3;		// Wants to display the U register
	else if (strcmp(TempStr, "v") == 0)  *RegMemSpecifier = 4;		// Wants to display the B register
	else if (strcmp(TempStr, "n") == 0) *RegMemSpecifier = 5;		// Wants to display the U register
	else if (strcmp(TempStr, "z") == 0) *RegMemSpecifier = 6;		// Wants to display the U register
	else if (strcmp(TempStr, "in") == 0) *RegMemSpecifier = 7;		// Wants to display input values
	else if (strcmp(TempStr, "out") == 0) *RegMemSpecifier = 8;		// Wants to display input values
	else if (strcmp(TempStr, "mem") == 0) *RegMemSpecifier = 9;		// Wants to display input values
	else if (strcmp(TempStr, "data") == 0) *RegMemSpecifier = 10;	// Wants to display input values
	else if (strcmp(TempStr, "reg") == 0) *RegMemSpecifier = 11;	// Wants to display input values
	else if (strcmp(TempStr, "scratch") == 0) *RegMemSpecifier = 12;// Wants to display input values
	else
	{
		if ((atoi(TempStr) == 0) && (strcmp(TempStr, "0") != 0))
			*RegMemSpecifier = -1;									// This is not a number but an unknown string
		else *RegMemSpecifier = 0;									// This is a number
	}
}

/************************************************************/
/* NAME       :	GetAddressStringFromSpecifier				*/
/* DESCRIPTION:	Helper function for CmdShow function		*/
/*					helps in getting the name and address	*/
/*					related to a given specifier			*/
/* INPUT      :												*/
/* OUTPUT     :												*/
/************************************************************/
INT32 GetAddressStringFromSpecifier(INT32 RegMemSpecifier, CHAR *IdentifierString, INT32 **ItemToDisplay)
{
	switch(RegMemSpecifier)
	{
	case 1:									// Wants to display the A register
		strcpy(IdentifierString, "A");
		*ItemToDisplay = &(pDspInstance->State.A);
		break;
	case 2:									// Wants to display the B register
		strcpy(IdentifierString, "B");
		*ItemToDisplay = &(pDspInstance->State.B);
		break;
	case 3:									// Wants to display the U register
		strcpy(IdentifierString, "U");
		*ItemToDisplay = &(pDspInstance->State.U);
		break;
	case 4:									// Wants to display the v flag
		strcpy(IdentifierString, "Flag v");
		*ItemToDisplay = &(pDspInstance->State.v);
		break;
	case 5:									// Wants to display the n flag
		strcpy(IdentifierString, "Flag n");
		*ItemToDisplay = &(pDspInstance->State.n);
		break;
	case 6:									// Wants to display the z flag
		strcpy(IdentifierString, "Flag z");
		*ItemToDisplay = &(pDspInstance->State.z);
		break;
	case 7:									// Wants to display input values
		strcpy(IdentifierString, "In");
		*ItemToDisplay = pDspInstance->State.In;
		break;
	case 8:									// Wants to display input values
		strcpy(IdentifierString, "Out");
		*ItemToDisplay = pDspInstance->State.Out;
		break;
	case 9:									// Wants to display input values
		strcpy(IdentifierString, "Mem");
		*ItemToDisplay = pDspInstance->State.Data;
		break;
	case 10:								// Wants to display input values
		strcpy(IdentifierString, "Data");
		*ItemToDisplay = pDspInstance->State.Data;
		break;
	case 11:								// Wants to display input values
		strcpy(IdentifierString, "Reg");
		*ItemToDisplay = pDspInstance->State.Reg;
		break;
	case 12:								// Wants to display input values
		strcpy(IdentifierString, "Scratch");
		*ItemToDisplay = pDspInstance->State.Reg;
		break;
	default:								// Display type is not known
		return -1;
	}

	return 0;
}

/************************************************************/
/* NAME       :	GetAddressStringsFromSpecifier				*/
/* DESCRIPTION:	Helper function for CmdShow function		*/
/*					helps in getting the name and address	*/
/*					related to a given specifier			*/
/* INPUT      :												*/
/* OUTPUT     :												*/
/************************************************************/
INT32 GetAddressStringsFromSpecifier(INT32 RegMemSpecifier, CHAR *IdentifierString, INT32 **ItemToDisplay, INT32 **LastAllowedItem)
{
	switch(RegMemSpecifier)
	{
	case 1:									// Wants to display the A register
		strcpy(IdentifierString, "A");
		*ItemToDisplay = &(pDspInstance->State.A);
		*LastAllowedItem = &(pDspInstance->State.A);
		break;
	case 2:									// Wants to display the B register
		strcpy(IdentifierString, "B");
		*ItemToDisplay = &(pDspInstance->State.B);
		*LastAllowedItem = &(pDspInstance->State.B);
		break;
	case 3:									// Wants to display the U register
		strcpy(IdentifierString, "U");
		*ItemToDisplay = &(pDspInstance->State.U);
		*LastAllowedItem = &(pDspInstance->State.U);
		break;
	case 4:									// Wants to display the v flag
		strcpy(IdentifierString, "Flag v");
		*ItemToDisplay = &(pDspInstance->State.v);
		*LastAllowedItem = &(pDspInstance->State.v);
		break;
	case 5:									// Wants to display the n flag
		strcpy(IdentifierString, "Flag n");
		*ItemToDisplay = &(pDspInstance->State.n);
		*LastAllowedItem = &(pDspInstance->State.n);
		break;
	case 6:									// Wants to display the z flag
		strcpy(IdentifierString, "Flag z");
		*ItemToDisplay = &(pDspInstance->State.z);
		*LastAllowedItem = &(pDspInstance->State.z);
		break;
	case 7:									// Wants to display input values
		strcpy(IdentifierString, "In");
		*ItemToDisplay = pDspInstance->State.In;
		*LastAllowedItem = pDspInstance->State.In + 8;
		break;
	case 8:									// Wants to display input values
		strcpy(IdentifierString, "Out");
		*ItemToDisplay = pDspInstance->State.Out;
		*LastAllowedItem = pDspInstance->State.Out + 8;
		break;
	case 9:									// Wants to display input values
		strcpy(IdentifierString, "Mem");
		*ItemToDisplay = pDspInstance->State.Data;
		*LastAllowedItem =  pDspInstance->State.Data + 1024;
		break;
	case 10:								// Wants to display input values
		strcpy(IdentifierString, "Data");
		*ItemToDisplay = pDspInstance->State.Data;
		*LastAllowedItem = pDspInstance->State.Data + 1024;
		break;
	case 11:								// Wants to display input values
		strcpy(IdentifierString, "Reg");
		*ItemToDisplay = pDspInstance->State.Reg;
		*LastAllowedItem = pDspInstance->State.Reg + 16;
		break;
	case 12:								// Wants to display input values
		strcpy(IdentifierString, "Scratch");
		*ItemToDisplay = pDspInstance->State.Reg;
		*LastAllowedItem = pDspInstance->State.Reg + 16;
		break;
	default:								// Display type is not known
		return -1;
	}

	return 0;
}
