/********************************************************************/
/*                                                                  */
/*                        A X O R I S                               */
/*                                                                  */
/*                Alesis AL3101/2 DSP Simulator                     */
/*                                                                  */
/********************************************************************/
/*                                                                  */
/* Origin Date    : June 2001                                       */
/* Project Version: v0.1                                            */
/* FileName       : AL3101_Core.h                                   */
/* File Version   : v0.1                                            */
/* Author         : Fabian Duchesne                                 */
/* Last Modif     : September 2001                                  */
/*                                                                  */
/********************************************************************/

#if !defined __1K_AL3101_CORE_INCLUDE
#define __1K_AL3101_CORE_INCLUDE

/********************************************************************/
/*       G L O B A L   I N C L U D E								*/
/********************************************************************/
//#include <windows.h>
#include <wx/list.h>
#include <wx/thread.h>

/********************************************************************/
/*       L O C A L   I N C L U D E									*/
/********************************************************************/
#include "./../../common/1K_types.h"
#include "./../1K_func/AL3101Struct.h"

/********************************************************************/
/*       C O N S T A N T S   D E F I N E S							*/
/********************************************************************/
#define PROG_MEMORY_SIZE	1024
#define DATA_MEMORY_SIZE	1024
#define PINS_NUMBER			  28
#define REGISTERS_NUMBER	  16
#define OUTPUTS_NUMBER         8
#define INPUTS_NUMBER          8

/********************************************************************/
/*       A L E S I S    D S P    P I N S    D E F I N E S           */
/********************************************************************/
#define _1KM_CE			 1
#define _1KM_IN3		 2
#define _1KM_IN2		 3
#define _1KM_IN1		 4
#define _1KM_IN0		 5
#define _1KM_AUTO		 6
#define _1KM_GND		 7
#define _1KM_SERIALM	 8
#define _1KM_WORDCLK	 9
#define _1KM_RST		10
#define _1KM_OUT0		11
#define _1KM_OUT1		12
#define _1KM_OUT2		13
#define _1KM_OUT3		14
#define _1KM_ADDR2		15
#define _1KM_ADDR1		16
#define _1KM_ADDR0		17
#define _1KM_D0			18
#define _1KM_D1			19
#define _1KM_D2			20
#define _1KM_BYPASS		21
#define _1KM_VDD		22
#define _1KM_D3			23
#define _1KM_D4			24
#define _1KM_D5			25
#define _1KM_D6			26
#define _1KM_D7			27
#define _1KM_RW			28

/********************************************************************/
/*     B R E A K P O I N T    D S P    S T R U C T U R E            */
/********************************************************************/
class AL3101_Value_Breakpoint
{
public:
	// breakpoint related variables
	INT32 *Address;				// Address where the value stands
	INT32 TestValue;			// Value to test
	// display related variables
	CHAR  DisplayString[120];	// String to display when breaking
};

WX_DECLARE_LIST(AL3101_Value_Breakpoint, ValBreakpoint_List);

typedef struct _AL3101_Breakpoint
{
	INT32					InstrBreakpoint[PROG_MEMORY_SIZE];	// Is there a breakpoint on that instruction? No, if 0. Yes if non 0.

	ValBreakpoint_List		ValBreakpoints;						// Breakpoints on value linked-list
	wxCriticalSection		BreakpointProcessingInProgress;		// A critical section to protect the use of the linked-list

} AL3101_Breakpoint;

/********************************************************************/
/*         W  A  T  C  H  E  S         S T R U C T U R E            */
/********************************************************************/
class AL3101_RegisterWatch
{
public:
	// watch related variables
	INT32 *Address;				// Address where to start the watching
	INT32 *CopyAddress;			// Address where to look at for value changes
	// display related variables
	INT32 DisplayFormat;		// Format to use when displaying value
	CHAR  DisplayString[120];	// String to display
};

WX_DECLARE_LIST(AL3101_RegisterWatch, RegisterWatch_List);

class AL3101_MemoryWatch
{
public:
	// watch related variables
	INT32 *Address;				// Address where to start the watching
	INT32 *CopyAddress;			// Address where to look at for values changes
	INT32 BeginIndex;			// Index in the memory bank of the 1st value to display
	INT32 NumberOfElements;		// Number of elements to display
	// display related variables
	INT32 DisplayFormat;		// Format to use when displaying value
	CHAR PreDisplayString[120]; // String to display before the index (like "Data[")
	CHAR PostDisplayString[120];// String to display after the index (like "]")
};

WX_DECLARE_LIST(AL3101_MemoryWatch, MemoryWatch_List);

typedef struct _AL3101_Watches
{
	RegisterWatch_List		RegisterWatches;			// Watches on registers linked-list
	MemoryWatch_List		MemoryWatches;				// Watches on memory linked-list

	AL3101_struct			StateCopy;					// Copy of the DSP state

	wxCriticalSection		WatchProcessingInProgress;	// A critical section to protect the use of the linked-list

} AL3101_Watches;

/********************************************************************/
/*         A L I A S E S               S T R U C T U R E            */
/********************************************************************/
class AL3101_CommandAlias
{
public:
	INT32 CommandID;
	CHAR  AliasString[CMD_MAX_LEN];
	CHAR  CommandString[CMD_MAX_LEN];
};

WX_DECLARE_LIST(AL3101_CommandAlias, CommandAlias_List);

/********************************************************************/
/*           C O N F I G    D S P    S T R U C T U R E              */
/********************************************************************/
typedef struct _AL3101_Config
{
	INT32 TrueCycleSimulation;
	INT32 DefaultResetType;
	INT32 ShowDisasmWhenStepping;
	INT32 DefaultDisplayTypeInShow;

	INT32 MaxNumberOfInputs;
	INT32 MaxNumberOfOutputs;
	INT32 MaxNumberOfSignals;
	INT32 MaxNumberOfTraces;
	INT32 MaxNumberOfTools;

	CommandAlias_List Alias;

} AL3101_Config;

/********************************************************************/
/*           S T R E A M I N G       S T R U C T U R E              */
/********************************************************************/
typedef struct _AL3101_Streaming
{
	// "External" stream parameters
	INT32 NumberOfChannels;
	INT32 *Input[8];
	INT32 *Output[8];

	// "Internal" stream parameters
	INT32 BlockSize;
	INT32 NumberOfBlocks;
	INT32 InputDeviceID;
	INT32 OutputDeviceID;

	INT32 DummyValue;			// Default memory location for reading/writing when streaming
								//  this is used to avoid unwanted memory accesses

} AL3101_Streaming;

/********************************************************************/
/*           A L 3 1 0 1    D S P    S T R U C T U R E              */
/********************************************************************/
typedef struct _AL3101_State
{
	// Program related variables
	AL3101_Disasm_struct	CurrentProgram[PROG_MEMORY_SIZE];	// Structure containing all information about the code		

	// DSP core related variables
	AL3101_struct			State;

	// Breakpoint related variables
	AL3101_Breakpoint		Break;

	// Watch related variable
	AL3101_Watches			Watch;

	// IO related variables
	INT32 SamplingFrequency;
	// Input instances
	IOTable *IO_Inputs;
	INT32 NumberOfInputInstances;
	// Output instances
	IOTable *IO_Outputs;
	INT32 NumberOfOutputInstances;
	// Signal instances
	IOTable *Signals;
	INT32 NumberOfSignalInstances;
	// Traces instances
	IOTable *Traces;
	INT32 NumberOfTraceInstances;
	// Tools instances
	ToolTable *Tools;
	INT32 NumberOfToolInstances;

	// Configuration structure
	AL3101_Config Config;

	// Streaming structure
	AL3101_Streaming StreamingParams;

} AL3101;


/********************************************************************/
/*      A L 3 1 0 1    D S P    C O R E    F U N C T I O N S        */
/********************************************************************/

// init, clean "process" functions
INT32 AL3101_Init(AL3101* pState);
INT32 AL3101_Clean(AL3101* pState);
INT32 AL3101_Start(AL3101* pState, INT32 nSteps, INT32 BlockingMode = 0);
INT32 AL3101_Stop(AL3101* pState);
INT32 AL3101_StartStream(AL3101* pState);
INT32 AL3101_StopStream(AL3101* pState);

// Reset functions
//INT32 AL3101_Reset(AL3101* pState);
void AL3101_Reset_State(AL3101 *pDspInstance);
INT32 AL3101_Reset_IO(AL3101 *pDspInstance, INT32 ReOpen);
void AL3101_Reset_Prog(AL3101 *pDspInstance);
void AL3101_Reset_Data(AL3101 *pDspInstance);
void AL3101_Reset_Breakpoints(AL3101 *pDspInstance);
void AL3101_Reset_Watches(AL3101 *pDspInstance);



INT32 _PrintDSPState();

INT32 AL3101_Step(AL3101* pDspInstance, INT32 NumberOfSteps);


#endif /* __1K_AL3101_CORE_INCLUDE */

