/***********************************************************************************************
 *
 * Functions	: CFSinGenerator v1.0
 *
 * Members Functions:
 *
 *		1. Initialization
 *			void SineGenerator_Init(SinGeneratorStruct *pInst, float ParFrequency, float ParAmplitude; float ParInitialPhase)			
 *
 *		2. Set Functions
 *			void SineGenerator_SetSamplingFrequency(SinGeneratorStruct *pInst, float SamplingFrequency)
 *			void SineGenerator_SetFrequency(SinGeneratorStruct *pInst, float Frequency)
 *			void SineGenerator_SetAmplitude(SinGeneratorStruct *pInst, float Amplitude)
 *			void SineGenerator_SetInitialPhase(SinGeneratorStruct *pInst, float InitialPhase)
 *		
 *		3.  Process Function
 *			void SineGenerator_Process(SinGeneratorStruct *pInst, float *pOut) 
 *          void SineGenerator_ProcessBlock(SinGeneratorStruct *pInst, float *pOut, INT32 BufLen)
 *
 ************************************************************************************************/

#ifndef __AXORIS__CFSINGENERATOR__H__
#define __AXORIS__CFSINGENERATOR__H__

#include "../../common/1K_types.h"		// for the simulator types

class CFSinGenerator
{
private:
	// user parameters
	float SamplingFrequency;
	float Frequency;
	float Amplitude;
	float InitialPhase;

	// state
	float lastsin;
	float lastcos;
	// DSP parameters
	float sinalpha;
	float cosalpha;

public:
// Construction
CFSinGenerator() {};

// Initialization
void Init(float SamplingFrequency, float Frequency, float Amplitude, float InitialPhase);
void Reset();

// Set Functions
void SetSamplingFrequency(float SamplingFrequency);
void SetFrequency(float Frequency);
void SetAmplitude(float Amplitude);
void SetInitialPhase(float InitialPhase, INT32 ResetCurrentPhase = 0);
		
// Process Function
void Process(float *pOut);
void ProcessBlock(float *pOut, INT32 BufLen);
};

#endif	// !defined(__AXORIS__CFSINGENERATOR__H__)
