// Tool_AmplitudeSweep_internal.h: internal definitions for the AmplitudeSweep tool functions.
//
/////////////////////////////////////////////////////////////////////////////////////////////////

#ifndef __AL3101__TOOL_AMPSWEEP__INTERNAL__H__
#define __AL3101__TOOL_AMPSWEEP__INTERNAL__H__

#include <stdio.h>							// for the file IO
#include "../../common/1K_types.h"	// for the simulator types
#include "CFSinGenerator.h"
//#include "FileIO_Generic.h"				// for the definition of the generic file IO stuff


typedef struct Tool_AmplitudeSweep_Struct_structtag
{
	// Parameters
	float InitialLevel;				// Initial amplitude level in dB
	float FinalLevel;				// Final amplitude level in dB
	float LevelIncrement;			// Level increment in dB

	INT32 TransientCount;			// Transient number of counts for output to be stable
	INT32 TotalCount;				// Total number of counts: measurement + transient
	INT32 OverallCount;				// Overall counter used to track position in simulation
	float SineFrequency;			// Frequency of operation of the sine generator
	float SamplingFrequency;		// Sampling frequency of the system

	// Internal variables
	INT32 CurrentCount;				// Current count value
	float CurrentLevel;				// Current level value in dB
	float CurrentLevelLinear;		// Current level value in linear scale
	INT32 InputAmplitudeMax;		// Max amplitude of the input
	INT32 OutputAmplitudeMax;		// Max amplitude of the output
	
	// Results variables
	Tool_AmplitudeSweep_Results *Results;	// Pointer to the results
	INT32 ResultSize;				// Giving the number of results that should be available
	INT32 ResultsIndex;				// Index of the current result
	INT32 ResultsAllocatedByTool;	// Tells if tool has allocated result itself: equal to 1. Equal to 0 otherwise.
	CHAR *AmplitudeFileName;		// Name of the amplitude result file
	CHAR *GainFileName;				// Name of the gain result file
	CHAR AmplitudeFileNameString[FILENAME_MAX];	// Storage for the Amplitude filename string
	CHAR GainFileNameString[FILENAME_MAX];		// Storage for the Gain filename string
	
	// Memory variables
	INT32 *InputMemory;				// Address of the input memory
	INT32 *OutputMemory;			// Address of the output memory

	// Sine generation
	CFSinGenerator SinGenerator;	// Class that generates the sine wave

} Tool_AmplitudeSweep_Struct;


#endif // !defined(__AL3101__TOOL_AMPSWEEP__INTERNAL__H__)
