// Tool_FrequencyResponse_internal.h: internal definitions for the FrequencyResponse tool functions.
//
/////////////////////////////////////////////////////////////////////////////////////////////////////

#ifndef __AL3101__TOOL_FREQRESP__INTERNAL__H__
#define __AL3101__TOOL_FREQRESP__INTERNAL__H__

#include <stdio.h>							// for the file IO
#include "../../common/1K_types.h"	// for the simulator types
#include "CFNoiseGenerator.h"	// for the noise generation
#include "FFTReal.h"			// for the real FFT
//#include "FileIO_Generic.h"				// for the definition of the generic file IO stuff


typedef struct Tool_FreqResp_Struct_structtag
{
	// Parameters
	float NoiseLevel;				// Level of the noise (in dB)
	float NoiseLevelLinear;			// Level of the noise (Linear)

	INT32 BlockSize;				// Size of a block (for FFT measurement)
	INT32 NSkipBlocks;				// Number of blocks to be skipped to avoid transients
	float SamplingFrequency;		// Sampling frequency of the system

	// Internal variables
	INT32 SkippingCounter;			// How many output samples should still be skipped
	INT32 ProcessingCounter;		// How many output samples should still be measured
	INT32 OverallCounter;			// Keeps track of the total number of samples done
	INT32 MeasuringMode;			// Tells what we should be currently doing from the write side
									//  if =  0 => not measuring yet
									//  if =  1 => measuring now
									//  if = -1 => should stop the simulation now
	float *InputBuffer;				// To keep input samples when measuring
	float *OutputBuffer;			// To keep output samples when measuring
	
	// Results variables
	Tool_FreqResp_Results *Results;	// Pointer to the results
	INT32 ResultsAllocatedByTool;	// Tells if tool has allocated result itself: equal to 1. Equal to 0 otherwise.
	CHAR *AmplitudeFileName;		// Name of the amplitude result file
	CHAR *PhaseFileName;			// Name of the phase result file
	CHAR AmplitudeFileNameString[FILENAME_MAX];	// Storage for the Amplitude filename string
	CHAR PhaseFileNameString[FILENAME_MAX];		// Storage for the Phase filename string
	
	// Memory variables
	INT32 *InputMemory;				// Address of the input memory
	INT32 *OutputMemory;			// Address of the output memory

	// Sine generation
	CFNoiseGenerator NoiseGenerator;			// Class that generates the white noise

} Tool_FreqResp_Struct;


// helper functions
INT32 Tool_FreqResp_GetResultSize(Tools_FreqResp_Parameters *Params, INT32 *ResultSize);
INT32 Tool_FreqResp_ComputeResults(void *pInstance);
INT32 Tool_FreqResp_WriteAmplitudeResults(Tool_FreqResp_Results *results, INT32 ResultSize, CHAR *AmplitudeFileName);
INT32 Tool_FreqResp_WritePhaseResults(Tool_FreqResp_Results *results, INT32 ResultSize, CHAR *PhaseFileName);


#endif // !defined(__AL3101__TOOL_FREQRESP__INTERNAL__H__)
